using System;
using System.Collections.Generic;
using System.Text;
using System.Drawing;
using System.Drawing.Drawing2D;
using Ascend;
using Ascend.Windows.Forms;

namespace Lokeiv.Games.Die.Forms
{
    class DieRender : GradientBackgroundRender
    {
        private Lokeiv.Games.Dice.Die _die;
        private float _dotSize = 4;
        private int _posOffset = 2;
        private SolidBrush _foreBrush;
        private Color _foreColor = Color.Yellow;

        /// <summary>
        /// Occurs when the ForeColor property changes.
        /// </summary>
        public event EventHandler ForeColorChanged;

        #region Public properties
        /// <summary>
        /// The size of the dots on the die.
        /// </summary>
        public float DotSize
        {
            get { return _dotSize; }
            set { _dotSize = value; }
        }
	
        /// <summary>
        /// The object implementing the die logic.
        /// </summary>
        public Lokeiv.Games.Dice.Die DieModel
        {
            get { return _die; }
            set { _die = value; }
        }

        /// <summary>
        /// The color assoicated with the text.
        /// </summary>
        public Color ForeColor
        {
            get
            {
                return this._foreColor;
            }

            set
            {
                if (value == this._foreColor)
                {
                    return;
                }

                this._foreColor = value;
                this.CreateForeBrush();
                this.OnForeColorChanged(new EventArgs());
            }
        }

        #endregion

        #region Contructors
        /// <summary>
        /// Initializes a new instance of the <see cref="DieRender"/> class.
        /// </summary>
        public DieRender(Lokeiv.Games.Dice.Die die)
            : base()
        {
            base.GradientHighColor = SystemColors.GradientActiveCaption;
            base.GradientLowColor = SystemColors.ActiveCaption;
            base.GradientMode = LinearGradientMode.Vertical;

            base.Border = new Border(1);

            this._die = die;
        }

        #endregion

        /// <summary>
        /// Raises the ForeColorChanged event. 
        /// </summary>
        /// <param name="e">An System.EventArgs that contains the event data.</param>
        protected virtual void OnForeColorChanged(System.EventArgs e)
        {
            if (this.ForeColorChanged != null)
            {
                this.ForeColorChanged(this, e);
            }
        }

        /// <summary>
        /// Draws the die control.
        /// </summary>
        /// <param name="e">A System.Windows.Forms.PaintEventArgs that contains the event data.</param>
        public override void Render(System.Windows.Forms.PaintEventArgs e)
        {
            if (e == null)
            {
                throw new ArgumentNullException("e");
            }

            RectangleF bounds;
            float topOffset = (this._posOffset + base.Padding.Top + this.Border.Top);
            float leftPosition = (this.DisplayRectangle.Left + base.Padding.Left + this._posOffset);
            float topPosition = (this.DisplayRectangle.Top + this._posOffset + base.Padding.Top + this.Border.Top);
            float rightOffset = (this._posOffset + base.Padding.Right + this.Border.Right);
            float bottomOffset = (this._posOffset + base.Padding.Bottom + this.Border.Bottom);
            bounds = new RectangleF(leftPosition, topPosition, (this.DisplayRectangle.Width - (leftPosition + rightOffset)), (this.DisplayRectangle.Height - (topOffset + bottomOffset)));

            if (this.DisplayRectangle.Height > 0 && this.DisplayRectangle.Width > 0)
            {
                base.Render(e);
                this.CreateForeBrush();

                if (this._die != null && this._die.Value != null && !bounds.IsEmpty) 
                {
                    if (this._die.GetType() == typeof(Lokeiv.Games.Dice.NumericOneToSixDie))
                    {
                        int number = (int)this._die.Value.Value;
                        this.RenderNumericOneToSix(e, number, bounds);
                    }
                }
            }
        }

        private void RenderNumericOneToSix(System.Windows.Forms.PaintEventArgs e, int number, RectangleF bounds)
        {
            if (number != 0 && this._dotSize != 0)
            {
                // Draw dots representing the dies value
                if (number == 1)
                {
                    e.Graphics.FillEllipse(this._foreBrush, this.GetCenterDotRectangle(bounds));
                }
                else if (number == 2)
                {
                    e.Graphics.FillEllipse(this._foreBrush, this.GetUpperLeftDotRectangle(bounds));
                    e.Graphics.FillEllipse(this._foreBrush, this.GetLowerRightDotRectangle(bounds));
                }
                else if (number == 3)
                {
                    e.Graphics.FillEllipse(this._foreBrush, this.GetUpperLeftDotRectangle(bounds));
                    e.Graphics.FillEllipse(this._foreBrush, this.GetCenterDotRectangle(bounds));
                    e.Graphics.FillEllipse(this._foreBrush, this.GetLowerRightDotRectangle(bounds));
                }
                else if (number == 4)
                {
                    e.Graphics.FillEllipse(this._foreBrush, this.GetUpperLeftDotRectangle(bounds));
                    e.Graphics.FillEllipse(this._foreBrush, this.GetUpperRightDotRectangle(bounds));
                    e.Graphics.FillEllipse(this._foreBrush, this.GetLowerLeftDotRectangle(bounds));
                    e.Graphics.FillEllipse(this._foreBrush, this.GetLowerRightDotRectangle(bounds));
                }
                else if (number == 5)
                {
                    e.Graphics.FillEllipse(this._foreBrush, this.GetUpperLeftDotRectangle(bounds));
                    e.Graphics.FillEllipse(this._foreBrush, this.GetUpperRightDotRectangle(bounds));
                    e.Graphics.FillEllipse(this._foreBrush, this.GetCenterDotRectangle(bounds));
                    e.Graphics.FillEllipse(this._foreBrush, this.GetLowerLeftDotRectangle(bounds));
                    e.Graphics.FillEllipse(this._foreBrush, this.GetLowerRightDotRectangle(bounds));
                }
                else if (number == 6)
                {
                    e.Graphics.FillEllipse(this._foreBrush, this.GetUpperLeftDotRectangle(bounds));
                    e.Graphics.FillEllipse(this._foreBrush, this.GetUpperRightDotRectangle(bounds));
                    e.Graphics.FillEllipse(this._foreBrush, this.GetMiddleLeftDotRectangle(bounds));
                    e.Graphics.FillEllipse(this._foreBrush, this.GetMiddleRightDotRectangle(bounds));
                    e.Graphics.FillEllipse(this._foreBrush, this.GetLowerLeftDotRectangle(bounds));
                    e.Graphics.FillEllipse(this._foreBrush, this.GetLowerRightDotRectangle(bounds));
                }
            }
        }

        private RectangleF GetCenterDotRectangle(RectangleF bounds)
        {
            return new RectangleF(bounds.Left + bounds.Width / 2 - this._dotSize / 2, bounds.Top + bounds.Height / 2 - this._dotSize / 2, this._dotSize, this._dotSize);
        }

        private RectangleF GetUpperLeftDotRectangle(RectangleF bounds)
        {
            return new RectangleF(bounds.Left + bounds.Width / 4 - this._dotSize / 2, bounds.Top + bounds.Height / 4 - this._dotSize / 2, this._dotSize, this._dotSize);

        }

        private RectangleF GetUpperRightDotRectangle(RectangleF bounds)
        {
            return new RectangleF(bounds.Left + 3 * bounds.Width / 4 - this._dotSize / 2, bounds.Top + bounds.Height / 4 - this._dotSize / 2, this._dotSize, this._dotSize);

        }

        private RectangleF GetMiddleLeftDotRectangle(RectangleF bounds)
        {
            return new RectangleF(bounds.Left + bounds.Width / 4 - this._dotSize / 2, bounds.Top + bounds.Height / 2 - this._dotSize / 2, this._dotSize, this._dotSize);

        }

        private RectangleF GetMiddleRightDotRectangle(RectangleF bounds)
        {
            return new RectangleF(bounds.Left + 3 * bounds.Width / 4 - this._dotSize / 2, bounds.Top + bounds.Height / 2 - this._dotSize / 2, this._dotSize, this._dotSize);

        }

        private RectangleF GetLowerLeftDotRectangle(RectangleF bounds)
        {
            return new RectangleF(bounds.Left + bounds.Width / 4 - this._dotSize / 2, bounds.Top + 3 * bounds.Height / 4 - this._dotSize / 2, this._dotSize, this._dotSize);

        }

        private RectangleF GetLowerRightDotRectangle(RectangleF bounds)
        {
            return new RectangleF(bounds.Left + 3 * bounds.Width / 4 - this._dotSize / 2, bounds.Top + 3 * bounds.Height / 4 - this._dotSize / 2, this._dotSize, this._dotSize);

        }

        /// <summary>
        /// Creates the solid brush needed to draw the foreground.
        /// </summary>
        protected virtual void CreateForeBrush()
        {
            if (base.Enabled)
            {
                this._foreBrush = new SolidBrush(this._foreColor);
            }
            else
            {
                this._foreBrush = new SolidBrush(SystemColors.GrayText);
            }
        }
    }
}
