using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Text;
using System.Windows.Forms;
using Lokeiv.Games.Ludo.Model;

namespace Lokeiv.Games.Ludo.Forms.Client
{
    public partial class Game : Form, IGame
    {

        #region Events
        /// <summary>
        /// Occurs when the game engine has been replaced by a new one.
        /// </summary>
        public event EventHandler NewGameEngineInitialized;

        /// <summary>
        /// Raises the <see cref="NewGameEngineInitialized"/> event.
        /// </summary>
        /// <param name="e"><see cref="EventArgs"/></param>
        protected void OnNewGameEngineInitialized(EventArgs e)
        {
            if (this.NewGameEngineInitialized != null)
                NewGameEngineInitialized(this, e);
        }
        #endregion

        #region Properties
        private NewGameForm _newGameForm = null;
        private Engine _engine = null;        
        private bool _isDesignMode = false;

        /// <summary>
        /// Gets the <see cref="Engine"/> that is used for the current game.
        /// </summary>
        public Engine Engine 
        {
            get
            {
                return _engine;
            }
        }

        /// <summary>
        /// Gets the running state of the game, since it can be paused.
        /// </summary>
        public bool IsRunning
        {
            get
            {
                return (this.Engine != null && this.Engine.IsRunning);
            }
        }

        /// <summary>
        /// Gets whether or not the game is set to debug mode.
        /// </summary>
        public bool IsDebugMode
        {
            get
            {
                return this.debugModeToolStripMenuItem.Checked;
            }
        }
        #endregion

        #region Constructors
        /// <summary>
        /// Constructor.
        /// </summary>
        public Game()
        {
            InitializeComponent();
            this._isDesignMode = (LicenseManager.UsageMode == LicenseUsageMode.Designtime);

            if (!this._isDesignMode)
            {
                this._newGameForm = new NewGameForm();
                this.redPlayerCaption.MouseDown += new MouseEventHandler(PlayerCaption_MouseDown);
                this.yellowPlayerCaption.MouseDown += new MouseEventHandler(PlayerCaption_MouseDown);
                this.greenPlayerCaption.MouseDown += new MouseEventHandler(PlayerCaption_MouseDown);
                this.bluePlayerCaption.MouseDown += new MouseEventHandler(PlayerCaption_MouseDown);
            }
        }
        #endregion

        #region Methods

        #region Other methods        
        /// <summary>
        /// Initializes a new game <see cref="Engine"/> and starts a new game.
        /// </summary>
        private void StartNewGame()
        {
            if (this._engine != null && this.IsRunning)
                this.EndGame();

            switch(this._newGameForm.ShowDialog(this))
            {
                case DialogResult.OK:                    
                    this.InitializeNewEngine(this._newGameForm.Players);  // Create a new engine and subscribe to its events
                    this.InitializePlayers();  // Update UI based on the players of the engine
                    this._engine.StartGame();
               
                    break;
                default:
                    break;
            }
        }

        /// <summary>
        /// Prepares a <see cref="Engine"/> for a new game.
        /// </summary>
        /// <param name="players"><see cref="PlayerCollection"/> that join the game.</param>
        private void InitializeNewEngine(PlayerCollection players)
        {            
            this.RemoveEngineEventHandlers(this._engine);
            this._engine = new Engine(players, null);
            this.AddEngineEventHandlers(this._engine);
            this.ConnectModelAndUi();
        }

        public void LoadRestoredEngine(Engine engine)
        {            
            this.RemoveEngineEventHandlers(this._engine);
            this._engine = engine;
            this.AddEngineEventHandlers(this._engine);
            this.ConnectModelAndUi();
        }

        private void RemoveEngineEventHandlers(Engine engine)
        {
            if (engine != null)
            {
                engine.GameStateChanged -= this.engine_GameStateChanged;
                engine.DieRolled -= this.engine_DieRolled;
                engine.PieceMoved -= this.engine_PieceMoved;
                engine.PlayerFinished -= this.engine_PlayerFinished;
                engine.GameOver -= this.engine_GameOver;
                engine.Die.DieValueChanged -= this.DieValueChanged;
            }
        }

        private void AddEngineEventHandlers(Engine engine)
        {
            if (engine != null)
            {
                engine.GameStateChanged += this.engine_GameStateChanged;
                engine.DieRolled += this.engine_DieRolled;
                engine.PieceMoved += this.engine_PieceMoved;
                engine.PlayerFinished += this.engine_PlayerFinished;
                engine.GameOver += this.engine_GameOver;
                engine.Die.DieValueChanged += this.DieValueChanged;
            }
        }

        private void ConnectModelAndUi()
        {
            // Connect the board model to the board UI and die model to die UI
            this.board1.SetGameModel(this._engine.Board, this);
            this.die1.DieModel = this._engine.Die;
            this.InitializePlayers();

            // Notify about the new game engine
            this.OnNewGameEngineInitialized(EventArgs.Empty);
        }
        
        /// <summary>
        /// Updates players in GUI.
        /// </summary>
        private void InitializePlayers()
        {            
            int defaultImageIndex = 9;
            this.redPlayerCaption.ImageIndex = defaultImageIndex;
            this.redPlayerCaption.Text = "Red player";
            this.yellowPlayerCaption.ImageIndex = defaultImageIndex;
            this.yellowPlayerCaption.Text = "Yellow player";
            this.greenPlayerCaption.ImageIndex = defaultImageIndex;
            this.greenPlayerCaption.Text = "Green player";
            this.bluePlayerCaption.ImageIndex = defaultImageIndex;
            this.bluePlayerCaption.Text = "Blue player";

            // Set the right image and caption
            foreach (Player player in this._engine.Players)
            {
                switch (player.Color)
                {
                    case Color.Red:
                        this.redPlayerCaption.Text = player.Name;

                        if (player.Type == PlayerType.Computer)
                            this.redPlayerCaption.ImageIndex = 5;
                        else
                            this.redPlayerCaption.ImageIndex = 0;
                        break;
                    case Color.Yellow:
                        this.yellowPlayerCaption.Text = player.Name;

                        if (player.Type == PlayerType.Computer)
                            this.yellowPlayerCaption.ImageIndex = 6;
                        else
                            this.yellowPlayerCaption.ImageIndex = 1;
                        break;
                    case Color.Green:
                        this.greenPlayerCaption.Text = player.Name;

                        if (player.Type == PlayerType.Computer)
                            this.greenPlayerCaption.ImageIndex = 7;
                        else
                            this.greenPlayerCaption.ImageIndex = 2;
                        break;
                    case Color.Blue:
                        this.bluePlayerCaption.Text = player.Name;

                        if (player.Type == PlayerType.Computer)
                            this.bluePlayerCaption.ImageIndex = 8;
                        else
                            this.bluePlayerCaption.ImageIndex = 3;
                        break;
                }
            }            
        }

        /// <summary>
        /// Toggles the enabled state of the control representing the <paramref name="player"/>, 
        /// to display whos turn it is.
        /// </summary>
        /// <param name="player"><see cref="Player"/> that has the turn.</param>
        private void UpdatePlayersTurn(Player player)
        {
            // Colors when it's the player's turn
            System.Drawing.Color redHighColorTurn = System.Drawing.Color.LightCoral;
            System.Drawing.Color redLowColorTurn = System.Drawing.Color.DarkRed;
            System.Drawing.Color redBorderColorTurn = System.Drawing.Color.Red;
            System.Drawing.Color yellowHighColorTurn = System.Drawing.Color.Yellow;
            System.Drawing.Color yellowLowColorTurn = System.Drawing.Color.Gold;
            System.Drawing.Color yellowBorderColorTurn = System.Drawing.Color.Yellow;
            System.Drawing.Color greenHighColorTurn = System.Drawing.Color.LightGreen;
            System.Drawing.Color greenLowColorTurn = System.Drawing.Color.DarkGreen;
            System.Drawing.Color greenBorderColorTurn = System.Drawing.Color.Green;
            System.Drawing.Color blueHighColorTurn = System.Drawing.Color.LightBlue;
            System.Drawing.Color blueLowColorTurn = System.Drawing.Color.DarkBlue;
            System.Drawing.Color blueBorderColorTurn = System.Drawing.Color.Blue;

            // Colors when it's not the player's turn            
            System.Drawing.Color redHighColorNormal = System.Drawing.Color.FromArgb(255, 239, 239, 239);
            System.Drawing.Color redLowColorNormal = System.Drawing.Color.FromArgb(255, 137, 68, 68);
            System.Drawing.Color yellowHighColorNormal = System.Drawing.Color.FromArgb(255, 255, 255, 127);
            System.Drawing.Color yellowLowColorNormal = System.Drawing.Color.FromArgb(255, 127, 106, 0);
            System.Drawing.Color greenHighColorNormal = System.Drawing.Color.FromArgb(255, 237, 237, 237);
            System.Drawing.Color greenLowColorNormal = System.Drawing.Color.FromArgb(255, 49, 99, 49);
            System.Drawing.Color blueHighColorNormal = System.Drawing.Color.FromArgb(255, 229, 229, 229);
            System.Drawing.Color blueLowColorNormal = System.Drawing.Color.FromArgb(255, 68, 68, 137);
            System.Drawing.Color borderColorNormal = System.Drawing.Color.Transparent;

            this.redPlayerCaption.GradientHighColor = redHighColorNormal;
            this.redPlayerCaption.GradientLowColor = redLowColorNormal;
            this.redPlayerCaption.BorderColor = new Ascend.BorderColor(borderColorNormal);
            this.yellowPlayerCaption.GradientHighColor = yellowHighColorNormal;
            this.yellowPlayerCaption.GradientLowColor = yellowLowColorNormal;
            this.yellowPlayerCaption.BorderColor = new Ascend.BorderColor(borderColorNormal);
            this.greenPlayerCaption.GradientHighColor = greenHighColorNormal;
            this.greenPlayerCaption.GradientLowColor = greenLowColorNormal;
            this.greenPlayerCaption.BorderColor = new Ascend.BorderColor(borderColorNormal);
            this.bluePlayerCaption.GradientHighColor = blueHighColorNormal;
            this.bluePlayerCaption.GradientLowColor = blueLowColorNormal;
            this.bluePlayerCaption.BorderColor = new Ascend.BorderColor(borderColorNormal);
            
            if (player.Name == this.redPlayerCaption.Text)
            {
                this.redPlayerCaption.GradientHighColor = redHighColorTurn;
                this.redPlayerCaption.GradientLowColor = redLowColorTurn;
                this.redPlayerCaption.BorderColor = new Ascend.BorderColor(redBorderColorTurn);
            }
            else if (player.Name == this.yellowPlayerCaption.Text)
            {
                this.yellowPlayerCaption.GradientHighColor = yellowHighColorTurn;
                this.yellowPlayerCaption.GradientLowColor = yellowLowColorTurn;
                this.yellowPlayerCaption.BorderColor = new Ascend.BorderColor(yellowBorderColorTurn);
            }
            else if (player.Name == this.greenPlayerCaption.Text)
            {
                this.greenPlayerCaption.GradientHighColor = greenHighColorTurn;
                this.greenPlayerCaption.GradientLowColor = greenLowColorTurn;
                this.greenPlayerCaption.BorderColor = new Ascend.BorderColor(greenBorderColorTurn);
            }
            else if (player.Name == this.bluePlayerCaption.Text)
            {
                this.bluePlayerCaption.GradientHighColor = blueHighColorTurn;
                this.bluePlayerCaption.GradientLowColor = blueLowColorTurn;
                this.bluePlayerCaption.BorderColor = new Ascend.BorderColor(blueBorderColorTurn);
            }
        }

        /// <summary>
        /// Delegate to enable async calls to update the view when die has been rolled.
        /// </summary>
        /// <param name="response"><see cref="Response"/> with information about what has happened.</param>
        delegate void UpdateViewOnDieRolledCallback(Response response);
        /// <summary>
        /// Updated the view on the GUI thread when the die has been rolled.
        /// </summary>
        /// <param name="response"><see cref="Response"/> with information about what has happened.</param>
        private void UpdateViewOnDieRolled(Response response)
        {
            this.die1.SetDieValue(this._engine.Die.Value);

            this.UpdatePlayersTurn(response.PlayersTurn);
            string text = "";

            if (response.NextAction == Action.MovePiece)
            {
                text += String.Format(Properties.Resources.InstructionMovePiece, response.PlayersTurn.Name);
            }
            else if (response.NextAction == Action.RollDie)
            {
                text += String.Format(Properties.Resources.InstructionRollDie, response.PlayersTurn.Name);
            }

            this.gameInstructionsLabel.Text = text;
        }

        /// <summary>
        /// Delegate to enable asyc calls to update the view when a piece has been moved.
        /// </summary>
        /// <param name="response"><see cref="Response"/> with information about what has happened.</param>
        delegate void UpdateViewOnPieceMovedCallback(Response response);
        /// <summary>
        /// Updates the view on the GUI thread when a piece has been moved.
        /// </summary>
        /// <param name="response"><see cref="Response"/> with information about what has happened.</param>
        private void UpdateViewOnPieceMoved(Response response)
        {
            this.UpdatePlayersTurn(response.PlayersTurn);
            string text = "";

            if (response.PlayerWon != null)
            {
                text += String.Format(Properties.Resources.InstructionHasFinished + (char)13, response.PlayerWon.Name);
            }

            if (response.NextAction == Action.RollDie)
            {
                text += String.Format(Properties.Resources.InstructionRollDie, response.PlayersTurn.Name);
            }

            this.gameInstructionsLabel.Text = text;
            this.board1.Invalidate();
        }

        /// <summary>
        /// Resumes a paused game.
        /// </summary>
        internal void StartGame()
        {
            if (this.Engine != null)
                this.Engine.StartGame();
        }

        /// <summary>
        /// Pauses a running game.
        /// </summary>
        internal void EndGame()
        {
            if (this.Engine != null)
                this.Engine.EndGame();
        }

        /// <summary>
        /// Gets the <see cref="GameStateForm"/> if it is open.
        /// </summary>
        /// <returns><see cref="GameStateForm"/></returns>
        private GameStateForm GetOpenGameStateForm()
        {
            foreach (Form form in Application.OpenForms)
                if (form.GetType() == typeof(GameStateForm))
                    return form as GameStateForm;

            return null;
        }
        #endregion  // Other methods

        #region Event handlers

        #region Form related events

        /// <summary>
        /// Handles the new game menu item's click event.
        /// </summary>
        private void newToolStripMenuItem_Click(object sender, EventArgs e)
        {
            this.StartNewGame();
        }

        /// <summary>
        /// Handles the exit menu item's click event.
        /// </summary>
        private void exitToolStripMenuItem_Click(object sender, EventArgs e)
        {
            this.Close();
        }

        /// <summary>
        /// Handles the debug mode menu item's checked changed event.
        /// </summary>
        private void debugModeToolStripMenuItem_CheckedChanged(object sender, EventArgs e)
        {
            if (this.debugModeToolStripMenuItem.Checked)
                this._engine.EndGame();
            else
                this._engine.StartGame();
        }

        /// <summary>
        /// Handles the show state window menu item's click event.
        /// </summary>
        private void showStateWindowToolStripMenuItem_Click(object sender, EventArgs e)
        {
            GameStateForm form = this.GetOpenGameStateForm();

            if (form != null)
                form.Show();
            else
                new GameStateForm(this).Show();
        }

        private void Game_Shown(object sender, EventArgs e)
        {
            this.StartNewGame();
        }

        private void die1_Click(object sender, EventArgs e)
        {
            if (!this.IsDebugMode)
                this._engine.RollDie();
            else
            {
                int dieValue = (int)this._engine.Die.Value.Value;
                dieValue++;
                if (dieValue > 6) dieValue = 1;
                this._engine.Die.Value = new Dice.DieValue(dieValue);
            }
        }

        private void Game_FormClosing(object sender, FormClosingEventArgs e)
        {
            if (this._engine != null)
                this._engine.EndGame();  // Stop the engine if it is running
        }

        private void Game_FormClosed(object sender, FormClosedEventArgs e)
        {
            Application.Exit();
        }
        #endregion // Form related events

        #region Engine related events

          /// <summary>
        /// Updates the view when engine reports that the die has been rolled.
        /// </summary>
        private void engine_DieRolled(object sender, ActionEventArgs e)
        {
            if (e.Response != null && !this.IsDisposed)
            {
                if (this.InvokeRequired)
                    this.Invoke(new UpdateViewOnDieRolledCallback(UpdateViewOnDieRolled), new object[] { e.Response });                    
                else
                    this.UpdateViewOnDieRolled(e.Response);                
            }
        }

        /// <summary>
        /// Updates the view when engine reports that a piece has been moved.
        /// </summary>
        private void engine_PieceMoved(object sender, ActionEventArgs e)
        {
            if (e.Response != null && !this.IsDisposed)
            {
                if (this.InvokeRequired)
                    this.Invoke(new UpdateViewOnPieceMovedCallback(UpdateViewOnPieceMoved), new object[] { e.Response });
                else
                    this.UpdateViewOnPieceMoved(e.Response);
            }            
        }

        /// <summary>
        /// Updates the view when enging reports that one of the players has finished.
        /// </summary>
        private void engine_PlayerFinished(object sender, ActionEventArgs e)
        {

        }

        /// <summary>
        /// Updates the view when the engine reports that the game is over.
        /// </summary>
        private void engine_GameOver(object sender, ActionEventArgs e)
        {

        }

        #endregion // Engine related events

        #region Debug mode related events

        /// <summary>
        /// Updates the view when die changes value (needed for debug mode).
        /// </summary>
        private void DieValueChanged(object sender, EventArgs e)
        {
            this.die1.DieValue = this._engine.Die.Value;
        }

        /// <summary>
        /// Changes player type when user clicks the player caption.
        /// </summary>
        private void PlayerCaption_MouseDown(object sender, MouseEventArgs e)
        {
            if (!this.IsDebugMode) return;
            Color playerColor = Color.White;

            if(sender == this.redPlayerCaption)
                playerColor = Color.Red;
            else if(sender == this.yellowPlayerCaption)
                playerColor = Color.Yellow;
            else if(sender == this.greenPlayerCaption)
                playerColor = Color.Green;
            else if(sender == this.bluePlayerCaption)
                playerColor = Color.Blue;

            Player player = this._engine.Players[playerColor];

            if (player != null)
            {
                PlayerType playerType = player.Type;

                if (playerType == PlayerType.Computer)
                    playerType = PlayerType.Human;
                else if (playerType == PlayerType.Human)
                    playerType = PlayerType.Computer;

                player.Type = playerType;
            }
        }

        /// <summary>
        /// Updates UI when the game state has changed
        /// </summary>
        private void engine_GameStateChanged(object sender, EventArgs e)
        {
            if(this.InvokeRequired)
            {
                this.Invoke(new MethodInvoker(ConnectModelAndUi));                            
            }                
            else
                this.ConnectModelAndUi();
        }

        #endregion

        #endregion  // Event handlers
        #endregion  // Methods
    }
}