﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.IO;

namespace Lokeiv.Games.Ludo.Forms.Client
{
    public partial class GameStateForm : Form
    {        
        private Game _game;
        private string _originalContent = string.Empty;
        private bool _updatingTextBoxFromSerializedEngine = false;
        private bool _isDesignMode = false;

        public GameStateForm()
        {
            InitializeComponent();
            this._isDesignMode = (LicenseManager.UsageMode == LicenseUsageMode.Designtime);
        }

        public GameStateForm(Game game) : this()
        {
            this._game = game;
            this.AddGameEngineEventHandlers();
            this.ToggleStartStop();
            this.SerializeGameState();
        }

        private void AddGameEngineEventHandlers()
        {
            this._game.Engine.GameStateChanged += new EventHandler(Engine_GameStateChanged);
        }

        private void RemoveGameEngineEventHandlers()
        {
            if (this._game != null && this._game.Engine != null)
            {
                this._game.Engine.GameStateChanged -= new EventHandler(Engine_GameStateChanged);
            }                
        }

        private void ToggleStartStop()
        {
            if (this._game != null)
            {
                bool isRunning = this._game.IsRunning;
                this.loadStateToolStripButton.Enabled = !isRunning;
                this.saveStateToolStripButton.Enabled = !isRunning;

                if (isRunning)
                {
                    this.startStopGameToolStripButton.Image = Properties.Resources.Stop_16x16;
                    this.startStopGameToolStripButton.Text = Properties.Resources.StopGame;
                    this.startStopGameToolStripButton.ToolTipText = this.startStopGameToolStripButton.Text;
                }
                else
                {
                    this.startStopGameToolStripButton.Image = Properties.Resources.Play_16x16;
                    this.startStopGameToolStripButton.Text = Properties.Resources.PlayGame;
                    this.startStopGameToolStripButton.ToolTipText = this.startStopGameToolStripButton.Text;
                }
            }
        }

        private Engine TryParseEngineData(string engineData)
        {
            try
            {
                DeepSerializer serializer = new DeepSerializer();
                return serializer.DeserializeEngine(engineData);
            }
            catch
            {
                return null;
            }
        }

        private void SerializeGameState()
        {
            DeepSerializer serializer = new DeepSerializer();
            string engineData = serializer.SerializeToXml(this._game.Engine);

            try
            {
                this._updatingTextBoxFromSerializedEngine = true;

                if (this.InvokeRequired)
                {
                    this.Invoke(new MethodInvoker(delegate()
                    {
                        this._originalContent = engineData;
                        this.gameStateTextBox.Text = this._originalContent;
                    }));
                }
                else
                {
                    this._originalContent = engineData;
                    this.gameStateTextBox.Text = this._originalContent;
                }
            }
            finally
            {
                this._updatingTextBoxFromSerializedEngine = false;
            }
        }

        private void ApplyGameState()
        {
            if (!this._updatingTextBoxFromSerializedEngine)
            {
                Engine engine = this.TryParseEngineData(this.gameStateTextBox.Text);

                if (engine != null)
                {
                    this.RemoveGameEngineEventHandlers();
                    this._game.LoadRestoredEngine(engine);
                    this.AddGameEngineEventHandlers();
                    this.ToggleStartStop();
                }
                else
                    MessageBox.Show(this, "Was not able to deserialize the game engine.", "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private void GameStateForm_Load(object sender, EventArgs e)
        {
            if (!this._isDesignMode)
            {                
                this.ToggleStartStop();
            }
        }

        private void loadStateToolStripButton_Click(object sender, EventArgs e)
        {            
            if (theOpenFileDialog.ShowDialog(this) == System.Windows.Forms.DialogResult.OK)
            {
                try
                {
                    FileInfo file = new FileInfo(theOpenFileDialog.FileName);

                    if (file.Exists)
                        using (StreamReader reader = file.OpenText())
                        {
                            this._originalContent = reader.ReadToEnd();
                            this.gameStateTextBox.Text = this._originalContent;
                            this.ApplyGameState();
                        }
                }
                catch (Exception ex)
                {
                    MessageBox.Show(this, ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                }
            }
        }

        private void saveStateToolStripButton_Click(object sender, EventArgs e)
        {
            if (theSaveFileDialog.ShowDialog(this) == System.Windows.Forms.DialogResult.OK)
            {
                try
                {
                    FileInfo file = new FileInfo(theSaveFileDialog.FileName);

                    using(TextWriter writer = new StreamWriter(file.OpenWrite()))
                        writer.Write(this.gameStateTextBox.Text);                            
                }
                catch (Exception ex)
                {
                    MessageBox.Show(this, ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                }
            }
        }

        private void startStopGameToolStripButton_Click(object sender, EventArgs e)
        {
            if (this._game != null)
                if (this._game.IsRunning)
                    this._game.EndGame();
                else
                    this._game.StartGame();

            this.ToggleStartStop();
        }

        private void applyToolStripButton_Click(object sender, EventArgs e)
        {
            this.ApplyGameState();
        }

        private void cancelToolStripButton_Click(object sender, EventArgs e)
        {
            this.gameStateTextBox.Text = this._originalContent;
        }

        private void gameStateTextBox_TextChanged(object sender, EventArgs e)
        {
            bool hasChanges = (this._originalContent != this.gameStateTextBox.Text);
            this.applyToolStripButton.Enabled = hasChanges;
            this.cancelToolStripButton.Enabled = hasChanges;
        }

        private void Game_NewGameEngineInitialized(object sender, EventArgs e)
        {
            this._game.Engine.GameStateChanged += new EventHandler(Engine_GameStateChanged);
        }

        void Engine_GameStateChanged(object sender, EventArgs e)
        {
            this.SerializeGameState();
        }        

        #region Dispose        
        /// <summary>
        /// Clean up any resources being used.
        /// </summary>
        /// <param name="disposing">true if managed resources should be disposed; otherwise, false.</param>
        protected override void Dispose(bool disposing)
        {
            if (disposing)
            {
                if (this.components != null) this.components.Dispose();
                
                if (this._game != null)
                {
                    this._game.NewGameEngineInitialized -= this.Game_NewGameEngineInitialized;
                    this._game.Engine.GameStateChanged -= this.Engine_GameStateChanged;
                }
            }

            this._game = null;
            base.Dispose(disposing);
        }
        #endregion
    }
}
