using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Text;
using System.Windows.Forms;
using Ascend.Windows.Forms;
using Lokeiv.Games.Ludo.Model;

namespace Lokeiv.Games.Ludo.Forms.Client
{
    public partial class NewGameForm : Form
    {
        private Player _redPlayer;
        private Player _yellowPlayer;
        private Player _greenPlayer;
        private Player _bluePlayer;
        private PlayerCollection _players = new PlayerCollection();	

        #region Public propeties
        public PlayerCollection Players
        {
            get { return _players; }
            set { _players = value; }
        }
        #endregion

        #region Constructors
        public NewGameForm()
        {
            InitializeComponent();
            InitializePlayers();
        }
        #endregion

        /// <summary>
        /// Sets up the player data, and updates GUI.
        /// </summary>
        private void InitializePlayers()
        {
            // First get default player object to avoid NullPointerException when
            // the check box controls checked property is changed.
            this._redPlayer = this.GetDefaultPlayerObjectForSettingsKey("RedPlayer");            
            this._yellowPlayer = this.GetDefaultPlayerObjectForSettingsKey("YellowPlayer");
            this._greenPlayer = this.GetDefaultPlayerObjectForSettingsKey("GreenPlayer");
            this._bluePlayer = this.GetDefaultPlayerObjectForSettingsKey("BluePlayer");            
            
            // Then try to deserialize settings
            this._redPlayer = this.DeserilizePlayer("RedPlayer");
            this._yellowPlayer = this.DeserilizePlayer("YellowPlayer");
            this._greenPlayer = this.DeserilizePlayer("GreenPlayer");
            this._bluePlayer = this.DeserilizePlayer("BluePlayer");

            this._players.Add(this._redPlayer);  // red player dosn't get added automatically because it cannot be disabled/enabled

            // Update GUI
            this.UpdateGuiOnPlayersInitialized(null, this.redPlayerImage, this.redPlayerCaption, this._redPlayer);
            this.UpdateGuiOnPlayersInitialized(this.yellowPlayerCheckBox, this.yellowPlayerImage, this.yellowPlayerCaption, this._yellowPlayer);
            this.UpdateGuiOnPlayersInitialized(this.greenPlayerCheckBox, this.greenPlayerImage, this.greenPlayerCaption, this._greenPlayer);
            this.UpdateGuiOnPlayersInitialized(this.bluePlayerCheckBox, this.bluePlayerImage, this.bluePlayerCaption, this._bluePlayer);
        }

        /// <summary>
        /// Serializes player object when dialog is closed.
        /// </summary>
        private void SerializePlayers()
        {
            string redPlayerSettings = this._redPlayer.Name + ", " +
                (this._redPlayer.Type == PlayerType.Computer ? "Computer" : "Human") + ", " +
                "Enabled";
            string yellowPlayerSettings = this._yellowPlayer.Name + ", " +
                (this._yellowPlayer.Type == PlayerType.Computer ? "Computer" : "Human") + ", " +
                (this.yellowPlayerCheckBox.Checked ? "Enabled" : "Disabled");
            string greenPlayerSettings = this._greenPlayer.Name + ", " +
                            (this._greenPlayer.Type == PlayerType.Computer ? "Computer" : "Human") + ", " +
                            (this.greenPlayerCheckBox.Checked ? "Enabled" : "Disabled");
            string bluePlayerSettings = this._bluePlayer.Name + ", " +
                (this._bluePlayer.Type == PlayerType.Computer ? "Computer" : "Human") + ", " +
                (this.bluePlayerCheckBox.Checked ? "Enabled" : "Disabled");

            Properties.Settings.Default.RedPlayer = redPlayerSettings;
            Properties.Settings.Default.YellowPlayer = yellowPlayerSettings;
            Properties.Settings.Default.GreenPlayer = greenPlayerSettings;
            Properties.Settings.Default.BluePlayer = bluePlayerSettings;
        }

        /// <summary>
        /// Reads user settings and creates a <see cref="Player"/> object.
        /// </summary>
        /// <param name="settingsKey">The key in the settings file</param>
        /// <returns><see cref="Player"/></returns>
        private Player DeserilizePlayer(string settingsKey)
        {
            Player player = null;

            try
            {
                System.Configuration.SettingsPropertyValue propertyValue = Properties.Settings.Default.PropertyValues[settingsKey];

                if (propertyValue != null)
                {
                    string playerValues = propertyValue.PropertyValue.ToString();
                    string[] valueArray = playerValues.Split(",".ToCharArray(), StringSplitOptions.None);
                    string playerName = "";
                    PlayerType playerType = PlayerType.Human;
                    bool playerEnabled = true;

                    for (int i = 0; i < valueArray.Length; i++)
                    {
                        string value = valueArray[i].Trim();

                        if (i == 0)
                        {
                            playerName = value;
                        }
                        else if (i == 1)
                        {
                            if (value.ToLower() == "human")
                            {
                                playerType = PlayerType.Human;
                            }
                        }
                        else if (i == 2)
                        {
                            if (value.ToLower() != "enabled")
                            {
                                playerEnabled = false;
                            }
                        }
                    }

                    player = this.GetDefaultPlayerObjectForSettingsKey(settingsKey);
                    player.Name = playerName;
                    player.Type = playerType;

                    switch (player.Color)
                    {
                        case Color.Yellow:
                            this.yellowPlayerCheckBox.Checked = playerEnabled;
                            break;
                        case Color.Green:
                            this.greenPlayerCheckBox.Checked = playerEnabled;
                            break;
                        case Color.Blue:
                            this.bluePlayerCheckBox.Checked = playerEnabled;
                            break;
                    }
                }
                else
                {
                    player = this.GetDefaultPlayerObjectForSettingsKey(settingsKey);
                }
            }
            catch (Exception)
            {
                player = this.GetDefaultPlayerObjectForSettingsKey(settingsKey);
            }

            return player;
        }        

        /// <summary>
        /// Creates a default <see cref="Player"/> object with the right color.
        /// Is used if deserilizing from settings fails.
        /// </summary>
        /// <param name="settingsKey">The settings key</param>
        /// <returns><see cref="Player"/></returns>
        private Player GetDefaultPlayerObjectForSettingsKey(string settingsKey)
        {
            switch (settingsKey)
            {
                case "RedPlayer":
                    return new Player("Red player", PlayerType.Human, Color.Red);
                case "YellowPlayer":
                    return new Player("Yellow player", PlayerType.Computer, Color.Yellow);
                case "GreenPlayer":
                    return new Player("Green player", PlayerType.Computer, Color.Green);
                case "BluePlayer":
                    return new Player("Blue player", PlayerType.Computer, Color.Blue);
            }

            return null;
        }

        /// <summary>
        /// Updates the GUI with the right picture and name for a player, when initialized.
        /// </summary>
        /// <param name="checkBox">Check box for the player</param>
        /// <param name="pictureBox">Picture box for the player</param>
        /// <param name="caption">Caption for the player</param>
        /// <param name="player">The <see cref="Player"/> object</param>
        private void UpdateGuiOnPlayersInitialized(CheckBox checkBox, PictureBox pictureBox, GradientCaption caption, Player player)
        {
            // Set the right picture
            if(checkBox != null && !checkBox.Checked)
            {
                if(player.Type == PlayerType.Computer)
                {
                    pictureBox.Image = Properties.Resources.computer_gray;
                }
                else
                {
                    pictureBox.Image = Properties.Resources.person_gray;
                }
            }
            else
            {
                if (player.Type == PlayerType.Computer)
                {
                    pictureBox.Image = GetImage(player.Color, player.Type);
                }
            }

            caption.Text = player.Name;
        }

        /// <summary>
        /// Gets the right image for the given color and player type.
        /// </summary>
        /// <param name="color"><see cref="Color"/></param>
        /// <param name="type"><see cref="PlayerType"/></param>
        /// <returns><see cref="System.Drawing.Image"/></returns>
        private System.Drawing.Image GetImage(Color color, PlayerType type)
        {
            switch (color)
            {
                case Color.Red:
                    if (type == PlayerType.Computer)
                    {
                        return Properties.Resources.computer_red;
                    }
                    else
                    {
                        return Properties.Resources.person_red;
                    }                    
                case Color.Yellow:
                    if (type == PlayerType.Computer)
                    {
                        return Properties.Resources.computer_yellow;
                    }
                    else
                    {
                        return Properties.Resources.person_yellow;
                    }
                case Color.Green:
                    if (type == PlayerType.Computer)
                    {
                        return Properties.Resources.computer_green;
                    }
                    else
                    {
                        return Properties.Resources.person_green;
                    }
                case Color.Blue:
                    if (type == PlayerType.Computer)
                    {
                        return Properties.Resources.computer_blue;
                    }
                    else
                    {
                        return Properties.Resources.person_blue;
                    }
            }

            return null;
        }

        /// <summary>
        /// Helps update GUI and data when a player is enabled or disabled.
        /// </summary>
        /// <param name="checkBox">The check box that enables and disables the player</param>
        /// <param name="pictureBox">The picture box displaying a image for the player</param>
        /// <param name="caption">The caption displaying the name for the player</param>
        /// <param name="player">The player object</param>
        private void EnableDisablePlayer(CheckBox checkBox, PictureBox pictureBox, GradientCaption caption, Player player)
        {
            bool enabled = checkBox.Checked;
            pictureBox.Enabled = enabled;
            caption.Enabled = enabled;

            if (enabled)
            {
                this._players.Add(player);
                pictureBox.Image = this.GetImage(player.Color, player.Type);
            }
            else
            {
                this._players.Remove(player);

                if (player.Type == PlayerType.Computer)
                {
                    pictureBox.Image = Properties.Resources.computer_gray;
                }
                else
                {
                    pictureBox.Image = Properties.Resources.person_gray;
                }
            }
        }

        /// <summary>
        /// Helps update GUI and data when a player changes type.
        /// </summary>
        /// <param name="pictureBox">The picture box displaying a image for the player</param>
        /// <param name="player">The player object</param>
        private void ChangePlayerType(PictureBox pictureBox, Player player)
        {
            if (player.Type == PlayerType.Computer)
            {                
                player.Type = PlayerType.Human;
            }
            else
            {
                player.Type = PlayerType.Computer;
            }

            pictureBox.Image = this.GetImage(player.Color, player.Type);
        }

        #region Event handlers
        private void yellowPlayerCheckBox_CheckedChanged(object sender, EventArgs e)
        {
            this.EnableDisablePlayer(this.yellowPlayerCheckBox, this.yellowPlayerImage, this.yellowPlayerCaption, this._yellowPlayer);
        }

        private void greenPlayerCheckBox_CheckedChanged(object sender, EventArgs e)
        {
            this.EnableDisablePlayer(this.greenPlayerCheckBox, this.greenPlayerImage, this.greenPlayerCaption, this._greenPlayer);
        }

        private void bluePlayerCheckBox_CheckedChanged(object sender, EventArgs e)
        {
            this.EnableDisablePlayer(this.bluePlayerCheckBox, this.bluePlayerImage, this.bluePlayerCaption, this._bluePlayer);
        }

        private void NewGameForm_FormClosing(object sender, FormClosingEventArgs e)
        {
            if (this.DialogResult == DialogResult.OK)
            {
                if (this._players.Count < 2)
                {
                    e.Cancel = true;
                    MessageBox.Show("At least two players.", Application.ProductName, MessageBoxButtons.OK, MessageBoxIcon.Error);
                }

                foreach (Player player in this._players)
                {
                    player.SetWon(false);
                }
            }
        }

        private void redPlayerImage_Click(object sender, EventArgs e)
        {
            this.ChangePlayerType(this.redPlayerImage, this._redPlayer);
        }

        private void yellowPlayerImage_Click(object sender, EventArgs e)
        {
            this.ChangePlayerType(this.yellowPlayerImage, this._yellowPlayer);
        }

        private void greenPlayerImage_Click(object sender, EventArgs e)
        {
            this.ChangePlayerType(this.greenPlayerImage, this._greenPlayer);
        }

        private void bluePlayerImage_Click(object sender, EventArgs e)
        {
            this.ChangePlayerType(this.bluePlayerImage, this._bluePlayer);
        }
        #endregion

    }
}