using System;
using System.Collections.Generic;
using System.Text;
using System.Xml.Serialization;

namespace Lokeiv.Games.Ludo.Model
{
    public class GameOptions : IXmlSerializable
    {

        #region Public properties
        /// <summary>
        /// Wheter or not two or more pieces of the same color on the same square should form 
        /// a block for pieces of other colors.
        /// </summary>
        public bool FormBlocks { get; set; }

        /// <summary>
        /// Whether or not the die should be rolled three times when all
        /// pieces are in the house. If allowed, players are able to start faster.
        /// </summary>
        public bool RollThreeTimes { get; set; }

        /// <summary>
        /// The type of Ludo to play, like Classic or the one with stars and earth.
        /// </summary>
        public GameType GameType { get; set; }

        /// <summary>
        /// Gets or sets how fast the computer player should react.
        /// </summary>
        public GameSpeed GameSpeed { get; set; }

        #endregion

        #region Constructors
        /// <summary>
        /// Creates a classic game type with default options.
        /// </summary>
        public GameOptions()
        {
            this.GameType = GameType.Classic;
            this.GameSpeed = Model.GameSpeed.Medium;
            this.FormBlocks = true;
            this.RollThreeTimes = false;
        }
        #endregion  // Constructors

        #region Methods
        public GameOptions Clone()
        {
            GameOptions other = new GameOptions();
            other.FormBlocks = this.FormBlocks;
            other.RollThreeTimes = this.RollThreeTimes;
            other.GameType = this.GameType;
            other.GameSpeed = this.GameSpeed;

            return other;
        }
        #endregion  // Methods

        #region IXmlSerializable
        public System.Xml.Schema.XmlSchema GetSchema()
        {
            return null;
        }

        public void ReadXml(System.Xml.XmlReader reader)
        {
            reader.MoveToContent();

            while (reader.Read())
            {                
                if (reader.NodeType == System.Xml.XmlNodeType.Element)
                {
                    string nodeName = reader.Name.ToLower();
                    string value = reader.ReadInnerXml();

                    if (!string.IsNullOrEmpty(value))
                    {
                        switch (nodeName)
                        {
                            case "gametype":
                                this.GameType = (GameType)System.Enum.Parse(typeof(GameType), value, true);
                                break;
                            case "rollthreetimes":
                                this.RollThreeTimes = bool.Parse(value);
                                break;
                            case "formblocks":
                                this.FormBlocks = bool.Parse(value);
                                break;
                            case "gamespeed":
                                this.GameSpeed = (GameSpeed)System.Enum.Parse(typeof(GameSpeed), value, true);
                                break;
                        }
                    }
                }
            }
        }

        public void WriteXml(System.Xml.XmlWriter writer)
        {
            // Write GameType
            writer.WriteStartElement("GameType");
            writer.WriteString(this.GameType.ToString());
            writer.WriteEndElement();

            // Write RollThreeTimes
            writer.WriteStartElement("RollThreeTimes");
            writer.WriteString(this.RollThreeTimes.ToString());
            writer.WriteEndElement();

            // Write FormBlocks
            writer.WriteStartElement("FormBlocks");
            writer.WriteString(this.FormBlocks.ToString());
            writer.WriteEndElement();

            // Write GameSpeed
            writer.WriteStartElement("GameSpeed");
            writer.WriteString(this.GameSpeed.ToString());
            writer.WriteEndElement();
        }
        #endregion

    }
}
