﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Navigation;
using System.Windows.Shapes;
using Microsoft.Phone.Controls;
using Microsoft.Phone.Shell;
using Ludo.WP7.Client.ViewModels;
using System.IO.IsolatedStorage;
using System.IO;
using System.ComponentModel;

namespace Ludo.WP7.Client
{
    public partial class App : Application
    {

        private static readonly string viewModelStateKey = "MainViewModel";
        private static readonly string gameStateFileName = "GameState.ludo";
        private static MainViewModel viewModel = null;        

        /// <summary>
        /// A static ViewModel used by the views to bind against.
        /// </summary>
        /// <returns>The MainViewModel object.</returns>
        public static MainViewModel ViewModel
        {
            get
            {
                // Delay creation of the view model until necessary
                if (viewModel == null)
                    viewModel = new MainViewModel();

                return viewModel;
            }
        }

        /// <summary>
        /// Determines if the application is being run by a design tool
        /// </summary>
        private bool IsDesignMode
        {
            get
            {
                // VisualStudio sometimes returns false for DesignMode, DesignTool is our backup
                return DesignerProperties.GetIsInDesignMode(this.RootFrame) || DesignerProperties.IsInDesignTool;
            }
        }


        /// <summary>
        /// Determines if the application is running in the dark theme
        /// </summary>
        private bool IsDarkTheme
        {
            get
            {
                if (IsDesignMode)
                {
                    return true;
                }
                else
                {
                    return (Visibility)Application.Current.Resources["PhoneDarkThemeVisibility"] == Visibility.Visible;
                }
            }
        }

        /// <summary>
        /// Provides easy access to the root frame of the Phone Application.
        /// </summary>
        /// <returns>The root frame of the Phone Application.</returns>
        public PhoneApplicationFrame RootFrame { get; private set; }

        /// <summary>
        /// Constructor for the Application object.
        /// </summary>
        public App()
        {
            // Global handler for uncaught exceptions. 
            UnhandledException += Application_UnhandledException;

            // Show graphics profiling information while debugging.
            if (System.Diagnostics.Debugger.IsAttached)
            {
                // Display the current frame rate counters
                //Application.Current.Host.Settings.EnableFrameRateCounter = true;

                // Show the areas of the app that are being redrawn in each frame.
                //Application.Current.Host.Settings.EnableRedrawRegions = true;

                // Enable non-production analysis visualization mode, 
                // which shows areas of a page that are being GPU accelerated with a colored overlay.
                //Application.Current.Host.Settings.EnableCacheVisualization = true;
            }

            // Standard Silverlight initialization
            InitializeComponent();

            // Phone-specific initialization
            InitializePhoneApplication();
        }

        public static void Quit()
        {
            throw new QuitException();
        }

        #region App state handling in application lifecycle

        private void LoadViewModelFromIsolatedStorage()
        {
            try
            {
                using (var store = IsolatedStorageFile.GetUserStoreForApplication())
                using (var stream = new IsolatedStorageFileStream(gameStateFileName, System.IO.FileMode.OpenOrCreate, FileAccess.Read, store))
                using (var reader = new StreamReader(stream))
                {
                    if (!reader.EndOfStream)
                    {
                        string xml = reader.ReadToEnd();
                        viewModel = Utils.DeepSerializer.DeserializeMainViewModel(xml);
                    }
                }
            }
            catch (Exception ex)
            {
                viewModel = new MainViewModel();
                App.ViewModel.ErrorViewModel.Exception = ex;
                App.ViewModel.ErrorViewModel.IsDeserializationException = true;
                (RootVisual as Microsoft.Phone.Controls.PhoneApplicationFrame).Source = new Uri("/Views/ErrorPage.xaml", UriKind.Relative);
            }
        }

        private void LoadViewModelFromAppState()
        {
            try
            {
                if (PhoneApplicationService.Current.State.ContainsKey(viewModelStateKey))
                    viewModel = PhoneApplicationService.Current.State[viewModelStateKey] as MainViewModel;
            }
            catch (Exception ex)
            {
                viewModel = new MainViewModel();
                App.ViewModel.ErrorViewModel.Exception = ex;
                App.ViewModel.ErrorViewModel.IsDeserializationException = true;
                (RootVisual as Microsoft.Phone.Controls.PhoneApplicationFrame).Source = new Uri("/Views/ErrorPage.xaml", UriKind.Relative);
            }
        }

        private void SaveViewModelToIsolatedStorage()
        {
            using (var store = IsolatedStorageFile.GetUserStoreForApplication())
            using (var stream = new IsolatedStorageFileStream(gameStateFileName, FileMode.Create, FileAccess.Write, store))
            using (var writer = new StreamWriter(stream))
            {
                string xml = Utils.DeepSerializer.SerializeToXml(viewModel);
                writer.Write(xml);
            }
        }

        private void SaveViewModelToAppState()
        {
            PhoneApplicationService.Current.State[viewModelStateKey] = ViewModel;
        }

        // Code to execute when the application is launching (eg, from Start)
        // This code will not execute when the application is reactivated
        private void Application_Launching(object sender, LaunchingEventArgs e)
        {
            this.LoadViewModelFromIsolatedStorage();

            if (viewModel == null)
            {
                viewModel = new MainViewModel();
                //viewModel.Update();
            }

            this.RootFrame.DataContext = ViewModel;
        }

        // Code to execute when the application is activated (brought to foreground)
        // This code will not execute when the application is first launched
        private void Application_Activated(object sender, ActivatedEventArgs e)
        {            
            // Ensure that application state is restored appropriately            
            //this.LoadViewModelFromAppState();
            // Since automatic XML deserialization fails the state must be manually deserialized from Isolated Storage anyway
            this.LoadViewModelFromIsolatedStorage();

            if (viewModel == null)
            {
                viewModel = new MainViewModel();
            }

            this.RootFrame.DataContext = ViewModel;
        }

        // Code to execute when the application is deactivated (sent to background)
        // This code will not execute when the application is closing
        private void Application_Deactivated(object sender, DeactivatedEventArgs e)
        {
            this.SaveViewModelToAppState();
            this.SaveViewModelToIsolatedStorage();
        }

        // Code to execute when the application is closing (eg, user hit Back)
        // This code will not execute when the application is deactivated
        private void Application_Closing(object sender, ClosingEventArgs e)
        {
            // Ensure that required application state is persisted here.
            this.SaveViewModelToIsolatedStorage();
        }
        #endregion  // App state handling in application lifecycle

        // Code to execute if a navigation fails
        private void RootFrame_NavigationFailed(object sender, NavigationFailedEventArgs e)
        {
            if (System.Diagnostics.Debugger.IsAttached)
            {
                // A navigation has failed; break into the debugger
                System.Diagnostics.Debugger.Break();
            }
        }

        // Code to execute on Unhandled Exceptions
        private void Application_UnhandledException(object sender, ApplicationUnhandledExceptionEventArgs e)
        {
            if (e.ExceptionObject is QuitException)
                return;

            if (System.Diagnostics.Debugger.IsAttached)
            {
                // An unhandled exception has occurred; break into the debugger
                System.Diagnostics.Debugger.Break();
            }

            // Running on a device / emulator without debugging
            e.Handled = true;
            App.ViewModel.ErrorViewModel.Exception = e.ExceptionObject;
            App.ViewModel.ErrorViewModel.IsDeserializationException = false;
            (RootVisual as Microsoft.Phone.Controls.PhoneApplicationFrame).Source = new Uri("/Views/ErrorPage.xaml", UriKind.Relative);
        }

        #region Phone application initialization

        // Avoid double-initialization
        private bool phoneApplicationInitialized = false;

        // Do not add any additional code to this method
        private void InitializePhoneApplication()
        {
            if (phoneApplicationInitialized)
                return;

            // Create the frame but don't set it as RootVisual yet; this allows the splash
            // screen to remain active until the application is ready to render.
            RootFrame = new PhoneApplicationFrame();
            RootFrame.Navigated += CompleteInitializePhoneApplication;

            // Handle navigation failures
            RootFrame.NavigationFailed += RootFrame_NavigationFailed;

            // Ensure we don't initialize again
            phoneApplicationInitialized = true;
        }

        // Do not add any additional code to this method
        private void CompleteInitializePhoneApplication(object sender, NavigationEventArgs e)
        {
            // Set the root visual to allow the application to render
            if (RootVisual != RootFrame)
                RootVisual = RootFrame;

            // Remove this handler since it is no longer needed
            RootFrame.Navigated -= CompleteInitializePhoneApplication;
        }

        #endregion

        private class QuitException : Exception {}
    }
}