﻿using System;
using System.ComponentModel;
using System.Text;
using System.Windows.Input;
using Ludo.WP7.Client.Utils;
using Microsoft.Phone.Tasks;

namespace Ludo.WP7.Client.ViewModels
{
    public class ErrorViewModel : INotifyPropertyChanged
    {

        #region Events
        public event EventHandler<EventArgs> NavigateToNewGame;
        #endregion

        #region Properties
        private MainViewModel _mainViewModel;
        public MainViewModel MainViewModel
        {
            get { return _mainViewModel; }
            set
            {
                if (_mainViewModel != value)
                {
                    _mainViewModel = value;
                    NotifyPropertyChanged("MainViewModel");
                }
            }
        }

        private Exception _exception;
        public Exception Exception
        {
            get
            {
                return this._exception;
            }
            set
            {
                if (this._exception != value)
                {
                    this._exception = value;
                    this.NotifyPropertyChanged("Exception");
                }
            }
        }

        private bool _isDeserializationException;
        public bool IsDeserializationException 
        { 
            get
            {
                return this._isDeserializationException;
            }
            set
            {
                if (this._isDeserializationException != value)
                {
                    this._isDeserializationException = value;
                    this.NotifyPropertyChanged("IsDeserializationException");
                    this.NotifyPropertyChanged("FriendlyMessage");
                }
            }
        }
        
        public string FriendlyMessage
        {
            get
            {
                return this.GetFriendlyMessage();
            }
        }

        public ICommand StartNewGame { get; private set; }
        public ICommand SendErrorReport { get; private set; }
        #endregion  // Properties

        #region Constructors
        public ErrorViewModel()
        {
            this.StartNewGame = new RelayCommand(this.OnStartNewGame);
            this.SendErrorReport = new RelayCommand(this.OnSendErrorReport);
        }

        public ErrorViewModel(MainViewModel mainViewModel)
            : this()
        {
            this.MainViewModel = mainViewModel;
        }

        public ErrorViewModel(Exception exception) 
            : this()
        {
            this.Exception = exception;
        }
        #endregion  // Constructors

        #region Methods

        #region Other methods
        private string CreateEmailBody()
        {
            StringBuilder sb = new StringBuilder();

            if (this.IsDeserializationException)
                sb.AppendLine("Unhandled exception");
            else
                sb.AppendLine("Handled exception");

            sb.AppendLine();
            sb.AppendLine("Time stamp: " + DateTime.Now.ToShortDateString() + " " + DateTime.Now.ToShortTimeString());
            sb.AppendLine();

            if (this.Exception != null)
                sb.AppendLine(this.Exception.ToString());

            return sb.ToString();
        }

        private string GetFriendlyMessage()
        {
            if (this.IsDeserializationException)
                return "An error occured loading your previous game. Please send an error report to help fix the problem, then start a new game.";
                
            return "An unexpected error occured. Please send an error report to help fix the problem, then start a new game.";
        }

        #endregion  // Other methods

        #region Event raisers

        /// <summary>
        /// Raises the <see cref="NavigateToNewGame"/> event.
        /// </summary>
        private void OnNavigateToNewGame(EventArgs e)
        {
            this.MainViewModel.NewGameViewModel.NumberOfStepsBackToGame = 2;

            if (this.NavigateToNewGame != null)
                this.NavigateToNewGame(this, e);
        }

        #endregion  // Event raisers

        #region Commands
        /// <summary>
        /// Navigates to the new game view to start a new game.
        /// </summary>
        private void OnStartNewGame()
        {
            this.OnNavigateToNewGame(EventArgs.Empty);
        }        

        /// <summary>
        /// Sends email with error report.
        /// </summary>
        private void OnSendErrorReport()
        {
            EmailComposeTask emailcomposer = new EmailComposeTask();
            emailcomposer.To = "ludodotnet@hotmail.com";
            emailcomposer.Subject = "Error report for Ludo.NET for WP7";
            emailcomposer.Body = this.CreateEmailBody();
            emailcomposer.Show();
        }
        #endregion  // Commands

        #endregion  // Methods

        #region INotifyPropertyChanged
        /// <summary>
        /// Event to notify the view when a property has changed.
        /// </summary>
        public event PropertyChangedEventHandler PropertyChanged;

        /// <summary>
        /// Fires the <see cref="PropertyChanged"/> event for the given <paramref name="propertyName"/>.
        /// </summary>
        /// <param name="propertyName">Name of property that has changed.</param>
        private void NotifyPropertyChanged(String propertyName)
        {
            PropertyChangedEventHandler handler = PropertyChanged;
            if (null != handler)
            {
                handler(this, new PropertyChangedEventArgs(propertyName));
            }
        }
        #endregion  // INotifyPropertyChanged

    }
}
