﻿using System;
using System.ComponentModel;
using System.Windows.Input;
using System.Xml.Serialization;
using Ludo.WP7.Client.Utils;

namespace Ludo.WP7.Client.ViewModels
{
    public class NewGamePlayerViewModel : INotifyPropertyChanged, IXmlSerializable
    {

        #region Properties

        private NewGamePlayerType _playerType = NewGamePlayerType.Human;
        public NewGamePlayerType PlayerType
        {
            get { return _playerType; }
            set
            {
                if (_playerType != value)
                {
                    _playerType = value;
                    NotifyPropertyChanged("PlayerType");
                    NotifyPropertyChanged("PlayerTypeImageSource");
                }
            }
        }

        private Lokeiv.Games.Ludo.Model.Color _playerColor = Lokeiv.Games.Ludo.Model.Color.Red;
        public Lokeiv.Games.Ludo.Model.Color PlayerColor
        {
            get { return _playerColor; }
            set
            {
                if (_playerColor != value)
                {
                    _playerColor = value;
                    NotifyPropertyChanged("PlayerColor");
                    NotifyPropertyChanged("PlayerName");  // Until support for custom player names is implemented, the player name changes with the color.
                    NotifyPropertyChanged("PlayerTypeImageSource");
                }
            }
        }

        public string PlayerName
        {
            get { return PlayerColor.ToString() + " Player"; }            
        }

        public string PlayerTypeImageSource
        {
            get
            {
                string imageSource = string.Empty;

                switch (PlayerType)
                {
                    case NewGamePlayerType.Computer:
                        imageSource = string.Format(@"\Images\Phone{0}.png", PlayerColor.ToString());
                        break;
                    case NewGamePlayerType.Human:
                        imageSource = string.Format(@"\Images\Human{0}.png", PlayerColor.ToString());
                        break;
                    case NewGamePlayerType.NoPlayer:
                        imageSource = string.Format(@"\Images\NoPlayer{0}.png", PlayerColor.ToString());
                        break;
                }

                return imageSource;
            }
        }

        [XmlIgnore]
        public ICommand ChangePlayerType { get; private set; }

        #endregion  // Properties

        #region Constructors

        public NewGamePlayerViewModel(Lokeiv.Games.Ludo.Model.Player player)
        {
            this.PlayerColor = player.Color;
            this.PlayerType = player.Type == Lokeiv.Games.Ludo.Model.PlayerType.Computer ? NewGamePlayerType.Computer : NewGamePlayerType.Human;
            this.ChangePlayerType = new RelayCommand(OnChangePlayerType);
        }

        #endregion  // Constructors

        #region Methods

        #region Other methods

        /// <summary>
        /// Gets the new player type based on the current <see cref="PlayerType"/> and the <see cref="PlayerColor"/>.
        /// Red player cannot be set to <see cref="NewGamePlayerType.NoPlayer"/>.
        /// </summary>
        /// <returns><see cref="NewGamePlayerType"/></returns>
        private NewGamePlayerType GetNewPlayerType()
        {
            NewGamePlayerType newPlayerType;

            if (this.PlayerType == NewGamePlayerType.Computer)
            {
                newPlayerType = NewGamePlayerType.Human;
            }
            else if (this.PlayerType == NewGamePlayerType.Human)
            {
                if (this.PlayerColor == Lokeiv.Games.Ludo.Model.Color.Red)
                {
                    // Red player is required in the game
                    newPlayerType = NewGamePlayerType.Computer;
                }
                else
                {
                    newPlayerType = NewGamePlayerType.NoPlayer;
                }
            }
            else
            {
                newPlayerType = NewGamePlayerType.Computer;
            }

            return newPlayerType;
        }

        /// <summary>
        /// Gets the <see cref="Lokeiv.Games.Ludo.Model.Player"/> for the settings in this new game player.
        /// </summary>
        /// <returns><see cref="Lokeiv.Games.Ludo.Model.Player"/></returns>
        public Lokeiv.Games.Ludo.Model.Player GetPlayer()
        {
            if(this.PlayerType == NewGamePlayerType.NoPlayer)
                return null;

            var playerType = this.PlayerType == NewGamePlayerType.Computer ? Lokeiv.Games.Ludo.Model.PlayerType.Computer : Lokeiv.Games.Ludo.Model.PlayerType.Human;
            
            return new Lokeiv.Games.Ludo.Model.Player(this.PlayerName, playerType, this.PlayerColor);
        }

        #endregion  // Other methods

        #region Commands
        private void OnChangePlayerType()
        {
            this.PlayerType = this.GetNewPlayerType();
        }
        #endregion  // Commands

        #endregion  // Methods
        
        #region INotifyPropertyChanged
        /// <summary>
        /// Event to notify the view when a property has changed.
        /// </summary>
        public event PropertyChangedEventHandler PropertyChanged;        

        /// <summary>
        /// Fires the <see cref="PropertyChanged"/> event for the given <paramref name="propertyName"/>.
        /// </summary>
        /// <param name="propertyName">Name of property that has changed.</param>
        private void NotifyPropertyChanged(String propertyName)
        {
            PropertyChangedEventHandler handler = PropertyChanged;
            if (null != handler)
            {
                handler(this, new PropertyChangedEventArgs(propertyName));
            }
        }
        #endregion  // INotifyPropertyChanged

        #region Structures and classes

        /// <summary>
        /// Enum used as value for the selection of players in a new game.
        /// </summary>
        public enum NewGamePlayerType
        {
            Computer,
            Human,
            NoPlayer
        }

        #endregion  // Structures and classes

        #region IXmlSerializable
        public System.Xml.Schema.XmlSchema GetSchema()
        {
            return null;
        }

        public void ReadXml(System.Xml.XmlReader reader)
        {
            reader.MoveToContent();
            string type = reader.GetAttribute("Type");
            if (!string.IsNullOrEmpty(type)) this.PlayerType = (NewGamePlayerType)Enum.Parse(typeof(NewGamePlayerType), type, true);
            string color = reader.GetAttribute("Color");
            if (!string.IsNullOrEmpty(color)) this.PlayerColor = (Lokeiv.Games.Ludo.Model.Color)Enum.Parse(typeof(Lokeiv.Games.Ludo.Model.Color), color, true);
        }

        public void WriteXml(System.Xml.XmlWriter writer)
        {
            writer.WriteAttributeString("Type", this.PlayerType.ToString());
            writer.WriteAttributeString("Color", this.PlayerColor.ToString());            
        }
        #endregion
    }
}
