﻿using System;
using System.ComponentModel;
using System.Windows.Input;
using System.Xml.Serialization;
using Lokeiv.Games.Ludo.Model;
using Ludo.WP7.Client.Utils;

namespace Ludo.WP7.Client.ViewModels
{
    public class NewGameViewModel : INotifyPropertyChanged, IXmlSerializable
    {

        #region Events

        /// <summary>
        /// Notifies the view to navigate to the main view.
        /// </summary>
        public EventHandler<EventArgs> NavigateToPlay;

        /// <summary>
        /// Notifies the view to navigate to the settings view.
        /// </summary>
        public EventHandler<EventArgs> NavigateToSettings;

        #endregion  // Events

        #region Properties

        private MainViewModel _mainViewModel;
        [XmlIgnore]
        public MainViewModel MainViewModel
        {
            get { return _mainViewModel; }
            set
            {
                if (_mainViewModel != value)
                {
                    _mainViewModel = value;
                    NotifyPropertyChanged("MainViewModel");
                }
            }
        }
        
        private NewGamePlayerViewModel _redPlayer;
        public NewGamePlayerViewModel RedPlayer
        {
            get { return _redPlayer; }
            set
            {
                if (_redPlayer != value)
                {
                    _redPlayer = value;
                    NotifyPropertyChanged("RedPlayer");
                }
            }
        }

        private NewGamePlayerViewModel _yellowPlayer;
        public NewGamePlayerViewModel YellowPlayer
        {
            get { return _yellowPlayer; }
            set
            {
                if (_yellowPlayer != value)
                {
                    _yellowPlayer = value;
                    NotifyPropertyChanged("YellowPlayer");
                }
            }
        }

        private NewGamePlayerViewModel _greenPlayer;
        public NewGamePlayerViewModel GreenPlayer
        {
            get { return _greenPlayer; }
            set
            {
                if (_greenPlayer != value)
                {
                    _greenPlayer = value;
                    NotifyPropertyChanged("GreenPlayer");
                }
            }
        }

        private NewGamePlayerViewModel _bluePlayer;
        public NewGamePlayerViewModel BluePlayer
        {
            get { return _bluePlayer; }
            set
            {
                if (_bluePlayer != value)
                {
                    _bluePlayer = value;
                    NotifyPropertyChanged("BluePlayer");
                }
            }
        }

        private SettingsViewModel _settingsViewModel;
        public SettingsViewModel SettingsViewModel
        {
            get { return _settingsViewModel; }
            set
            {
                if (_settingsViewModel != value)
                {
                    _settingsViewModel = value;
                    NotifyPropertyChanged("SettingsViewModel");
                }
            }
        }

        private int _numberOfStepsBackToGame;
        public int NumberOfStepsBackToGame
        {
            get { return _numberOfStepsBackToGame; }
            set
            {
                if (_numberOfStepsBackToGame != value)
                {
                    _numberOfStepsBackToGame = value;
                    NotifyPropertyChanged("NumberOfStepsBackToGame");
                }
            }
        }

        [XmlIgnore]
        public ICommand StartNewGame { get; private set; }

        [XmlIgnore]
        public ICommand OpenSettings { get; private set; }

        #endregion  // Properties

        #region Constructors

        /// <summary>
        /// Constructor.
        /// </summary>
        public NewGameViewModel(MainViewModel mainViewModel, PlayerViewModel redPlayer, PlayerViewModel yellowPlayer, PlayerViewModel greenPlayer, PlayerViewModel bluePlayer) :
            this(mainViewModel, redPlayer.Player, yellowPlayer.Player, greenPlayer.Player, bluePlayer.Player)
        {
        }

        /// <summary>
        /// Constructor.
        /// </summary>
        public NewGameViewModel(MainViewModel mainViewModel, Player redPlayer, Player yellowPlayer, Player greenPlayer, Player bluePlayer)
        {
            MainViewModel = mainViewModel;
            SettingsViewModel = new ViewModels.SettingsViewModel();
            RedPlayer = new NewGamePlayerViewModel(redPlayer);
            YellowPlayer = new NewGamePlayerViewModel(yellowPlayer);
            GreenPlayer = new NewGamePlayerViewModel(greenPlayer);
            BluePlayer = new NewGamePlayerViewModel(bluePlayer);
            StartNewGame = new RelayCommand(OnStartNewGame);
            OpenSettings = new RelayCommand(OnOpenSettings);
        }

        #endregion  // Constructors

        #region Methods

        #region Other methods
        public Player GetRedPlayer() { return GetPlayer(Lokeiv.Games.Ludo.Model.Color.Red); }
        public Player GetYellowPlayer() { return GetPlayer(Lokeiv.Games.Ludo.Model.Color.Yellow); }
        public Player GetGreenPlayer() { return GetPlayer(Lokeiv.Games.Ludo.Model.Color.Green); }
        public Player GetBluePlayer() { return GetPlayer(Lokeiv.Games.Ludo.Model.Color.Blue); }


        public Player GetPlayer(Lokeiv.Games.Ludo.Model.Color color)
        {
            switch (color)
            {
                case Lokeiv.Games.Ludo.Model.Color.Red:
                    return RedPlayer.GetPlayer();
                case Lokeiv.Games.Ludo.Model.Color.Yellow:
                    return YellowPlayer.GetPlayer();
                case Lokeiv.Games.Ludo.Model.Color.Green:
                    return GreenPlayer.GetPlayer();
                case Lokeiv.Games.Ludo.Model.Color.Blue:
                    return BluePlayer.GetPlayer();
            }

            return null;
        }
        #endregion  // Other methods

        #region Commands

        /// <summary>
        /// Navigates to the play view to start a new game.
        /// </summary>
        private void OnStartNewGame()
        {
            if (this.NavigateToPlay != null)
            {
                NavigateToPlay(this, EventArgs.Empty);
                this.MainViewModel.StartNewGame.Execute(null);
            }                
        }

        /// <summary>
        /// Navigates to the settings view.
        /// </summary>
        private void OnOpenSettings()
        {
            if (this.NavigateToSettings != null)
            {
                NavigateToSettings(this, EventArgs.Empty);
            }
        }

        #endregion  // Commands

        #endregion  // Methods

        #region INotifyPropertyChanged
        /// <summary>
        /// Event to notify the view when a property has changed.
        /// </summary>
        public event PropertyChangedEventHandler PropertyChanged;

        /// <summary>
        /// Fires the <see cref="PropertyChanged"/> event for the given <paramref name="propertyName"/>.
        /// </summary>
        /// <param name="propertyName">Name of property that has changed.</param>
        private void NotifyPropertyChanged(String propertyName)
        {
            PropertyChangedEventHandler handler = PropertyChanged;
            if (null != handler)
            {
                handler(this, new PropertyChangedEventArgs(propertyName));
            }
        }
        #endregion  // INotifyPropertyChanged

        #region IXmlSerializable
        public System.Xml.Schema.XmlSchema GetSchema()
        {
            return null;
        }

        public void ReadXml(System.Xml.XmlReader reader)
        {
            while (reader.Read())
            {
                if (reader.NodeType == System.Xml.XmlNodeType.Element)
                {
                    switch(reader.Name.ToLower())
                    {
                        case "redplayer":
                            this.RedPlayer.ReadXml(reader.ReadSubtree());
                            reader.Skip();
                            break;
                        case "yellowplayer":
                            this.YellowPlayer.ReadXml(reader.ReadSubtree());
                            reader.Skip();
                            break;
                        case "greenplayer":
                            this.GreenPlayer.ReadXml(reader.ReadSubtree());
                            reader.Skip();
                            break;
                        case "blueplayer":
                            this.BluePlayer.ReadXml(reader.ReadSubtree());
                            reader.Skip();
                            break;
                    }
                }
            }
        }

        public void WriteXml(System.Xml.XmlWriter writer)
        {
            // Write RedPlayer
            writer.WriteStartElement("RedPlayer");
            if (this.RedPlayer != null) this.RedPlayer.WriteXml(writer);
            writer.WriteEndElement();

            // Write YellowPlayer
            writer.WriteStartElement("YellowPlayer");
            if (this.YellowPlayer != null) this.YellowPlayer.WriteXml(writer);
            writer.WriteEndElement();

            // Write GreenPlayer
            writer.WriteStartElement("GreenPlayer");
            if (this.GreenPlayer != null) this.GreenPlayer.WriteXml(writer);
            writer.WriteEndElement();

            // Write BluePlayer
            writer.WriteStartElement("BluePlayer");
            if (this.BluePlayer != null) this.BluePlayer.WriteXml(writer);
            writer.WriteEndElement();
        }
        #endregion
    }
}
