﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Windows;
using System.Windows.Threading;
using System.Xml.Serialization;
using Lokeiv.Games.Ludo.Model;
using Ludo.WP7.Model;

namespace Ludo.WP7.Client.ViewModels
{
    public class PlayerViewModel : INotifyPropertyChanged, IXmlSerializable
    {
        #region Properties
        private Dispatcher _dispatcher;

        private MainViewModel _mainViewModel;
        private MainViewModel MainViewModel 
        { 
            get 
            { 
                return _mainViewModel;
            }            
            set
            {
                if (_mainViewModel != value)
                {
                    _mainViewModel = value;
                }
            }
        }

        private Player _player = null;
        public Player Player
        {
            get
            {
                return _player;
            }
            set
            {
                if (_player != value)
                {
                    if (_player != null)
                    {
                        _player.ProgressChanged -= Player_ProgressChanged;
                    }

                    _player = value;

                    if (_player != null)
                    {
                        _player.ProgressChanged += Player_ProgressChanged;

                        switch(_player.CurrentAction)
                        {
                            case Lokeiv.Games.Ludo.Action.RollDie:
                                this.PlayerAction = Model.PlayerAction.RollDie;
                                break;
                            case Lokeiv.Games.Ludo.Action.MovePiece:
                                this.PlayerAction = Model.PlayerAction.MovePiece;
                                break;
                        }

                        if (this._player.HasWon)
                            this.PlayerAction = Model.PlayerAction.Finished;

                        this.PlayerName = _player.Name;
                    }

                    NotifyPropertyChanged("Player");
                    NotifyPropertyChanged("PlayerColor");
                    NotifyPropertyChanged("PlayerType");
                    NotifyPropertyChanged("PlayerTypeImageSource");
                    NotifyPropertyChanged("PlayerProgress");
                    NotifyPropertyChanged("PlayerMovesLeft");
                    NotifyPropertyChanged("PlayerName");
                    NotifyPropertyChanged("Visibility");
                }
            }
        }

        public Lokeiv.Games.Ludo.Model.Color PlayerColor 
        { 
            get
            {
                if (_player == null) return Lokeiv.Games.Ludo.Model.Color.White;
                return _player.Color;
            }            
        }

        public PlayerType? PlayerType 
        {
            get 
            {
                if (_player == null) return null;
                return _player.Type;
            } 
        }

        public Visibility Visibility
        {
            get { return PlayerType.HasValue ? System.Windows.Visibility.Visible : System.Windows.Visibility.Collapsed; }
        }

        public string PlayerTypeImageSource
        {
            get
            {                
                string imageSource = string.Empty;

                if (PlayerType.HasValue)
                {
                    switch (PlayerType.Value)
                    {
                        case Lokeiv.Games.Ludo.Model.PlayerType.Computer:
                            imageSource = string.Format(@"\Images\Phone{0}.png", PlayerColor.ToString());
                            break;
                        case Lokeiv.Games.Ludo.Model.PlayerType.Human:
                            imageSource = string.Format(@"\Images\Human{0}.png", PlayerColor.ToString());
                            break;
                    }
                }

                return imageSource;
            }
        }

        public int PlayerMovesLeft
        {
            get
            {
                if (_player == null) return 0;
                return _player.NumberOfMovesLeft;
            }
        }

        public double PlayerProgress
        {
            get
            {
                if (_player == null) return 0;
                return _player.Progress;
            }
        }

        private PlayerAction _playerAction = PlayerAction.Nothing;
        public PlayerAction PlayerAction 
        { 
            get 
            { 
                return _playerAction; 
            } 
            set 
            {
                if (_playerAction != value)
                {
                    _playerAction = value;
                    NotifyPropertyChanged("PlayerAction");
                    NotifyPropertyChanged("PlayerActionImageSource");
                }
            } 
        }

        public string PlayerActionImageSource
        {
            get
            {
                string imageSource = string.Empty;
                
                switch (PlayerAction)
                {
                    case Ludo.WP7.Model.PlayerAction.RollDie:
                        imageSource = @"\Images\Roll.png";
                        break;
                    case Ludo.WP7.Model.PlayerAction.MovePiece:
                        imageSource = @"\Images\Move.png";
                        break;
                    case Model.PlayerAction.Finished:
                        imageSource = @"\Images\Finish.png";
                        break;
                }

                return imageSource;
            }
        }

        public string PlayerName
        {
            get
            {
                return this.Player.Name;
            }
            set
            {
                if (this.Player != null)
                {
                    if (this.Player.Name != value)
                    {
                        this.Player.Name = value;
                        NotifyPropertyChanged("PlayerName");
                    }
                }                
            }
        }
                
        #endregion  // Properties

        #region Constructors

        /// <summary>
        /// Constructor
        /// </summary>
        public PlayerViewModel() : this(new MainViewModel(), new Player("Red player", Lokeiv.Games.Ludo.Model.PlayerType.Computer, Lokeiv.Games.Ludo.Model.Color.Red))
        {

        }

        /// <summary>
        /// Constructor
        /// </summary>
        /// <param name="mainViewModel"><see cref="MainViewModel"/> which this view model relates to.</param>
        /// <param name="playerColor"><see cref="Lokeiv.Games.Ludo.Model.Color"/> for this player.</param>
        public PlayerViewModel(MainViewModel mainViewModel, Player player)
        {
            this._dispatcher = Deployment.Current.Dispatcher;
            this.MainViewModel = mainViewModel;
            this.Player = player;
        }

        #endregion  // Constructors

        #region Event handlers

        /// <summary>
        /// Handles the <see cref="Player.ProgressChanged"/> event to notify about property changes to update the view.
        /// </summary>
        private void Player_ProgressChanged(object sender, EventArgs e)
        {
            if (this._dispatcher.CheckAccess())
            {
                NotifyPropertyChanged("PlayerProgress");
                NotifyPropertyChanged("PlayerMovesLeft");
            }
            else
            {
                _dispatcher.BeginInvoke(() =>
                {
                    NotifyPropertyChanged("PlayerProgress");
                    NotifyPropertyChanged("PlayerMovesLeft");
                });
            }            
        }

        #endregion  // Event handlers

        #region INotifyPropertyChanged
        /// <summary>
        /// Event to notify the view when a property has changed.
        /// </summary>
        public event PropertyChangedEventHandler PropertyChanged;

        /// <summary>
        /// Fires the <see cref="PropertyChanged"/> event for the given <paramref name="propertyName"/>.
        /// </summary>
        /// <param name="propertyName">Name of property that has changed.</param>
        private void NotifyPropertyChanged(String propertyName)
        {
            PropertyChangedEventHandler handler = PropertyChanged;
            if (null != handler)
            {
                handler(this, new PropertyChangedEventArgs(propertyName));
            }
        }
        #endregion  // INotifyPropertyChanged

        #region IXmlSerializable
        public System.Xml.Schema.XmlSchema GetSchema()
        {
            return null;
        }

        public void ReadXml(System.Xml.XmlReader reader)
        {

        }

        public void WriteXml(System.Xml.XmlWriter writer)
        {
            // Write reference to the player in game model
            Guid playerId = Guid.Empty;
            if (this.Player != null) playerId = this.Player.PlayerId;
            writer.WriteAttributeString("PlayerId", playerId.ToString());

            // Write PlayerAction
            writer.WriteAttributeString("PlayerAction", this.PlayerAction.ToString());
        }
        #endregion
    }

    public class PlayerViewModelCollection : List<PlayerViewModel>
    {
        public PlayerViewModel this[int index]
        {
            get
            {
                return base[index];
            }
        }

        public PlayerViewModel this[Lokeiv.Games.Ludo.Model.Color color]
        {
            get
            {
                foreach (PlayerViewModel p in this)
                    if (p.PlayerColor == color)
                        return p;

                return null;
            }				
        }
    }
}

