﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Runtime.Serialization;
using System.ServiceModel;
using System.Text;
using System.Threading;
using Lokeiv.Games.Ludo.Model;

namespace Lokeiv.Games.Ludo.Service
{
    [ServiceBehavior(InstanceContextMode = InstanceContextMode.Single, ConcurrencyMode = ConcurrencyMode.Reentrant)]
    public class LudoService : ILudoService
    {        
        private IStateProvider _stateProvider;
        private readonly Dictionary<string, ILudoServiceCallback> clients = new Dictionary<string, ILudoServiceCallback>();

        public LudoService()
        {
            this._stateProvider = StateProviderFactory.CreateStateProvider();
        }

        #region ILudoService
        public Table CreateTable(string name)
        {
            // Todo: Ensure the name is unique
            var table = new Table { Name = name };
            this.GetGameRoom().Tables.Add(table);
            this.NotifyClientTableCreated(table);
            return table;
        }

        public List<Client> GetClients()
        {
            return this._stateProvider.GetGameRoom().Clients;
        }

        public List<Table> GetTables()
        {
            return this._stateProvider.GetGameRoom().Tables;
        }

        public GameRoom GetGameRoom()
        {
            return this._stateProvider.GetGameRoom();
        }

        public Table RefreshTable(string tableName)
        {
            foreach (Table table in this.GetGameRoom().Tables)
            {
                if (table.Name == tableName)
                    return table;
            }

            return null;
        }

        public Client EnterWithNickName(string nickName)
        {
            ILudoServiceCallback callback = OperationContext.Current.GetCallbackChannel<ILudoServiceCallback>();

            if (callback != null)
            {
                lock (clients)
                {
                    if (!clients.ContainsKey(nickName))
                    {
                        clients.Add(nickName, callback);
                    }
                }
            }
            
            var client = new Client { NickName = nickName };
            this.GetGameRoom().Clients.Add(client);
            this.NotifyClientEnteredGameRoom(nickName);
            return client;
        }

        public void LeaveGameRoom(string nickName)
        {
            lock (clients)
            {
                if (clients.ContainsKey(nickName))
                {
                    clients.Remove(nickName);
                }
            }

            this.NotifyClientLeftGameRoom(nickName);
        }

        public void SitDown(string nickName, string tableName, Color color)
        {
            Table table = this._stateProvider.GetTable(tableName);
            Client client = this._stateProvider.GetClient(nickName);

            if (table != null && client != null)
            {
                switch (color)
                {
                    case Color.Red:
                        if (table.RedPlayer == null) table.RedPlayer = client;
                        break;
                    case Color.Yellow:
                        if (table.YellowPlayer == null) table.YellowPlayer = client;
                        break;
                    case Color.Green:
                        if (table.GreenPlayer == null) table.GreenPlayer = client;
                        break;
                    case Color.Blue:
                        if (table.BluePlayer == null) table.BluePlayer = client;
                        break;
                }

                // If there is no current operator, set this client as operator.
                if (table.Operator == null) table.Operator = client;
                
                this.NotifyClientSatDown(table);
            }
                
        }

        public void RaiseUp(string nickName, string tableName)
        {
            Table table = this._stateProvider.GetTable(tableName);
            Client client = this._stateProvider.GetClient(nickName);

            if (table != null && client != null)
            {
                if (table.RedPlayer != null && table.RedPlayer.NickName == client.NickName) table.RedPlayer = null;
                else if (table.YellowPlayer != null && table.YellowPlayer.NickName == client.NickName) table.YellowPlayer = null;
                else if (table.GreenPlayer != null && table.GreenPlayer.NickName == client.NickName) table.GreenPlayer = null;
                else if (table.BluePlayer != null && table.BluePlayer.NickName == client.NickName) table.BluePlayer = null;

                this.NotifyClientRaisedUp(table);
            }
        }

        public void EnterTable(string nickName, string tableName)
        {
            Table table = this._stateProvider.GetTable(tableName);
            Client client = this._stateProvider.GetClient(nickName);

            if (table != null && client != null)
            {
                // Clean up if the same client is entering the same table without leaving it.
                this.LeaveTable(nickName, tableName);
                table.Watchers.Add(client);

                // The first client becomes the table operator.
                if (table.Operator == null) table.Operator = client;

                this.NotifyClientEnteredTable(table);
            }
        }

        public void LeaveTable(string nickName, string tableName)
        {
            Table table = this._stateProvider.GetTable(tableName);
            Client client = this._stateProvider.GetClient(nickName);

            if (table != null && client != null)
            {
                if (table.RedPlayer != null && string.Equals(table.RedPlayer.NickName, client.NickName)) table.RedPlayer = null;
                if (table.YellowPlayer != null && string.Equals(table.YellowPlayer.NickName, client.NickName)) table.YellowPlayer = null;
                if (table.GreenPlayer != null && string.Equals(table.GreenPlayer.NickName, client.NickName)) table.GreenPlayer = null;
                if (table.BluePlayer != null && string.Equals(table.BluePlayer.NickName, client.NickName)) table.BluePlayer = null;

                foreach (Client watcher in table.Watchers)
                {
                    if (watcher.NickName == client.NickName)
                    {
                        table.Watchers.Remove(watcher);
                        break;
                    }
                }

                // Change operator if this client is the operator
                if (table.Operator != null && string.Equals(table.Operator.NickName, client.NickName))
                {
                    if (table.RedPlayer != null)
                        table.Operator = table.RedPlayer;
                    else if (table.YellowPlayer != null)
                        table.Operator = table.YellowPlayer;
                    else if (table.GreenPlayer != null)
                        table.Operator = table.GreenPlayer;
                    else if (table.BluePlayer != null)
                        table.Operator = table.BluePlayer;
                    else
                        table.Operator = null;
                }

                this.NotifyClientLeftTable(table);
            }
        }

        public void SendMessageToGameRoom(string nickName, string message)
        {
            this.NotifyClientSentMessageToGameRoom(nickName, message);
        }

        public void SendMessageToTable(string nickName, string tableName, string message)
        {
            this.NotifyClientSentMessageToTable(nickName, tableName, message);
        }

        public void KeepConnection()
        {

        }
        #endregion // ILudoService

        #region Notification of clients

        private void NotifyClientEnteredGameRoom(string nickName)
        {
            this.NotifyClients(new ClientNotificationInfo { Operation = ClientNotificationInfo.Operations.ClientEnteredGameRoom, NickName = nickName });
        }

        private void NotifyClientLeftGameRoom(string nickName)
        {
            this.NotifyClients(new ClientNotificationInfo { Operation = ClientNotificationInfo.Operations.ClientLeftGameRoom, NickName = nickName }); 
        }

        private void NotifyClientTableCreated(Table table)
        {
            this.NotifyClients(new ClientNotificationInfo { Operation = ClientNotificationInfo.Operations.TableCreated, Table = table });
        }

        private void NotifyClientEnteredTable(Table table)
        {
            this.NotifyClients(new ClientNotificationInfo { Operation = ClientNotificationInfo.Operations.ClientEnteredTable, Table = table });
        }

        private void NotifyClientLeftTable(Table table)
        {
            this.NotifyClients(new ClientNotificationInfo { Operation = ClientNotificationInfo.Operations.ClientLeftTable, Table = table });
        }

        private void NotifyClientSatDown(Table table)
        {
            this.NotifyClients(new ClientNotificationInfo { Operation = ClientNotificationInfo.Operations.ClientSatDown, Table = table });
        }

        private void NotifyClientRaisedUp(Table table)
        {
            this.NotifyClients(new ClientNotificationInfo { Operation = ClientNotificationInfo.Operations.ClientRaisedUp, Table = table });
        }

        private void NotifyClientSentMessageToGameRoom(string nickName, string message)
        {
            this.NotifyClients(new ClientNotificationInfo { Operation = ClientNotificationInfo.Operations.ClientSentMessageToGameRoom, NickName = nickName, Message = message });
        }

        private void NotifyClientSentMessageToTable(string nickName, string tableName, string message)
        {
            this.NotifyClients(new ClientNotificationInfo { Operation = ClientNotificationInfo.Operations.ClientSentMessageToTable, NickName = nickName, TableName = tableName, Message = message });
        }

        private void NotifyClients(ClientNotificationInfo info)
        {
            ThreadPool.QueueUserWorkItem
            (
                delegate
                {
                    lock (clients)
                    {
                        List<string> disconnectedClients = new List<string>();

                        foreach (KeyValuePair<string, ILudoServiceCallback> client in clients)
                        {
                            try
                            {
                                switch (info.Operation)
                                {
                                    case ClientNotificationInfo.Operations.ClientEnteredGameRoom:
                                        client.Value.ClientEnteredGameRoom(info.NickName);
                                        break;
                                    case ClientNotificationInfo.Operations.ClientLeftGameRoom:
                                        client.Value.ClientLeftGameRoom(info.NickName);
                                        break;
                                    case ClientNotificationInfo.Operations.TableCreated:
                                        client.Value.TableCreated(info.Table);
                                        break;
                                    case ClientNotificationInfo.Operations.ClientEnteredTable:
                                        client.Value.ClientEnteredTable(info.Table);
                                        break;
                                    case ClientNotificationInfo.Operations.ClientLeftTable:
                                        client.Value.ClientLeftTable(info.Table);
                                        break;
                                    case ClientNotificationInfo.Operations.ClientSatDown:
                                        client.Value.ClientSatDown(info.Table);
                                        break;
                                    case ClientNotificationInfo.Operations.ClientRaisedUp:
                                        client.Value.ClientRaisedUp(info.Table);
                                        break;
                                    case ClientNotificationInfo.Operations.ClientSentMessageToGameRoom:
                                        client.Value.ClientSentMessageToGameRoom(info.NickName, info.Message);
                                        break;
                                    case ClientNotificationInfo.Operations.ClientSentMessageToTable:
                                        client.Value.ClientSentMessageToTable(info.NickName, info.TableName, info.Message);
                                        break;
                                }
                            }
                            catch (Exception)
                            {
                                disconnectedClients.Add(client.Key);
                            }
                        }

                        foreach (string nickName in disconnectedClients)
                        {
                            clients.Remove(nickName);
                        }

                        foreach (string nickName in disconnectedClients)
                        {
                            NotifyClientLeftGameRoom(nickName);
                        }
                    }
                }
            );
        }

        #endregion  // Notification of clients
    }
}
