﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Lidgren.Network;
using GameFx.Entities;
using GameFx.Entities.Messages;

namespace GameFx.GameClient
{
    public class LobbyClient
    {
        #region Events

        /// <summary>
        /// Raised when <see cref="IsConnected"/> has changed.
        /// </summary>
        public event EventHandler<EventArgs> ConnectionStatusChanged;

        public event EventHandler<ClientListChangedEventArgs> ClientListChanged;

        public event EventHandler<ChatEventArgs> ChatMessageArrived;

        #endregion  // Events

        #region Properties
        private NetClient _lindgrenClient;
        private Client _client;

        private bool _isConnected;
        /// <summary>
        /// Gets whether the client is connected to the server.
        /// </summary>
        public bool IsConnected 
        {
            get { return this._isConnected; }            
            private set
            {
                if (this._isConnected != value)
                {
                    this._isConnected = value;
                    this.OnConnectionStatusChanged(EventArgs.Empty);
                }
            }
        }

        /// <summary>
        /// Gets the reason why a connect message was regreted.
        /// </summary>
        public string ConnectError { get; private set; }
        #endregion  // Properties

        #region Constructors
        public LobbyClient(Client client)
        {
            this._client = client;            
        }
        #endregion  // Constructors

        #region Methods

        #region Connection
        public void Connect()
        {
            // Configure NetClient
            NetPeerConfiguration config = new NetPeerConfiguration("Lobby");
            config.AutoFlushSendQueue = true;
            this._lindgrenClient = new NetClient(config);
            string host = Properties.Settings.Default.LobbyServiceHost;
            int port = Properties.Settings.Default.LobbyServicePort;

            // Setup callback channel
            this._lindgrenClient.RegisterReceivedCallback(OnIncomingMessage);

            // Create ConnectMessage and connect to the server
            ConnectMessage connectMessage = new ConnectMessage(this._client);
            this._lindgrenClient.Start();
            this._lindgrenClient.Connect(host, port, this.SerializeMessage(connectMessage));
        }

        public void Disconnect()
        {
            this._lindgrenClient.Shutdown("bye");
        }
        #endregion  // Connection

        #region Chat
        public void Say(string text)
        {
            ChatMessage chatMessage = new ChatMessage(this._client.Token, text);
            this._lindgrenClient.SendMessage(this.SerializeMessage(chatMessage), NetDeliveryMethod.ReliableOrdered);
        }
        #endregion  // Chat

        #region Outgoing message handling
        /// <summary>
        /// Writes the message content to a <see cref="NetOutgoingMessage"/> sent from this client.
        /// </summary>
        /// <param name="message">Message to be serialized.</param>
        /// <returns><see cref="NetOutgoingMessage"/></returns>
        private NetOutgoingMessage SerializeMessage(MessageBase message)
        {
            return message.ToNetOutgoingMessage(this._lindgrenClient);
        }
        #endregion

        #region Incoming message handling
        private void OnIncomingMessage(object peer)
        {
            NetIncomingMessage incomingMessage;

            while ((incomingMessage = this._lindgrenClient.ReadMessage()) != null)
            {
                // handle incoming message
                switch (incomingMessage.MessageType)
                {
                    case NetIncomingMessageType.DebugMessage:
                    case NetIncomingMessageType.ErrorMessage:
                    case NetIncomingMessageType.WarningMessage:
                    case NetIncomingMessageType.VerboseDebugMessage:
                        string text = incomingMessage.ReadString();
                        
                        break;
                    case NetIncomingMessageType.StatusChanged:
                        NetConnectionStatus status = (NetConnectionStatus)incomingMessage.ReadByte();
                        this.ConnectError = incomingMessage.ReadString();

                        if (status == NetConnectionStatus.Connected)
                            this.IsConnected = true;
                        else
                            this.IsConnected = false;
                        
                        break;
                    case NetIncomingMessageType.Data:
                        MessageBase message = MessageBase.FromNetIncomingMessage(incomingMessage);

                        if (message is ClientListMessage)
                        {
                            ClientListMessage clientListMessage = (ClientListMessage)message;
                            this.OnClientListChanged(new ClientListChangedEventArgs(clientListMessage.Clients, null));
                        }
                        else if (message is ChatMessage)
                        {
                            ChatMessage chatMessage = (ChatMessage)message;
                            this.OnChatMessageArrived(new ChatEventArgs(chatMessage.ClientToken.Id, chatMessage.Text));
                        }

                        break;
                    default:                        
                        //Logger.Log("Unhandled type: " + netInMessage.MessageType + " " + netInMessage.LengthBytes + " bytes");
                        break;
                }
            }
        }
        #endregion  // Incoming message handling

        #region Event raising
        protected virtual void OnConnectionStatusChanged(EventArgs e)
        {
            if (this.ConnectionStatusChanged != null)
                this.ConnectionStatusChanged(this, e);
        }

        protected virtual void OnClientListChanged(ClientListChangedEventArgs e)
        {
            if (this.ClientListChanged != null)
                this.ClientListChanged(this, e);
        }

        protected virtual void OnChatMessageArrived(ChatEventArgs e)
        {
            if (this.ChatMessageArrived != null)
                this.ChatMessageArrived(this, e);
        }
        #endregion  // Event raising

        #endregion  // Methods
    }
}
