﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using GameFx.GameClient;
using System.Threading;
using System.ComponentModel;
using GameFx.Entities;
using System.Windows.Input;
using System.Collections.ObjectModel;

namespace GameFx.ViewModels
{
    public class LobbyViewModel : INotifyPropertyChanged
    {
        #region Properties
        private SynchronizationContext _syncContext;
        private Client _client;
        private LobbyClient _lobbyClient;
        private bool _isEntering, _hasEntered;

        private string _nickName;
        public string NickName
        {
            get { return this._nickName; }
            set
            {
                if (this._nickName != value)
                {
                    this._nickName = value;
                    this.NotifyPropertyChanged("NickName");
#if WINDOWS_PHONE || ANDROID || WINDOWS_FORMS
                    ((RelayCommand)this.Enter).RaiseCanExecuteChanged();
#endif
                }
            }
        }

        private ObservableCollection<Client> _clients;
        public ObservableCollection<Client> Clients
        {
            get { return this._clients; }
            set
            {
                if (this._clients != value)
                {
                    this._clients = value;
                    this.NotifyPropertyChanged("Clients");
                }
            }
        }

        private string _messages;
        public string Messages
        {
            get { return this._messages; }
            set
            {
                if (this._messages != value)
                {
                    this._messages = value;
                    this.NotifyPropertyChanged("Messages");
                }
            }
        }

        private string _chatMessage;
        public string ChatMessage
        {
            get { return this._chatMessage; }
            set
            {
                if (this._chatMessage != value)
                {
                    this._chatMessage = value;
                    this.NotifyPropertyChanged("ChatMessage");
                }
            }
        }

        public ICommand Enter { get; private set; }
        private bool CanEnter
        {
            get
            {
                return !(this._isEntering || this._hasEntered) && !string.IsNullOrEmpty(this._nickName);
            }
        }

        public ICommand Chat { get; private set; }
        private bool CanChat
        {
            get
            {
                return !string.IsNullOrEmpty(this.ChatMessage);
            }
        }
        #endregion  // Properties

        #region Constructors
        public LobbyViewModel(bool isDesignMode)
        {
            if (!isDesignMode)
            {
#if WINDOWS_PHONE || ANDROID || WINDOWS_FORMS
                this.Enter = new RelayCommand(param => this.OnEnter(), param => this.CanEnter);
                this.Chat = new RelayCommand(param => this.OnChat(), param => this.CanChat);
#else
                this.Enter = new RoutedCommand();
                this.Chat = new RoutedCommand();
#endif
                this._syncContext = SynchronizationContext.Current;
                this.Clients = new ObservableCollection<Client>();
            }
        }
        #endregion  // Constructors

        #region Methods

        #region Initialization

        /// <summary>
        /// Creates a <see cref="LobbyClient"/>, subscribes to its events and connects to the lobby server.
        /// </summary>
        private void CreateLobbyClientAndConnect()
        {
            this._lobbyClient = new LobbyClient(this._client);
            this.AppendMessage("Trying to connect...");
            this._lobbyClient.ConnectionStatusChanged += new EventHandler<EventArgs>(LobbyClient_ConnectionStatusChanged);
            this._lobbyClient.ChatMessageArrived += new EventHandler<ChatEventArgs>(LobbyClient_ChatMessageArrived);
            this._lobbyClient.ClientListChanged += new EventHandler<ClientListChangedEventArgs>(LobbyClient_ClientListChanged);
            this._lobbyClient.Connect();
        }
        #endregion  // Initialization

        #region Other methods
        /// <summary>
        /// Gets the <see cref="Client"/> with the given <paramref name="clientId"/>. Throws exception if the client is not found.
        /// </summary>
        /// <param name="clientId"><see cref="Client.Token.Id"/></param>
        /// <returns><see cref="Client"/></returns>
        private Client GetClient(Guid clientId)
        {
            return this.Clients.Where(c => c.Token.Id == clientId).First();
        }
        #endregion  // Other methods

        #region Event handlers

        #region LobbyClient event handlers
        void LobbyClient_ChatMessageArrived(object sender, ChatEventArgs e)
        {
            this.AppendMessage(this.GetClient(e.ClientId).NickName + " says: " + e.Message);
        }

        void LobbyClient_ConnectionStatusChanged(object sender, EventArgs e)
        {
            this.AppendMessage("Connected to server");
        }

        void LobbyClient_ClientListChanged(object sender, ClientListChangedEventArgs e)
        {
            foreach (Client clientEntered in e.ClientsEntered)
                if (!this.Clients.Contains(clientEntered))
                    this.Clients.Add(clientEntered);

            foreach (Client clientLeft in e.ClientsLeft)
                if (this.Clients.Contains(clientLeft))
                    this.Clients.Remove(clientLeft);
        }
        #endregion  // LobbyClient event handlers

        #endregion  // Event handlers

        #region Commands
        private void OnEnter()
        {
            try
            {
                this._isEntering = true;
#if WINDOWS_PHONE || ANDROID || WINDOWS_FORMS
                ((RelayCommand)this.Enter).RaiseCanExecuteChanged();
#endif
                this._client = new Client(this.NickName);                
                this.CreateLobbyClientAndConnect();
            }
            catch (Exception)
            {
                this._hasEntered = false;
                this._isEntering = false;
#if WINDOWS_PHONE || ANDROID || WINDOWS_FORMS
                ((RelayCommand)this.Enter).RaiseCanExecuteChanged();
#endif
            }
        }

        private void OnChat()
        {
            if (!string.IsNullOrEmpty(this.ChatMessage))
            {
                this._lobbyClient.Say(this.ChatMessage);
                this.ChatMessage = null;
            }
        }
        #endregion  // Commands

        #endregion  // Methods

        #region INotifyPropertyChanged
        public event PropertyChangedEventHandler PropertyChanged;

        private void NotifyPropertyChanged(string propertyName)
        {
            if (this.PropertyChanged != null)
                this.PropertyChanged(this, new PropertyChangedEventArgs(propertyName));
        }
        #endregion

        #region ILobbyViewModel
        public void AppendMessage(string text)
        {
            this.Messages += text + "\r\n";
        }

        public void ClientEntered()
        {
            this._hasEntered = true;
            this._isEntering = false;
#if WINDOWS_PHONE || ANDROID || WINDOWS_FORMS
            ((RelayCommand)this.Enter).RaiseCanExecuteChanged();
#else
            NotifyPropertyChanged("CanEnter");
#endif
        }
        #endregion  // ILobbyViewModel
    }
}
