﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Lidgren.Network;
using System.Runtime.Serialization;
using System.Xml.Serialization;
using System.IO;
using System.Xml;
using System.Reflection;

namespace GameFx.Entities.Messages
{
    public abstract class MessageBase : IXmlSerializable
    {
        /// <summary>
        /// Gets the type name of the message subtype.
        /// </summary>
        public string MessageType { get; private set; }

        /// <summary>
        /// Constructor that sets the <see cref="MessageType"/> property to the name of the message subtype.
        /// </summary>
        public MessageBase()
        {
            this.MessageType = this.GetType().Name;
        }

        /// <summary>
        /// Converts this message to a <see cref="NetOutgoingMessage"/> created by the <paramref name="netClient"/>.
        /// </summary>
        /// <param name="netClient"><see cref="NetClient"/> that will be sender of the message.</param>
        /// <returns><see cref="NetOutgoingMessage"/> ready to be sent.</returns>
        public NetOutgoingMessage ToNetOutgoingMessage(NetClient netClient)
        {
            NetOutgoingMessage netOutMsg = netClient.CreateMessage();
            this.SerializeIntoOutgoingMessage(netOutMsg);

            return netOutMsg;
        }

        /// <summary>
        /// Converts this message to a <see cref="NetOutgoingMessage"/> created by the <paramref name="netServer"/>.
        /// </summary>
        /// <param name="netServer"><see cref="NetServer"/> that broadcasts the message.</param>
        /// <returns><see cref="NetOutgoingMessage"/> ready to be sent.</returns>
        public NetOutgoingMessage ToNetOutgoingMessage(NetServer netServer)
        {
            NetOutgoingMessage netOutMsg = netServer.CreateMessage();
            this.SerializeIntoOutgoingMessage(netOutMsg);

            return netOutMsg;
        }

        private void SerializeIntoOutgoingMessage(NetOutgoingMessage netOutMsg)
        {
            XmlSerializer serializer = new XmlSerializer(this.GetType());

            using (StringWriter stringWriter = new StringWriter())
            {
                serializer.Serialize(stringWriter, this);
                string data = stringWriter.ToString();
                netOutMsg.Write(data);
            }
        }

        /// <summary>
        /// Creates an instance of the correct type of message and reads its content from the <paramref name="netInMsg"/>.
        /// </summary>
        /// <param name="netInMsg"><see cref="NetIncomingMessage"/> containing message content.</param>
        /// <returns>A deserialized message of a subtype of <see cref="MessageBase"/></returns>
        public static MessageBase FromNetIncomingMessage(NetIncomingMessage netInMsg)
        {
            if (netInMsg.PositionInBytes == netInMsg.LengthBytes)
                return null;

            try
            {
                if (string.IsNullOrEmpty(netInMsg.PeekString()))
                    return null;
            }
            catch (NetException)
            {
                return null;
            }

            string xml = netInMsg.ReadString();
            Type messageType = MessageBase.GetMessageTypeFromXml(xml);
            XmlSerializer serializer = new XmlSerializer(messageType);

            using (StringReader stringReader = new StringReader(xml))
            using (XmlReader xmlReader = XmlReader.Create(stringReader))
            {
                return serializer.Deserialize(xmlReader) as MessageBase;
            }
        }

        private static MessageBase CreateMessageInstanceFromXml(string xml)
        {
            Type messageType = MessageBase.GetMessageTypeFromXml(xml);
            ConstructorInfo constructor = messageType.GetConstructor(BindingFlags.Public | BindingFlags.NonPublic | BindingFlags.Instance, null, new Type[] {}, null);
            
            if (constructor == null)
                throw new Exception(string.Format("Message type {0} does not provide a private or public default constructor.", messageType.Name));
            
            return constructor.Invoke(null) as MessageBase;
        }

        private static Type GetMessageTypeFromXml(string xml)
        {
            string messageTypeName = MessageBase.ReadMessageTypeFromXml(xml);
            string namespaceName = typeof(MessageBase).Namespace;
            Type messageType = typeof(MessageBase).Assembly.GetType(namespaceName + "." + messageTypeName);

            if (messageType == null)
                throw new Exception(string.Format("Message type {0} is not declared in Entities assembly.", messageTypeName));

            return messageType;
        }

        /// <summary>
        /// Reads the message type attribute from <paramref name="xml"/>.
        /// </summary>
        /// <param name="xml">XML containg the message data.</param>
        /// <returns>Message type name.</returns>
        private static string ReadMessageTypeFromXml(string xml)
        {
            using (StringReader stringReader = new StringReader(xml))
            using (XmlReader xmlReader = XmlReader.Create(stringReader))
            {
                xmlReader.MoveToContent();                
                return xmlReader.Name;
            }
        }

        #region IXmlSerializable
        public System.Xml.Schema.XmlSchema GetSchema()
        {
            return null;
        }

        public abstract void ReadXml(XmlReader reader);
        public abstract void WriteXml(XmlWriter writer);
        #endregion  // IXmlSerializable
    }
}
