﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Lidgren.Network;
using System.Threading;
using GameFx.Entities.Messages;
using GameFx.Entities;

namespace GameFx.Server
{
    public class LobbyService
    {
        #region Properties
        private NetServer _server;
        private Thread _serviceThread;
        private ILogger _logger;
        private Dictionary<Guid, Client> _clients;

        /// <summary>
        /// Gets wheter the service thread processing requests is running.
        /// </summary>
        public bool IsRunning 
        { 
            get 
            {
                if (this._serviceThread == null)
                    return false;

                return this._serviceThread.IsAlive;
            } 
        }

        /// <summary>
        /// Gets whether the service is open for connections.
        /// </summary>
        public bool IsOpen
        {
            get
            {
                if (this._server == null)
                    return false;

                return this._server.Status == NetPeerStatus.Starting || this._server.Status == NetPeerStatus.Running;
            }
        }
        #endregion  // Properties

        #region Constructors
        public LobbyService(ILogger logger)
        {
            this._logger = logger;
            this._clients = new Dictionary<Guid, Client>();
        }
        #endregion  // Constructor

        public void StartService()
        {
            NetPeerConfiguration config = new NetPeerConfiguration("Lobby");
            config.MaximumConnections = Properties.Settings.Default.LobbyServiceMaxConnections;
            config.Port = Properties.Settings.Default.LobbyServicePort;
            config.LocalAddress = new System.Net.IPAddress(new byte[] { 192, 168, 0, 3 });
            this._server = new NetServer(config);
            this._server.Configuration.SetMessageTypeEnabled(NetIncomingMessageType.ConnectionApproval, true);
            this._serviceThread = new Thread(new ParameterizedThreadStart(ProcessRequests));
            this._serviceThread.Start();
            this._server.Start();
        }

        public void ShutDownService()
        {
            if (this._server != null)
                if (this._server.Status == NetPeerStatus.Running || this._server.Status == NetPeerStatus.Starting)
                    this._server.Shutdown("shutting down...");

            if (this._serviceThread != null && this._serviceThread.IsAlive)
                this._serviceThread.Abort();

            this._server = null;
            this._serviceThread = null;
        }

        private void ProcessRequests(object parameter)
        {
            while (true)
            {
                NetIncomingMessage incomingMessage;

                while ((incomingMessage = this._server.ReadMessage()) != null)
                {
                    MessageBase message = null;

                    switch (incomingMessage.MessageType)
                    {
                        case NetIncomingMessageType.DebugMessage:
                        case NetIncomingMessageType.ErrorMessage:
                        case NetIncomingMessageType.WarningMessage:
                        case NetIncomingMessageType.VerboseDebugMessage:
                            string text = incomingMessage.ReadString();
                            this.Output(text);

                            break;
                        case NetIncomingMessageType.StatusChanged:
                            NetConnectionStatus status = (NetConnectionStatus)incomingMessage.ReadByte();
                            string reason = incomingMessage.ReadString();
                            this.Output(NetUtility.ToHexString(incomingMessage.SenderConnection.RemoteUniqueIdentifier) + " " + status + ": " + reason);
                            
                            if (incomingMessage.SenderConnection.RemoteHailMessage != null)
                                message = MessageBase.FromNetIncomingMessage(incomingMessage.SenderConnection.RemoteHailMessage);

                            if (message is ConnectMessage)
                            {
                                ConnectMessage connectMessage = (ConnectMessage)message;
                                EnterLeaveMessage enterLeaveMessage = null;

                                if (status == NetConnectionStatus.Connected)
                                {
                                    if (this._clients.ContainsKey(connectMessage.ClientToken.Id))
                                    {
                                        this._clients.Remove(connectMessage.ClientToken.Id);
                                        enterLeaveMessage = new EnterLeaveMessage(connectMessage.Client, EnterLeave.Leave);
                                    }
                                }
                                else if (status == NetConnectionStatus.RespondedAwaitingApproval)
                                {
                                    if (!this._clients.ContainsKey(connectMessage.ClientToken.Id))
                                    {
                                        // Approve the connection
                                        incomingMessage.SenderConnection.Approve();

                                        // Add client to lobby.
                                        this._clients.Add(connectMessage.ClientToken.Id, connectMessage.Client);
                                        enterLeaveMessage = new EnterLeaveMessage(connectMessage.Client, EnterLeave.Enter);

                                        // Include the new client as receiver of the ClientListMessage
                                        List<NetConnection> allClients = this._server.Connections;
                                        allClients.Add(incomingMessage.SenderConnection);

                                        // Send list of existing clients to the one that entered the lobby.
                                        ClientListMessage clientListMessage = new ClientListMessage(this._clients.Values);
                                        NetOutgoingMessage netOutMsg = clientListMessage.ToNetOutgoingMessage(this._server);
                                        this._server.SendMessage(netOutMsg, allClients, NetDeliveryMethod.ReliableOrdered, 0);
                                    }
                                }

                                if (enterLeaveMessage != null)
                                {
                                    // Broadcast to everyone except the client itself that a client has entered or left the lobby.
                                    List<NetConnection> all = this._server.Connections;

                                    if (all.Contains(incomingMessage.SenderConnection))
                                        all.Remove(incomingMessage.SenderConnection);

                                    if (all.Count > 0)
                                    {
                                        NetOutgoingMessage netOutMsg = enterLeaveMessage.ToNetOutgoingMessage(this._server);
                                        this._server.SendMessage(netOutMsg, all, NetDeliveryMethod.ReliableOrdered, 0);
                                    }
                                }
                            }

                            break;
                        case NetIncomingMessageType.Data:
                            message = MessageBase.FromNetIncomingMessage(incomingMessage);

                            if (message is ChatMessage)
                            {
                                ChatMessage chatMessage = (ChatMessage)message;
                                this.Output("Broadcasting '" + chatMessage.Text + "'");
                                List<NetConnection> all = this._server.Connections;

                                if (all.Count > 0)
                                {
                                    NetOutgoingMessage netOutMsg = chatMessage.ToNetOutgoingMessage(this._server);
                                    this._server.SendMessage(netOutMsg, all, NetDeliveryMethod.ReliableOrdered, 0);
                                }
                            }
                            break;
                        default:
                            this.Output("Unhandled type: " + incomingMessage.MessageType + " " + incomingMessage.LengthBytes + " bytes " + incomingMessage.DeliveryMethod + "|" + incomingMessage.SequenceChannel);
                            break;
                    }
                }
            }
        }

        private void Output(string message)
        {
            if (this._logger != null)
                this._logger.Write(message);
        }
    }
}
