﻿using System;
using System.IO;
using System.Xml;
using GameFx.Entities;
using GameFx.Entities.Messages;
using SignalR.Client.Hubs;
using SignalR.Client.Http;
using System.Threading.Tasks;
using System.Threading;
using System.ComponentModel;
using System.Collections.Specialized;
using System.Collections.Generic;

namespace GameFx.GameClient
{
    public class LobbyClient
    {
        #region Events

        /// <summary>
        /// Raised when <see cref="IsConnected"/> has changed.
        /// </summary>
        public event EventHandler<EventArgs> ConnectionStatusChanged;

        public event EventHandler<ClientListChangedEventArgs> ClientListChanged;

        public event EventHandler<ChatEventArgs> ChatMessageArrived;

        #endregion  // Events

        #region Properties
        private Client _client;
        private HubConnection _connection;
        private IHubProxy _lobbyHub;

        private bool _isConnected;
        /// <summary>
        /// Gets whether the client is connected to the server.
        /// </summary>
        public bool IsConnected 
        {
            get { return this._isConnected; }            
            private set
            {
                if (this._isConnected != value)
                {
                    this._isConnected = value;
                    this.OnConnectionStatusChanged(EventArgs.Empty);
                }
            }
        }

        /// <summary>
        /// Gets the reason why a connect message was regreted.
        /// </summary>
        public string ConnectError { get; private set; }
        #endregion  // Properties

        #region Constructors
        public LobbyClient(Client client)
        {
            this._client = client;
            this.CreateLobbyHubAndSubscribeToCallbacks();
        }
        #endregion  // Constructors

        #region Methods

        #region Initialization of lobby hub
        private void CreateLobbyHubAndSubscribeToCallbacks()
        {
            this._connection = new HubConnection(Properties.Settings.Default.LobbyHubUrl);    
            this._lobbyHub = this._connection.CreateProxy("Lobby");

            // Subscribe to all methods the LobbyHub can call            
            this._lobbyHub.On<ChatMessage>("incomingChatMessage", this.OnIncomingChatMessage);

            // Start must be called before the hub can be used
            this._connection.Start().Wait(5000);
        }
        #endregion  // Initialization of lobby hub

        #region Connection
        private Dictionary<object, AsyncOperation> connectUserStateToLifetime = new Dictionary<object,AsyncOperation>();
        public event EventHandler<ConnectCompletedEventArgs> ConnectCompleted;
        public class ConnectCompletedEventArgs : AsyncCompletedEventArgs
        {
            public Client Result { get; private set; }

            public ConnectCompletedEventArgs(Client result, object userState)
                : this (null, false, userState)
            {
                this.Result = result;
            }

            public ConnectCompletedEventArgs(Client result)
                : this(result, null)
            { }

            public ConnectCompletedEventArgs(Exception error, bool cancelled, object userState)
                : base(error, cancelled, userState)
            { }
        }
        public void ConnectAsync(string nickname, object userState)
        {
            AsyncOperation operation = AsyncOperationManager.CreateOperation(userState);
            
            lock (connectUserStateToLifetime)
            {
                if (connectUserStateToLifetime.ContainsKey(userState))
                    throw new ArgumentException("User State parameter must be unique", "userState");

                connectUserStateToLifetime[userState] = operation;
            }

            throw new NotImplementedException();  // Insane much work to implement the async event based pattern!!
        }

        public void Connect()
        {
            this._lobbyHub.Invoke("Join", this._client).Wait();
        }

        public void Disconnect()
        {
            Task disconnect = this._lobbyHub.Invoke("Disconnect", this._client.Token);
        }
        #endregion  // Connection

        #region Chat
        public void Say(string text)
        {
            ChatMessage chatMessage = new ChatMessage(this._client.Token, text);
            this._lobbyHub.Invoke("BroadcastMessage", chatMessage);
        }

        private void OnIncomingChatMessage(ChatMessage message)
        {
            this.OnChatMessageArrived(new ChatEventArgs(message.ClientToken.Id, message.Text));
        }
        #endregion  // Chat

        #region Outgoing message handling
        /// <summary>
        /// Writes the message content to a <see cref="NetOutgoingMessage"/> sent from this client.
        /// </summary>
        /// <param name="message">Message to be serialized.</param>
        /// <returns><see cref="NetOutgoingMessage"/></returns>
        private string SerializeMessage(MessageBase message)
        {                        
            using (MemoryStream stream = new MemoryStream())
            using (XmlWriter writer = XmlWriter.Create(stream))
            {
                message.WriteXml(writer);

                using (StreamReader reader = new StreamReader(stream))
                {
                    return reader.ReadToEnd();
                }
            }
        }
        #endregion

        #region Event raising
        protected virtual void OnConnectionStatusChanged(EventArgs e)
        {
            if (this.ConnectionStatusChanged != null)
                this.ConnectionStatusChanged(this, e);
        }

        protected virtual void OnClientListChanged(ClientListChangedEventArgs e)
        {
            if (this.ClientListChanged != null)
                this.ClientListChanged(this, e);
        }

        protected virtual void OnChatMessageArrived(ChatEventArgs e)
        {
            if (this.ChatMessageArrived != null)
                this.ChatMessageArrived(this, e);
        }
        #endregion  // Event raising

        #endregion  // Methods
    }
}
