﻿using System;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Ink;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using System.Threading;
using System.IO.IsolatedStorage;

namespace GameFx.GameClient.Properties
{
    public class Settings
    {

        #region Singleton pattern
        private static readonly object _instanceLock = new object();
        private static Settings _instance;
        public static Settings Default
        {
            get
            {
                try
                {
                    Monitor.Enter(_instanceLock);

                    if (_instance == null)
                        _instance = new Settings();
                }
                finally
                {
                    Monitor.Exit(_instanceLock);
                }

                return _instance;
            }
        }

        private Settings()
        {
            this._settings = IsolatedStorageSettings.ApplicationSettings;
        }
        #endregion  // Singleton pattern

        #region Properties
        private IsolatedStorageSettings _settings;

        public string LobbyServiceHost
        {
            get { return this.GetValueOrDefault<string>("LobbyServiceHost", "127.0.0.1"); }
            set
            {
                if (this.AddOrUpdateValue("LobbyServiceHost", value))
                    this.Save();
            }
        }

        public int LobbyServicePort
        {
            get { return this.GetValueOrDefault<int>("LobbyServicePort", 14725); }
            set
            {
                if (this.AddOrUpdateValue("LobbyServicePort", value))
                    this.Save();
            }
        }
        #endregion  // Properties

        #region Methods
        /// <summary>
        /// Update a setting value for our application. If the setting does not
        /// exist, then add the setting.
        /// </summary>
        /// <param name="Key"></param>
        /// <param name="value"></param>
        /// <returns></returns>
        public bool AddOrUpdateValue(string Key, Object value)
        {
            bool valueChanged = false;

            // If the key exists
            if (this._settings.Contains(Key))
            {
                // If the value has changed
                if (this._settings[Key] != value)
                {
                    // Store the new value
                    this._settings[Key] = value;
                    valueChanged = true;
                }
            }
            // Otherwise create the key.
            else
            {
                this._settings.Add(Key, value);
                valueChanged = true;
            }
            return valueChanged;
        }

        /// <summary>
        /// Get the current value of the setting, or if it is not found, set the 
        /// setting to the default setting.
        /// </summary>
        /// <typeparam name="T"></typeparam>
        /// <param name="Key"></param>
        /// <param name="defaultValue"></param>
        /// <returns></returns>
        public T GetValueOrDefault<T>(string Key, T defaultValue)
        {
            T value;

            // If the key exists, retrieve the value.
            if (this._settings.Contains(Key))
            {
                value = (T)this._settings[Key];
            }
            // Otherwise, use the default value.
            else
            {
                value = defaultValue;
            }
            return value;
        }

        /// <summary>
        /// Save the settings.
        /// </summary>
        public void Save()
        {
            this._settings.Save();
        }
        #endregion  // Methods
    }
}
