﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Runtime.Serialization;
using System.Xml.Serialization;
using System.IO;
using System.Xml;
using System.Reflection;

namespace GameFx.Entities.Messages
{
    public abstract class MessageBase : IXmlSerializable
    {
        /// <summary>
        /// Gets the type name of the message subtype.
        /// </summary>
        public string MessageType { get; private set; }

        /// <summary>
        /// Constructor that sets the <see cref="MessageType"/> property to the name of the message subtype.
        /// </summary>
        public MessageBase()
        {
            this.MessageType = this.GetType().Name;
        }

        private static MessageBase CreateMessageInstanceFromXml(string xml)
        {
            Type messageType = MessageBase.GetMessageTypeFromXml(xml);
            ConstructorInfo constructor = messageType.GetConstructor(BindingFlags.Public | BindingFlags.NonPublic | BindingFlags.Instance, null, new Type[] {}, null);
            
            if (constructor == null)
                throw new Exception(string.Format("Message type {0} does not provide a private or public default constructor.", messageType.Name));
            
            return constructor.Invoke(null) as MessageBase;
        }

        private static Type GetMessageTypeFromXml(string xml)
        {
            string messageTypeName = MessageBase.ReadMessageTypeFromXml(xml);
            string namespaceName = typeof(MessageBase).Namespace;
            Type messageType = typeof(MessageBase).Assembly.GetType(namespaceName + "." + messageTypeName);

            if (messageType == null)
                throw new Exception(string.Format("Message type {0} is not declared in Entities assembly.", messageTypeName));

            return messageType;
        }

        /// <summary>
        /// Reads the message type attribute from <paramref name="xml"/>.
        /// </summary>
        /// <param name="xml">XML containg the message data.</param>
        /// <returns>Message type name.</returns>
        private static string ReadMessageTypeFromXml(string xml)
        {
            using (StringReader stringReader = new StringReader(xml))
            using (XmlReader xmlReader = XmlReader.Create(stringReader))
            {
                xmlReader.MoveToContent();                
                return xmlReader.Name;
            }
        }

        #region IXmlSerializable
        public System.Xml.Schema.XmlSchema GetSchema()
        {
            return null;
        }

        public abstract void ReadXml(XmlReader reader);
        public abstract void WriteXml(XmlWriter writer);
        #endregion  // IXmlSerializable
    }
}
