using System;
using System.Diagnostics;
using System.Collections.Generic;
using System.Text;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.ComponentModel;
using System.Windows.Forms;
using Ascend;
using Ascend.Resources;
using Ascend.Windows.Forms;

namespace Lokeiv.Games.Die.Forms
{
    public class Die : System.Windows.Forms.Control 
    {
        private Lokeiv.Games.Dice.Die _die = null;        
        private bool _rollDieOnClick = true;
        internal DieRender DieRender;        

        #region Events

        ///// <summary>
        ///// Occurs when the die is rolled.
        ///// </summary>
        //[ResourceDescription("DieRolledDescription"), ResourceCategory("DieCategory"), ResourceDisplayName("DisplayNameDieRolled")]
        //public event EventHandler DieRolled;

        /// <summary>
        /// Occurs when the Border property changes.
        /// </summary>
        [ResourceDescriptionAttribute("BorderChangedDescription"), ResourceCategoryAttribute("PropertyChangedCategory"), ResourceDisplayName("DisplayNameBorderChanged")]
        public event EventHandler BorderChanged;

        /// <summary>
        /// Occurs when the BorderColor property changes.
        /// </summary>
        [ResourceDescriptionAttribute("BorderColorChangedDescription"), ResourceCategoryAttribute("PropertyChangedCategory"), ResourceDisplayName("DisplayNameBorderColorChanged")]
        public event EventHandler BorderColorChanged;

        /// <summary>
        /// Occurs when the GradientLowColor property changes.
        /// </summary>
        [ResourceDescriptionAttribute("GradientLowColorChangedDescription"), ResourceCategoryAttribute("PropertyChangedCategory"), ResourceDisplayName("DisplayNameGradientLowColorChanged")]
        public event EventHandler GradientLowColorChanged;

        /// <summary>
        /// Occurs when the GradientHighColor property changes.
        /// </summary>
        [ResourceDescriptionAttribute("GradientHighColorChangedDescription"), ResourceCategoryAttribute("PropertyChangedCategory"), ResourceDisplayName("DisplayNameGradientHighColorChanged")]
        public event EventHandler GradientHighColorChanged;

        /// <summary>
        /// Occurs when the GradientMode property changes.
        /// </summary>
        [ResourceDescriptionAttribute("GradientModeChangedDescription"), ResourceCategoryAttribute("PropertyChangedCategory"), ResourceDisplayName("DisplayNameGradientModeChanged")]
        public event EventHandler GradientModeChanged;

        #endregion

        #region Public properties

        [Browsable(false), DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        public Lokeiv.Games.Dice.Die DieModel
        {
            get { return _die; }
            set 
            { 
                _die = value;
                this.InitializeRender();  // Connect the DieRender to the new die
                this._die.DieRolled += this.DieModel_DieRolled;
            }
        }

        [Category("Behavior"), Description("Defines if the die should be rolled automatically")]
        public bool RollDieOnClick
        {
            get { return this._rollDieOnClick; }
            set { this._rollDieOnClick = value; }
        }

        [Category("Behavior"), Description("Type of die")]
        public Lokeiv.Games.Dice.DieTypes DieType
        {
            get
            {
                if (this._die == null)
                {                    
                    return Lokeiv.Games.Dice.DieTypes.NumericOneToSix;
                }

                if (this._die.GetType() == typeof(Lokeiv.Games.Dice.StarAndEarthDie))
                {
                    return Lokeiv.Games.Dice.DieTypes.StarAndEarth;
                }
                else
                {
                    return Lokeiv.Games.Dice.DieTypes.NumericOneToSix;
                }
            }

            set
            {
                if (value == Lokeiv.Games.Dice.DieTypes.StarAndEarth)
                {
                    this.DieModel = new Lokeiv.Games.Dice.StarAndEarthDie();
                }
                else
                {
                    this.DieModel = new Lokeiv.Games.Dice.NumericOneToSixDie();
                }
            }
        }

        #endregion

        #region Contructors
        /// <summary>
        /// Initializes a new instance of the Die class with the default die type of NumericOneToSix.
        /// </summary>
        public Die() : base()
        {
            // Create a new die for this control            
            this.DieType = Lokeiv.Games.Dice.DieTypes.NumericOneToSix;

            this.InitializeRender();
            this.BorderColor = new BorderColor(SystemColors.ActiveCaption);

            base.SetStyle(ControlStyles.SupportsTransparentBackColor | ControlStyles.OptimizedDoubleBuffer | ControlStyles.UserPaint | ControlStyles.AllPaintingInWmPaint | ControlStyles.Selectable, true);
            base.BackColor = Color.FromArgb(0, SystemColors.Window);

            this.GradientMode = LinearGradientMode.Horizontal;
            this.GradientHighColor = SystemColors.InactiveCaptionText;
            this.GradientLowColor = SystemColors.GradientActiveCaption;
        }

        /// <summary>
        /// Initializes a new instance of the Die class with the given die type.
        /// </summary>
        /// <param name="dieType"></param>
        public Die(Lokeiv.Games.Dice.DieTypes dieType) : this()
        {             
            // Create a new die of the given type for this control
            this.DieType = dieType;
        }

        #endregion

        #region Ascend members
        /// <summary>
        /// Gets or sets the border for the control.
        /// </summary>
        /// <value>
        /// <para>
        /// Border. An object of type Border representing the control's border width characteristics.
        /// </para>
        /// <para>
        /// This property is read/write.
        /// </para>
        /// </value>
        /// <remarks>
        /// For containers such as GradientPanel and GradientCaption, the Border property gets or sets their respective border widths inside the DisplayRectangle.
        /// </remarks>
        [ResourceCategoryAttribute("AppearanceCategory"), ResourceDescriptionAttribute("BorderDescription"), ResourceDisplayName("DisplayNameBorder"), DefaultValueAttribute(typeof(Border), "0, 0, 0, 0")]
        public Border Border
        {
            get
            {
                return this.DieRender.Border;
            }

            set
            {
                if (value == this.DieRender.Border)
                {
                    return;
                }

                this.DieRender.Border = value;
                base.Invalidate();
                this.OnBorderChanged(new EventArgs());
            }
        }

        /// <summary>
        /// Gets or sets the border color(s) for the control.
        /// </summary>
        /// <value>
        /// <para>
        /// BorderColor. An object of type Border representing the control's border color characteristics.
        /// </para>
        /// <para>
        /// This property is read/write.
        /// </para>
        /// </value>
        /// <remarks>
        /// For containers such as GradientPanel and GradientCaption, the BorderColor property gets or sets their respective border colors inside the DisplayRectangle.
        /// </remarks>
        [ResourceCategoryAttribute("AppearanceCategory"), ResourceDescriptionAttribute("BorderColorDescription"), ResourceDisplayName("DisplayNameBorderColor"), DefaultValueAttribute(typeof(BorderColor), "Left=Color [ActiveCaption],Top=Color [ActiveCaption],Right=Color [ActiveCaption],Bottom=Color [ActiveCaption]")]
        public BorderColor BorderColor
        {
            get
            {
                return this.DieRender.BorderColor;
            }

            set
            {
                if (value == this.DieRender.BorderColor)
                {
                    return;
                }

                this.DieRender.BorderColor = value;
                base.Invalidate();
                this.OnBorderColorChanged(new EventArgs());
            }
        }

        /// <summary>
        /// Specifies the direction of a linear gradient.
        /// </summary>
        /// <value>
        /// <para>
        /// System.Drawing.Drawing2D.LinearGradientMode . Specifies the direction of a linear gradient.
        /// </para>
        /// <para>
        /// This property is read/write. 
        /// </para>
        /// </value>
        [ResourceCategoryAttribute("AppearanceCategory"), ResourceDescriptionAttribute("GradientModeDescription"), ResourceDisplayName("DisplayNameGradientMode"), DefaultValue(LinearGradientMode.Horizontal)]
        public LinearGradientMode GradientMode
        {
            get
            {
                return this.DieRender.GradientMode;
            }

            set
            {
                if (value == this.DieRender.GradientMode)
                {
                    return;
                }

                this.DieRender.GradientMode = value;
                base.Invalidate();
                this.OnGradientModeChanged(new EventArgs());
            }

        }

        /// <summary>
        /// Gets or sets the gradient high (lighter) color for the control.
        /// </summary>
        /// <value>
        /// <para>
        /// System.Drawing.Color . A Color that represents the gradient high color of the control.
        /// </para>
        /// <para>
        /// This property is read/write. 
        /// </para>
        /// </value>
        [ResourceCategoryAttribute("AppearanceCategory"), ResourceDescriptionAttribute("GradientHighColorDescription"), ResourceDisplayName("DisplayNameGradientHighColor"), DefaultValueAttribute(typeof(Color), "InactiveCaptionText")]
        public Color GradientHighColor
        {
            get
            {
                return this.DieRender.GradientHighColor;
            }

            set
            {
                if (value == this.DieRender.GradientHighColor)
                {
                    return;
                }

                this.DieRender.GradientHighColor = value;
                this.BackColor = value;
                base.Invalidate();
                this.OnGradientHighColorChanged(new EventArgs());
            }
        }

        /// <summary>
        /// Gets or sets the gradient low (darker) color for the control.
        /// </summary>
        /// <value>
        /// <para>
        /// System.Drawing.Color . A Color that represents the gradient low color of the control.
        /// </para>
        /// <para>
        /// This property is read/write. 
        /// </para>
        /// </value>
        [ResourceCategoryAttribute("AppearanceCategory"), ResourceDisplayName("DisplayNameGradientLowColor"), ResourceDescriptionAttribute("GradientLowColorDescription"), DefaultValueAttribute(typeof(Color), "GradientActiveCaption")]
        public Color GradientLowColor
        {
            get
            {
                return this.DieRender.GradientLowColor;
            }

            set
            {
                if (value == this.DieRender.GradientLowColor)
                {
                    return;
                }

                this.DieRender.GradientLowColor = value;
                base.Invalidate();
                this.OnGradientLowColorChanged(new EventArgs());
            }
        }

        /// <summary>
        /// The value of the die.
        /// </summary>
        public Lokeiv.Games.Dice.DieValue DieValue
        {
            get { return this._die.Value; }
            set { this._die.Value = value; this.Invalidate(); }
        }

        /// <summary>
        /// Fires the event indicating that the control Border has changed.  Inheriting controls should use this in favour of actually listening to the event, but should not forget to call base.OnBorderChanged() to ensure that the event is still fired for external listeners.
        /// </summary>
        /// <param name="e">A System.EventArgs that contains the event data.</param>
        protected void OnBorderChanged(EventArgs e)
        {
            if (this.BorderChanged != null)
            {
                this.BorderChanged(this, e);
            }
        }

        /// <summary>
        /// Fires the event indicating that the control BorderColor has changed.  Inheriting controls should use this in favour of actually listening to the event, but should not forget to call base.OnBorderColorChanged() to ensure that the event is still fired for external listeners.
        /// </summary>
        /// <param name="e">A System.EventArgs that contains the event data.</param>
        protected void OnBorderColorChanged(EventArgs e)
        {
            if (this.BorderColorChanged != null)
            {
                this.BorderColorChanged(this, e);
            }
        }

        /// <summary>
        /// Fires the event indicating that the control gradient high color has changed.  Inheriting controls should use this in favour of actually listening to the event, but should not forget to call base.OnGradientHighColorChanged() to ensure that the event is still fired for external listeners.
        /// </summary>
        /// <param name="e">A System.EventArgs that contains the event data.</param>
        protected void OnGradientHighColorChanged(EventArgs e)
        {
            if (this.GradientHighColorChanged != null)
            {
                this.GradientHighColorChanged(this, e);
            }
        }

        /// <summary>
        /// Fires the event indicating that the control gradient low color has changed.  Inheriting controls should use this in favour of actually listening to the event, but should not forget to call base.OnGradientLowColorChanged() to ensure that the event is still fired for external listeners.
        /// </summary>
        /// <param name="e">A System.EventArgs that contains the event data.</param>
        protected void OnGradientLowColorChanged(EventArgs e)
        {
            if (this.GradientLowColorChanged != null)
            {
                this.GradientLowColorChanged(this, e);
            }
        }

        /// <summary>
        /// Fires the event indicating that the control GradientMode has changed.  Inheriting controls should use this in favour of actually listening to the event, but should not forget to call base.OnGradientModeChanged() to ensure that the event is still fired for external listeners.
        /// </summary>
        /// <param name="e">A System.EventArgs that contains the event data.</param>
        protected void OnGradientModeChanged(EventArgs e)
        {
            if (this.GradientModeChanged != null)
            {
                this.GradientModeChanged(this, e);
            }
        }
        #endregion

        /// <summary>
        /// Initializes the class used for rendering. 
        /// Rendering class must be based on IGradientReder.
        /// </summary>
        protected virtual void InitializeRender()
        {
            if (this.DieRender != null)
                this.DieRender.Dispose();

            this.DieRender = new DieRender(this._die);            
        }

        protected void DieModel_DieRolled(object sender, EventArgs e)
        {
            this.Invalidate();
        }

        /// <summary>
        /// Raises the Paint event.
        /// </summary>
        /// <param name="e">A System.Windows.Forms.PaintEventArgs that contains the event data.</param>
        /// <remarks>
        /// <para>
        /// Raising an event invokes the event handler through a delegate. For more information, Raising an Event.
        /// </para>
        /// <para>
        /// The OnPaint method also allows derived classes to handle the event without attaching a delegate. This is the preferred technique for handling the event in a derived class.
        /// </para>
        /// <para>
        /// Note for Inheritors:
        /// When overriding OnPaint in a derived class, be sure to call the base class's OnPaint method so that registered delegates receive the event.
        /// </para>
        /// </remarks>
        protected override void OnPaint(System.Windows.Forms.PaintEventArgs e)
        {
            if (!this.Disposing)
            {                
                base.OnPaint(e);
                Rectangle modifiedRectangle = new Rectangle(0, 0, this.Width, this.Height);
                this.DieRender.DisplayRectangle = modifiedRectangle;
                this.DieRender.Render(e);
            }
        }

        protected override void OnClick(EventArgs e)
        {            
            if (!this.Disposing)
            {
                base.OnClick(e);

                if (this._rollDieOnClick)
                {
                    this._die.Roll();
                    this.OnDieRolled(new DieRolledEventArgs((int)this._die.Value.Value));
                    this.Invalidate();
                }
            }            
        }

        /// <summary>
        /// Raises the DieRolledEvent.
        /// </summary>
        /// <param name="e">EventArgs with the die value</param>
        protected void OnDieRolled(DieRolledEventArgs e)
        {
            //if(this.DieRolled != null)
            //{
            //    this.DieRolled(this, e);
            //}
        }

        /// <summary>
        /// Sets the die value manually. Should be used if the die object is tied to some
        /// game engine that rolls the die.
        /// </summary>
        /// <param name="value">Die value</param>
        public void SetDieValue(Lokeiv.Games.Dice.DieValue value)
        {
            if (this.DieType == Lokeiv.Games.Dice.DieTypes.NumericOneToSix && value.Value.GetType() != typeof(int))
            {
                throw new ArgumentException(value.Value.ToString());
            }

            if (this.DieType == Lokeiv.Games.Dice.DieTypes.NumericOneToSix)
            {
                int intValue = (int)value.Value;
                
                if (intValue < 1 || intValue > 6)
                {
                    throw new ArgumentOutOfRangeException(value.ToString());
                }
            }

            this._die.Value = value;
            this.Invalidate();
        }
    }

    /// <summary>
    /// EventArgs for the DieRolled event.
    /// </summary>
    public class DieRolledEventArgs : EventArgs
    {
        private int _dieValue;

        /// <summary>
        /// Value of the die after it is rolled.
        /// </summary>
        public int DieValue
        {
            get { return this._dieValue; }
        }

        /// <summary>
        /// Ctor
        /// </summary>
        /// <param name="dieValue">Value of die after it is rolled</param>
        public DieRolledEventArgs(int dieValue)
        {
            this._dieValue = dieValue;
        }
    }
}
