using System;
using System.Collections.Generic;
using System.Text;
using System.Threading;
using System.ComponentModel;
using Lokeiv.Games.Ludo.Model;
using Lokeiv.Games.Dice;
using System.Xml.Serialization;

namespace Lokeiv.Games.Ludo
{

    public delegate void DieRolledEventHandler(object engine, ActionEventArgs e);
    public delegate void PieceMovedEventHandler(object engine, ActionEventArgs e);
    public delegate void PlayerFinishedEventHandler(object engine, ActionEventArgs e);
    public delegate void GameOverEventHandler(object engine, ActionEventArgs e);

    public class Engine : IXmlSerializable
    {

        #region Events

        /// <summary>
        /// Occurs when the die has been rolled.
        /// </summary>
        public event DieRolledEventHandler DieRolled;

        /// <summary>
        /// Occurs when a piece has been moved.
        /// </summary>
        public event PieceMovedEventHandler PieceMoved;

        /// <summary>
        /// Occurs when a player has finished.
        /// </summary>
        public event PlayerFinishedEventHandler PlayerFinished;

        /// <summary>
        /// Occurs when all players have finished.
        /// </summary>
        public event GameOverEventHandler GameOver;

        /// <summary>
        /// Occurs when the game state has changed.
        /// </summary>
        public event EventHandler GameStateChanged;

        /// <summary>
        /// Occurs when <see cref="GameStatus"/> changed.
        /// </summary>
        public event EventHandler<EventArgs> GameStatusChanged;

        /// <summary>
        /// Occurs when a <see cref="Player"/>s current <see cref="Action"/> has changed.
        /// </summary>
        public event EventHandler<PlayerActionChangedEventArgs> PlayerActionChanged;

        /// <summary>
        /// Fires the <see cref="DieRolled" /> event.
        /// </summary>
        /// <param name="engine"><see cref="Engine"/></param>
        /// <param name="e"><see cref="ActionEventArgs"/></param>
        protected void OnDieRolled(object engine, ActionEventArgs e)
        {
            if (DieRolled != null)
            {
                DieRolled(engine, e);
            }

            this.OnGameStateChanged(engine, EventArgs.Empty);
        }        

        /// <summary>
        /// Fires the <see cref="PieceMoved" /> event.
        /// </summary>
        /// <param name="engine"><see cref="Engine"/></param>
        /// <param name="e"><see cref="ActionEventArgs"/></param>
        protected void OnPieceMoved(object engine, ActionEventArgs e)
        {
            if (PieceMoved != null)
            {
                PieceMoved(engine, e);
            }

            this.OnGameStateChanged(engine, EventArgs.Empty);
        }

        /// <summary>
        /// Fires the <see cref="PlayerFinished" /> event.
        /// </summary>
        /// <param name="engine"><see cref="Engine"/></param>
        /// <param name="e"><see cref="ActionEventArgs"/></param>
        protected void OnPlayerFinished(object engine, ActionEventArgs e)
        {
            if (PlayerFinished != null)
            {
                PlayerFinished(engine, e);
            }

            this.OnGameStateChanged(engine, EventArgs.Empty);
        }

        /// <summary>
        /// Fires the <see cref="GameOver" /> event.
        /// </summary>
        /// <param name="engine"><see cref="Engine"/></param>
        /// <param name="e"><see cref="ActionEventArgs"/></param>
        protected void OnGameOver(object engine, ActionEventArgs e)
        {
            if (GameOver != null)
            {
                GameOver(engine, e);
            }

            this.OnGameStateChanged(engine, EventArgs.Empty);
        }

        /// <summary>
        /// Fires the <see cref="GameStateChanged"/> event.
        /// </summary>
        /// <param name="engine"><see cref="Engine"/></param>
        /// <param name="e"><see cref="EventArgs"/></param>
        protected void OnGameStateChanged(object engine, EventArgs e)
        {
            if (GameStateChanged != null)
            {
                GameStateChanged(engine, e);
            }
        }

        #endregion

        #region Fields and properties
        private MoveablePiece _lastMovablePiece;
        private BackgroundWorker _worker = null;
        private int _rollDieSleepTime = 500;
        private int _movePieceSleepTime = 500;
        
        /// <summary>
        /// The current game options used for the game this engine runs.
        /// </summary>
        public GameOptions Options { get; set; }

        /// <summary>
        /// Gets the running and progress status of the game.
        /// </summary>
        public GameStatus GameStatus { get; private set; }

        /// <summary>
        /// Gets whether or not the game is running. <see cref="GameStatus"/> provides the reason why the game is not running.
        /// </summary>
        public bool IsRunning
        {
            get { return this.GameStatus == Games.Ludo.GameStatus.Running; }
        }

        private NumericOneToSixDie _numericOneToSixDie;
        public Die Die
        {
            get { return this._numericOneToSixDie; }
        }

        public Board Board { get; private set; }

        public PlayerCollection Players { get; private set; }

        internal int NumberOfPlayers { get; set; }

        internal Response LastResponse { get; set; }

        internal bool LastRollWasSix { get; set; }

        internal int CurrentPlayer { get; set; }

        #endregion

        #region Constructors

        /// <summary>
        /// None-public default constructor for deserialization.
        /// </summary>
        public Engine() 
        {
            this.Options = new GameOptions();
            this.Board = new Board();
            this.Players = new PlayerCollection();
            this._numericOneToSixDie = new NumericOneToSixDie();
            this._numericOneToSixDie.DieValueChanged += new EventHandler(DieValueChanged);
            this.InitializeBackgroundWorker();
        }

        /// <summary>
        /// Constructor creating a new game engine for the given players and options.
        /// </summary>
        /// <param name="players">List of players that participates in the game</param>
        /// <param name="options">Game options to use for the current game</param>
        public Engine(PlayerCollection players, GameOptions options) 
        {
            this.Options = options;
            if (this.Options == null)
                this.Options = new GameOptions();

            this.GameStatus = Games.Ludo.GameStatus.NotStarted;
            this.Players = players;
            this.AddPlayerEventHandlers();
            this.NumberOfPlayers = this.Players.Count;
            this.CurrentPlayer = 0;
            this._numericOneToSixDie = new NumericOneToSixDie();
            this._numericOneToSixDie.DieValueChanged += new EventHandler(DieValueChanged);
            this.Board = new Board(players);
            this.LastResponse = new Response(this._numericOneToSixDie.Value, this.Players[this.CurrentPlayer], Action.RollDie);
            this.LastResponse.PlayersTurn.CurrentAction = this.LastResponse.NextAction;
            this.InitializeBackgroundWorker();
        }

        #endregion

        #region Methods

        #region Other methods
        /// <summary>
        /// Initializes BackgroundWorker for computer players.
        /// </summary>
        private void InitializeBackgroundWorker()
        {
            this._worker = new BackgroundWorker();
            this._worker.DoWork += new DoWorkEventHandler(Worker_DoWork);
            this._worker.ProgressChanged += new ProgressChangedEventHandler(Worker_ProgressChanged);
            this._worker.RunWorkerCompleted += new RunWorkerCompletedEventHandler(Worker_RunWorkerCompleted);
            this._worker.WorkerSupportsCancellation = true;
            this._worker.WorkerReportsProgress = true;
        }

        /// <summary>
        /// Gets whether there are computer players in the game or not.
        /// </summary>        
        private bool HasComputerPlayers()
        {
            bool computerPlayers = false;

            foreach (Player player in this.Players)
            {
                if (player.Type == PlayerType.Computer)
                {
                    computerPlayers = true;
                }
            }

            return computerPlayers;
        }

        internal void AddPlayerEventHandlers()
        {
            foreach (Player player in this.Players)
                player.PlayerTypeChanged += PlayerTypeChanged;
        }

        internal void RemovePlayerEventHandlers()
        {
            foreach (Player player in this.Players)
                player.PlayerTypeChanged -= PlayerTypeChanged;
        }

        /// <summary>
        /// Starts the game if it isn't over.
        /// </summary>
        public void StartGame()
        {
            switch (this.Options.GameSpeed)
            {
                case GameSpeed.Fast:
                    this._rollDieSleepTime = 0;
                    this._movePieceSleepTime = 0;
                    break;
                case GameSpeed.Medium:
                    this._rollDieSleepTime = 500;
                    this._movePieceSleepTime = 500;
                    break;
                case GameSpeed.Slow:
                    this._rollDieSleepTime = 1000;
                    this._movePieceSleepTime = 1000;
                    break;
            }

            if (this.GameStatus == Games.Ludo.GameStatus.NotStarted || this.GameStatus == Games.Ludo.GameStatus.Paused)
            { 
                // Fire one of the events to have the UI update its state to the initial action
                this.OnDieRolled(this, new ActionEventArgs(this.LastResponse));

                if (!this.AreAllFinished())
                {
                    if (this.HasComputerPlayers())
                    {
                        // There are computer players in this game, so start the background worker running the computer AI
                        if (!this._worker.IsBusy)
                            this._worker.RunWorkerAsync();
                    }

                    this.GameStatus = Games.Ludo.GameStatus.Running;
                }
            }
        }
     
        /// <summary>
        /// Stops the game engine if it is running.
        /// </summary>
        public void EndGame()
        {
            if (this._worker.IsBusy)
            {
                this._worker.CancelAsync();
            }

            if (this.AreAllFinished())
                this.GameStatus = Games.Ludo.GameStatus.Finished;
            else
                this.GameStatus = Games.Ludo.GameStatus.Paused;
        }

        private void DieValueChanged(object sender, EventArgs e)
        {
            this.OnGameStateChanged(this, EventArgs.Empty);
        }

        private void PlayerTypeChanged(object sender, EventArgs e)
        {
            Player player = sender as Player;

            if (player != null)
            {
                this.OnGameStateChanged(this, EventArgs.Empty);
            }            
        }

        void Worker_ProgressChanged(object sender, ProgressChangedEventArgs e)
        {
            
        }

        void Worker_RunWorkerCompleted(object sender, RunWorkerCompletedEventArgs e)
        {
            if (this.AreAllFinished())
                this.GameStatus = Games.Ludo.GameStatus.Finished;
            else
                this.GameStatus = Games.Ludo.GameStatus.Paused;
        }
        #endregion  // Other methods

        #region Worker thread methods

        /// <summary>
        /// Runs computer AI
        /// </summary>
        private void Worker_DoWork(object sender, DoWorkEventArgs e)
        {
            BackgroundWorker worker = sender as BackgroundWorker;

            while (!e.Cancel)
            {
                if (this.IsComputerPlayer())
                {
                    // Do action for the current computer player
                    if (this.LastResponse.NextAction == Action.RollDie)
                    {
                        // Roll die
                        Thread.Sleep(this._rollDieSleepTime);
                        this.RollDie();
                    }
                    else if (this.LastResponse.NextAction == Action.MovePiece)
                    {
                        MoveablePiece pieceToMove = null;

                        if (this.LastResponse.MovablePieces.Count > 1)
                        {
                            // Select which piece to move
                            pieceToMove = Model.ComputerPlayer.MoveStrategyFactory.Instance.SelectPieceToMove(this.LastResponse.MovablePieces);
                        }
                        else
                        {
                            pieceToMove = this.LastResponse.MovablePieces[0];
                        }

                        // Move piece
                        Thread.Sleep(this._movePieceSleepTime);
                        this.MovePiece(pieceToMove);
                    }
                }

                if (this.AreAllFinished())
                {
                    e.Cancel = true;
                }

                if (worker.CancellationPending)
                {
                    e.Cancel = true;
                }

                Thread.Sleep(500);
            }
        }

        /// <summary>
        /// Returns true if all players are finished.
        /// </summary>
        public bool AreAllFinished()
        {
            foreach (Player player in this.Players)
            {
                if (!player.HasWon)
                {
                    return false;
                }
            }

            return true;
        }

        /// <summary>
        /// Returns true if the current player is a computer player.
        /// </summary>        
        private bool IsComputerPlayer()
        {
            return (this.Players[this.CurrentPlayer].Type == PlayerType.Computer);
        }

        /// <summary>
        /// Moves a pice and returns a response with what's to happen next.
        /// </summary>
        /// <param name="movablePieces">Pieces to move</param>
        /// <returns><see cref="Response">What's to happen next</see></returns>
        public Response MovePiece(MoveablePiece movablePiece)
        {
            if (this.LastResponse == null || this.LastResponse.NextAction != Action.MovePiece)
            {
                return null;  // A piece should not be moved now
            }

            Response response = new Response();
            List<MoveablePiece> moveablePieces = this.Board.MovePiece(movablePiece);

            // Save new position of moved pieces
            response.MovablePieces.AddRange(moveablePieces);

            if(this.Board.HasFinished(movablePiece.Piece.Color))
            {
                this.Players[this.CurrentPlayer].SetWon(true);
                response.PlayerWon = this.Players[this.CurrentPlayer];
            }

            this.SetPlayerAction(this.CurrentPlayer, Action.Nothing);

            // Pass on to the next player
            this.NextPlayersTurn();

            response.PlayersTurn = this.Players[this.CurrentPlayer];
            response.NextAction = Action.RollDie;
            this.SetPlayerAction(this.CurrentPlayer, response.NextAction);
            this.LastResponse = response;

            // Calculate moves left for all players
            foreach (Player player in this.Players)
            {
                player.SetNumberOfMovesLeft(this.Board.GetNumberOfMovesLeftForPlayer(player.Color));
            }

            // Raise event to update UI
            this.OnPieceMoved(this, new ActionEventArgs(response));

            return response;
        }

        /// <summary>
        /// Rolls the die and returns a response with what's to happen next.
        /// </summary>
        /// <returns><see cref="Response">What's to happen next</see></returns>
        public Response RollDie()
        {
            if (this.LastResponse == null || this.LastResponse.NextAction != Action.RollDie)
            {
                return null;  // The die should not be rolled now
            }

            Response response = new Response();
            response.DieValue = this._numericOneToSixDie.Roll();            

            if ((int)this._numericOneToSixDie.Value.Value == 6)
            {
                this.LastRollWasSix = true;
            }
            else
            {
                this.LastRollWasSix = false;
            }
      
            response.MovablePieces = this.Board.GetPossibleMoves(this._numericOneToSixDie, this.Players[this.CurrentPlayer].Color);
            this.SetPlayerAction(this.CurrentPlayer, Action.Nothing);

            if (response.MovablePieces.Count == 0)
            {
                // No possible moves, pass on to the next player
                this.NextPlayersTurn();                
                response.NextAction = Action.RollDie;
            }
            else
            {
                // Possible moves
                response.NextAction = Action.MovePiece;
            }
            
            response.PlayersTurn = this.Players[this.CurrentPlayer];
            this.SetPlayerAction(this.CurrentPlayer, response.NextAction);
            this.LastResponse = response;

            // Raise event to update UI
            this.OnDieRolled(this, new ActionEventArgs(response));  

            return response;
        }

        /// <summary>
        /// Finds which player is next.
        /// </summary>
        private void NextPlayersTurn()
        {
            if (!this.LastRollWasSix || this.Players[this.CurrentPlayer].HasWon)
            {
                bool passOn = true;
                int index = 0;

                while (passOn)
                {
                    this.CurrentPlayer++;

                    if (this.CurrentPlayer > this.NumberOfPlayers - 1)
                    {
                        this.CurrentPlayer = 0;
                    }

                    if (!this.Players[this.CurrentPlayer].HasWon)
                    {
                        // It's this players turn
                        passOn = false;
                    }

                    index++;

                    if (index >= 4)
                    {
                        // All players have finished, no players can keep on
                        passOn = false;
                    }
                }
            }
        }

        /// <summary>
        /// Sets the <see cref="Player.CurrentAction"/> to the given <paramref name="action"/>.
        /// </summary>
        /// <param name="player">Player index</param>
        /// <param name="action"><see cref="Action"/></param>
        private void SetPlayerAction(int player, Action action)
        {
            this.Players[player].CurrentAction = action;
            this.OnPlayerActionChanged(new PlayerActionChangedEventArgs(this.Players[player]));
        }
        #endregion  // Worker thread methods

        #region Event raiser methods

        /// <summary>
        /// Raises the <see cref="GameStatusChanged"/> event.
        /// </summary>
        /// <param name="e"><see cref="EventArgs"/></param>
        protected void OnGameStatusChanged(EventArgs e)
        {
            if (this.GameStatusChanged != null)
                GameStatusChanged(this, e);
        }

        /// <summary>
        /// Raises the <see cref="PlayerActionChanged"/> event.
        /// </summary>
        /// <param name="e"><see cref="PlayerActionChangedEventArgs"/></param>
        protected void OnPlayerActionChanged(PlayerActionChangedEventArgs e)
        {
            if (this.PlayerActionChanged != null)
                PlayerActionChanged(this, e);
        }

        #endregion  // Event raiser methods

        #endregion  // Methods

        #region IXmlSerializable
        public System.Xml.Schema.XmlSchema GetSchema()
        {
            return null;
        }

        public void ReadXml(System.Xml.XmlReader reader)
        {
            reader.MoveToContent();
            string xml = reader.ReadOuterXml();

            // Read attributes
            this.ReadEngineAttributes(xml);

            // Read objects that other content has dependencies on
            this.ReadPlayers(xml);
            this.ReadBoard(xml);

            // Read content that has not yet been read
            this.ReadOtherContent(xml);
        }

        private void ReadEngineAttributes(string xml)
        {
            using (System.IO.StringReader stringReader = new System.IO.StringReader(xml))
            using (System.Xml.XmlReader reader = System.Xml.XmlReader.Create(stringReader))
            {
                reader.MoveToContent();

                string lastRollWasSix = reader.GetAttribute("LastRollWasSix");
                this.LastRollWasSix = bool.Parse(lastRollWasSix);

                string currentPlayer = reader.GetAttribute("CurrentPlayer");
                this.CurrentPlayer = int.Parse(currentPlayer);
            }
        }

        private void ReadBoard(string xml)
        {
            using (System.IO.StringReader stringReader = new System.IO.StringReader(xml))
            using (System.Xml.XmlReader reader = System.Xml.XmlReader.Create(stringReader))
            {
                if (reader.ReadToDescendant("Board"))
                    this.Board = Lokeiv.Games.Ludo.Model.Board.ReadXml(reader.ReadSubtree(), this.Players);
            }
        }

        private void ReadPlayers(string xml)
        {
            using (System.IO.StringReader stringReader = new System.IO.StringReader(xml))
            using (System.Xml.XmlReader reader = System.Xml.XmlReader.Create(stringReader))
            {
                if (reader.ReadToDescendant("Players"))                    
                    this.Players = Player.ReadCollectionFromXml(reader.ReadSubtree());
            }

            this.NumberOfPlayers = this.Players.Count;
        }

        private void ReadOtherContent(string xml)
        {
            using (System.IO.StringReader stringReader = new System.IO.StringReader(xml))
            using (System.Xml.XmlReader reader = System.Xml.XmlReader.Create(stringReader))
            {
                while (reader.Read())
                {
                    if (reader.NodeType == System.Xml.XmlNodeType.Element)
                    {
                        switch (reader.Name.ToLower())
                        {
                            case "gameoptions":
                                if (this.Options != null) this.Options.ReadXml(reader.ReadSubtree());
                                reader.Skip();
                                break;
                            case "die":
                                this.Die.ReadXml(reader.ReadSubtree());
                                reader.Skip();
                                break;
                            case "lastmovablepiece":
                                this._lastMovablePiece = MoveablePiece.ReadXml(reader.ReadSubtree(), this.Board);
                                reader.Skip();
                                break;
                            case "lastresponse":
                                this.LastResponse = Response.ReadXml(reader.ReadSubtree(), this.Board, this.Players);
                                reader.Skip();
                                break;
                        }
                    }
                }
            }
        }

        public void WriteXml(System.Xml.XmlWriter writer)
        {
            // Write attributes
            writer.WriteAttributeString("LastRollWasSix", this.LastRollWasSix.ToString());
            writer.WriteAttributeString("CurrentPlayer", this.CurrentPlayer.ToString());            

            // Write GameOptions
            writer.WriteStartElement("GameOptions");
            if (this.Options != null) this.Options.WriteXml(writer);
            writer.WriteEndElement();

            // Write Die
            writer.WriteStartElement("Die");
            writer.WriteAttributeString("Type", this._numericOneToSixDie.GetType().Name);
            this._numericOneToSixDie.WriteXml(writer);
            writer.WriteEndElement();

            // Write board
            writer.WriteStartElement("Board");
            this.Board.WriteXml(writer);
            writer.WriteEndElement();

            // Write LastMovablePiece
            writer.WriteStartElement("LastMovablePiece");
            if (this._lastMovablePiece != null) this._lastMovablePiece.WriteXml(writer);
            writer.WriteEndElement();

            // Write LastResponse
            writer.WriteStartElement("LastResponse");
            this.LastResponse.WriteXml(writer);
            writer.WriteEndElement();

            // Write players
            writer.WriteStartElement("Players");
            foreach (Player player in this.Players)
            {
                writer.WriteStartElement("Player");
                player.WriteXml(writer);
                writer.WriteEndElement();
            }
            writer.WriteEndElement();
        }
        #endregion
    }
}
