using System;
using System.Diagnostics;
using System.Collections.Generic;
using System.Text;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.ComponentModel;
using System.Windows.Forms;
using Ascend;
using Ascend.Resources;
using Ascend.Windows.Forms;
using Lokeiv.Games.Ludo.Model;

namespace Lokeiv.Games.Ludo.Forms.Client
{
    public class Board : System.Windows.Forms.Control, IBoard
    {
        internal BoardRender BoardRender;
        internal protected float SquareWidth;
        internal protected float SquareHeight;
        internal protected float Square1Left;
        internal protected float Square1Top;
        internal protected float BetweenSquaresInHouseHeight;
        internal protected float BetweenSquaresInHouseWidth;
        internal protected float ColoredHouseHeight;
        internal protected float ColoredHouseWidth;
        internal protected float WhiteHouseHeight;
        internal protected float WhiteHouseWidth;
        internal protected float HouseHeight;
        internal protected float HouseWidth;
        internal protected HouseRender RedHouseRender;
        internal protected HouseRender YellowHouseRender;
        internal protected HouseRender GreenHouseRender;
        internal protected HouseRender BlueHouseRender;
        internal protected GoalRender RedGoalRender;
        internal protected GoalRender YellowGoalRender;
        internal protected GoalRender GreenGoalRender;
        internal protected GoalRender BlueGoalRender;

        private Game _game;
        private Lokeiv.Games.Ludo.Model.Board _board;	
        private List<SquareRender> _squares = new List<SquareRender>(92);
        private List<PieceRender> _pieces = new List<PieceRender>();

        #region Events
        /// <summary>
        /// Occurs when the Border property changes.
        /// </summary>
        [ResourceDescriptionAttribute("BorderChangedDescription"), ResourceCategoryAttribute("PropertyChangedCategory"), ResourceDisplayName("DisplayNameBorderChanged")]
        public event EventHandler BorderChanged;

        /// <summary>
        /// Occurs when the BorderColor property changes.
        /// </summary>
        [ResourceDescriptionAttribute("BorderColorChangedDescription"), ResourceCategoryAttribute("PropertyChangedCategory"), ResourceDisplayName("DisplayNameBorderColorChanged")]
        public event EventHandler BorderColorChanged;

        /// <summary>
        /// Occurs when the GradientLowColor property changes.
        /// </summary>
        [ResourceDescriptionAttribute("GradientLowColorChangedDescription"), ResourceCategoryAttribute("PropertyChangedCategory"), ResourceDisplayName("DisplayNameGradientLowColorChanged")]
        public event EventHandler GradientLowColorChanged;

        /// <summary>
        /// Occurs when the GradientHighColor property changes.
        /// </summary>
        [ResourceDescriptionAttribute("GradientHighColorChangedDescription"), ResourceCategoryAttribute("PropertyChangedCategory"), ResourceDisplayName("DisplayNameGradientHighColorChanged")]
        public event EventHandler GradientHighColorChanged;

        /// <summary>
        /// Occurs when the GradientMode property changes.
        /// </summary>
        [ResourceDescriptionAttribute("GradientModeChangedDescription"), ResourceCategoryAttribute("PropertyChangedCategory"), ResourceDisplayName("DisplayNameGradientModeChanged")]
        public event EventHandler GradientModeChanged;

        #endregion

        #region Public properties
        /// <summary>
        /// The <see cref="Game"/> for which this <see cref="Board"/> belongs.
        /// </summary>
        public Game Game
        {
            get { return this._game; }
            set { this._game = value; }
        }
        
        /// <summary>
        /// The model that represents this <see cref="Board"/>.
        /// </summary>
        public Lokeiv.Games.Ludo.Model.Board BoardModel
        {
            get { return _board; }            
        }

        /// <summary>
        /// The visible <see cref="Square">squares</see> of this <see cref="Board"/>.
        /// </summary>
        public List<SquareRender> Squares
        {
            get { return this._squares; }
        }

        #endregion

        #region Contructors
        /// <summary>
        /// Initializes a new instance of the Die class.
        /// </summary>
        public Board() : base()
        {
            this.InitializeComponent();
            this._board = new Lokeiv.Games.Ludo.Model.Board(new List<Player>());
            this.RedHouseRender = new HouseRender(Lokeiv.Games.Ludo.Model.Color.Red);
            this.YellowHouseRender = new HouseRender(Lokeiv.Games.Ludo.Model.Color.Yellow);
            this.GreenHouseRender = new HouseRender(Lokeiv.Games.Ludo.Model.Color.Green);
            this.BlueHouseRender = new HouseRender(Lokeiv.Games.Ludo.Model.Color.Blue);
            this.RedGoalRender = new GoalRender(Lokeiv.Games.Ludo.Model.Color.Red);
            this.YellowGoalRender = new GoalRender(Lokeiv.Games.Ludo.Model.Color.Yellow);
            this.GreenGoalRender = new GoalRender(Lokeiv.Games.Ludo.Model.Color.Green);
            this.BlueGoalRender = new GoalRender(Lokeiv.Games.Ludo.Model.Color.Blue);
            this.InitializeRender();
            this.InitializeBoard();
            this.BorderColor = new BorderColor(SystemColors.ActiveCaption);

            base.SetStyle(ControlStyles.SupportsTransparentBackColor | ControlStyles.OptimizedDoubleBuffer | ControlStyles.UserPaint | ControlStyles.AllPaintingInWmPaint | ControlStyles.Selectable, true);
            base.BackColor = System.Drawing.Color.FromArgb(0, SystemColors.Window);

            this.GradientMode = LinearGradientMode.Horizontal;
            this.GradientHighColor = SystemColors.Window;
            this.GradientLowColor = SystemColors.MenuBar;
        }

        #endregion

        #region Ascend members
        /// <summary>
        /// Gets or sets the border for the control.
        /// </summary>
        /// <value>
        /// <para>
        /// Border. An object of type Border representing the control's border width characteristics.
        /// </para>
        /// <para>
        /// This property is read/write.
        /// </para>
        /// </value>
        /// <remarks>
        /// For containers such as GradientPanel and GradientCaption, the Border property gets or sets their respective border widths inside the DisplayRectangle.
        /// </remarks>
        [ResourceCategoryAttribute("AppearanceCategory"), ResourceDescriptionAttribute("BorderDescription"), ResourceDisplayName("DisplayNameBorder"), DefaultValueAttribute(typeof(Border), "0, 0, 0, 0")]
        public Border Border
        {
            get
            {
                return this.BoardRender.Border;
            }

            set
            {
                if (value == this.BoardRender.Border)
                {
                    return;
                }

                this.BoardRender.Border = value;
                base.Invalidate();
                this.OnBorderChanged(new EventArgs());
            }
        }

        /// <summary>
        /// Gets or sets the border color(s) for the control.
        /// </summary>
        /// <value>
        /// <para>
        /// BorderColor. An object of type Border representing the control's border color characteristics.
        /// </para>
        /// <para>
        /// This property is read/write.
        /// </para>
        /// </value>
        /// <remarks>
        /// For containers such as GradientPanel and GradientCaption, the BorderColor property gets or sets their respective border colors inside the DisplayRectangle.
        /// </remarks>
        [ResourceCategoryAttribute("AppearanceCategory"), ResourceDescriptionAttribute("BorderColorDescription"), ResourceDisplayName("DisplayNameBorderColor"), DefaultValueAttribute(typeof(BorderColor), "Left=Color [ActiveCaption],Top=Color [ActiveCaption],Right=Color [ActiveCaption],Bottom=Color [ActiveCaption]")]
        public BorderColor BorderColor
        {
            get
            {
                return this.BoardRender.BorderColor;
            }

            set
            {
                if (value == this.BoardRender.BorderColor)
                {
                    return;
                }

                this.BoardRender.BorderColor = value;
                base.Invalidate();
                this.OnBorderColorChanged(new EventArgs());
            }
        }

        /// <summary>
        /// Specifies the direction of a linear gradient.
        /// </summary>
        /// <value>
        /// <para>
        /// System.Drawing.Drawing2D.LinearGradientMode . Specifies the direction of a linear gradient.
        /// </para>
        /// <para>
        /// This property is read/write. 
        /// </para>
        /// </value>
        [ResourceCategoryAttribute("AppearanceCategory"), ResourceDescriptionAttribute("GradientModeDescription"), ResourceDisplayName("DisplayNameGradientMode"), DefaultValue(LinearGradientMode.Horizontal)]
        public LinearGradientMode GradientMode
        {
            get
            {
                return this.BoardRender.GradientMode;
            }

            set
            {
                if (value == this.BoardRender.GradientMode)
                {
                    return;
                }

                this.BoardRender.GradientMode = value;
                base.Invalidate();
                this.OnGradientModeChanged(new EventArgs());
            }

        }

        /// <summary>
        /// Gets or sets the gradient high (lighter) color for the control.
        /// </summary>
        /// <value>
        /// <para>
        /// System.Drawing.Color . A Color that represents the gradient high color of the control.
        /// </para>
        /// <para>
        /// This property is read/write. 
        /// </para>
        /// </value>
        [ResourceCategoryAttribute("AppearanceCategory"), ResourceDescriptionAttribute("GradientHighColorDescription"), ResourceDisplayName("DisplayNameGradientHighColor"), DefaultValueAttribute(typeof(System.Drawing.Color), "InactiveCaptionText")]
        public System.Drawing.Color GradientHighColor
        {
            get
            {
                return this.BoardRender.GradientHighColor;
            }

            set
            {
                if (value == this.BoardRender.GradientHighColor)
                {
                    return;
                }

                this.BoardRender.GradientHighColor = value;
                this.BackColor = value;
                base.Invalidate();
                this.OnGradientHighColorChanged(new EventArgs());
            }
        }

        /// <summary>
        /// Gets or sets the gradient low (darker) color for the control.
        /// </summary>
        /// <value>
        /// <para>
        /// System.Drawing.Color . A Color that represents the gradient low color of the control.
        /// </para>
        /// <para>
        /// This property is read/write. 
        /// </para>
        /// </value>
        [ResourceCategoryAttribute("AppearanceCategory"), ResourceDisplayName("DisplayNameGradientLowColor"), ResourceDescriptionAttribute("GradientLowColorDescription"), DefaultValueAttribute(typeof(System.Drawing.Color), "GradientActiveCaption")]
        public System.Drawing.Color GradientLowColor
        {
            get
            {
                return this.BoardRender.GradientLowColor;
            }

            set
            {
                if (value == this.BoardRender.GradientLowColor)
                {
                    return;
                }

                this.BoardRender.GradientLowColor = value;
                base.Invalidate();
                this.OnGradientLowColorChanged(new EventArgs());
            }
        }


        /// <summary>
        /// Fires the event indicating that the control Border has changed.  Inheriting controls should use this in favour of actually listening to the event, but should not forget to call base.OnBorderChanged() to ensure that the event is still fired for external listeners.
        /// </summary>
        /// <param name="e">A System.EventArgs that contains the event data.</param>
        protected void OnBorderChanged(EventArgs e)
        {
            if (this.BorderChanged != null)
            {
                this.BorderChanged(this, e);
            }
        }

        /// <summary>
        /// Fires the event indicating that the control BorderColor has changed.  Inheriting controls should use this in favour of actually listening to the event, but should not forget to call base.OnBorderColorChanged() to ensure that the event is still fired for external listeners.
        /// </summary>
        /// <param name="e">A System.EventArgs that contains the event data.</param>
        protected void OnBorderColorChanged(EventArgs e)
        {
            if (this.BorderColorChanged != null)
            {
                this.BorderColorChanged(this, e);
            }
        }

        /// <summary>
        /// Fires the event indicating that the control gradient high color has changed.  Inheriting controls should use this in favour of actually listening to the event, but should not forget to call base.OnGradientHighColorChanged() to ensure that the event is still fired for external listeners.
        /// </summary>
        /// <param name="e">A System.EventArgs that contains the event data.</param>
        protected void OnGradientHighColorChanged(EventArgs e)
        {
            if (this.GradientHighColorChanged != null)
            {
                this.GradientHighColorChanged(this, e);
            }
        }

        /// <summary>
        /// Fires the event indicating that the control gradient low color has changed.  Inheriting controls should use this in favour of actually listening to the event, but should not forget to call base.OnGradientLowColorChanged() to ensure that the event is still fired for external listeners.
        /// </summary>
        /// <param name="e">A System.EventArgs that contains the event data.</param>
        protected void OnGradientLowColorChanged(EventArgs e)
        {
            if (this.GradientLowColorChanged != null)
            {
                this.GradientLowColorChanged(this, e);
            }
        }

        /// <summary>
        /// Fires the event indicating that the control GradientMode has changed.  Inheriting controls should use this in favour of actually listening to the event, but should not forget to call base.OnGradientModeChanged() to ensure that the event is still fired for external listeners.
        /// </summary>
        /// <param name="e">A System.EventArgs that contains the event data.</param>
        protected void OnGradientModeChanged(EventArgs e)
        {
            if (this.GradientModeChanged != null)
            {
                this.GradientModeChanged(this, e);
            }
        }
#endregion

        /// <summary>
        /// Starts a new game.
        /// </summary>
        /// <param name="players">Players to join the game</param>        
        /// <param name="game">The <see cref="Game"/> which this board belongs to</param>
        public void StartNewGame(List<Player> players, Game game)
        {            
            this._board = new Lokeiv.Games.Ludo.Model.Board(players);
            this.SetGameModel(this._board, game);
        }

        /// <summary>
        /// Sets the board model used in the game that this client belongs to.
        /// </summary>
        /// <param name="board"><see cref="Lokeiv.Games.Ludo.Model.Board"/></param>
        /// <param name="game">The <see cref="Game"/> which this board belongs to</param>
        public void SetGameModel(Lokeiv.Games.Ludo.Model.Board board, Game game)
        {
            this._game = game;
            this._board = board;
            this.InitializeBoard();
            this.CalculateBoardSize();
            this.Invalidate();
        }

        /// <summary>
        /// Raises the SizeChanged event.
        /// </summary>
        /// <param name="e">An System.EventArgs that contains the event data.</param>
        protected override void OnSizeChanged(System.EventArgs e)
        {
            base.OnSizeChanged(e);
            this.CalculateBoardSize();
            this.Invalidate();
        }

        /// <summary>
        /// Fires the event indicating that the gradient panel has been resized. Inheriting controls should use this in favour of actually listening to the event, but should not forget to call base.onResize() to ensure that the event is still fired for external listeners.
        /// </summary>
        /// <param name="e">An System.EventArgs that contains the event data.</param>
        protected override void OnResize(System.EventArgs e)
        {
            base.OnResize(e);
            base.Invalidate();
        }

        /// <summary>
        /// Initializes the class used for rendering. 
        /// Rendering class must be based on IGradientReder.
        /// </summary>
        protected virtual void InitializeRender()
        {
            this.BoardRender = new BoardRender();            
        }

        /// <summary>
        /// Creates a <see cref="SquareRender"/> for each <see cref="Square"/>.
        /// </summary>
        private void InitializeBoard()
        {
            // Connect to squares in model
            this._squares.Clear();

            foreach(Square square in this._board.Squares)
            {
                this._squares.Add(new SquareRender(square));
            }

            // Connect to pieces in model
            this._pieces.Clear();

            foreach(Piece piece in this._board.Pieces)
            {
                this._pieces.Add(new PieceRender(piece, this));
            }
        }

        /// <summary>
        /// Finds the best values for the size of squares etc.
        /// </summary>
        private void CalculateBoardSize()
        {
            this.SquareHeight = this.Bounds.Height / 15;
            this.SquareWidth = this.Bounds.Width / 15;
            this.Square1Left = this.SquareWidth * 6;
            this.Square1Top = 0;
            this.ColoredHouseHeight = SquareHeight;
            this.ColoredHouseWidth = SquareWidth;
            this.WhiteHouseHeight = SquareHeight * 2 / 3;
            this.WhiteHouseWidth = SquareWidth * 2 / 3;
            this.BetweenSquaresInHouseHeight = this.WhiteHouseHeight;
            this.BetweenSquaresInHouseWidth = this.WhiteHouseWidth;
            this.HouseHeight = this.ColoredHouseHeight + this.WhiteHouseHeight;
            this.HouseWidth = this.ColoredHouseWidth + this.WhiteHouseWidth;
            
            // Update the bounds of each and every square
            this.UpdateSquareBounds();
        }

        /// <summary>
        /// Updates the bounds of each and every square
        /// </summary>
        private void UpdateSquareBounds()
        {
            for (int index = 0; index < this._board.Squares.Count; index++)
            {
                Square square = this._board.Squares[index];
                SquareRender squareRender = this._squares[index];
                RectangleF bounds = new RectangleF() ;

                switch (index)
                {
                    case 0:
                    case 1:
                    case 2:
                        // The three white squares at the top of the board
                        bounds.X = this.Square1Left + index * this.SquareWidth;
                        bounds.Y = this.Square1Top;
                        break;
                    case 3:
                    case 4:
                    case 5:
                    case 6:
                    case 7:
                        // To the left of the yellow house
                        bounds.X = this._squares[index - 1].Bounds.X;
                        bounds.Y = this._squares[index - 1].Bounds.Y + this.SquareHeight;
                        break;
                    case 8:
                        // First square beneath the yellow house
                        bounds.X = this._squares[index - 1].Bounds.X + this.SquareWidth;
                        bounds.Y = this._squares[index - 1].Bounds.Y + this.SquareHeight;
                        break;
                    case 9:
                    case 10:
                    case 11:
                    case 12:
                    case 13:
                        // Beneath the yellow house
                        bounds.X = this._squares[index - 1].Bounds.X + this.SquareWidth;
                        bounds.Y = this._squares[index - 1].Bounds.Y;
                        break;
                    case 14:
                    case 15:
                        // Down towards the green house
                        bounds.X = this._squares[index - 1].Bounds.X;
                        bounds.Y = this._squares[index - 1].Bounds.Y + this.SquareHeight;
                        break;
                    case 16:
                    case 17:
                    case 18:
                    case 19:
                    case 20:
                        // Over the green house
                        bounds.X = this._squares[index - 1].Bounds.X - this.SquareWidth;
                        bounds.Y = this._squares[index - 1].Bounds.Y;
                        break;
                    case 21:
                        // First square to the left of the green house
                        bounds.X = this._squares[index - 1].Bounds.X - this.SquareWidth;
                        bounds.Y = this._squares[index - 1].Bounds.Y + this.SquareHeight;
                        break;
                    case 22:
                    case 23:
                    case 24:
                    case 25:
                    case 26:
                        // To the left of the green house
                        bounds.X = this._squares[index - 1].Bounds.X;
                        bounds.Y = this._squares[index - 1].Bounds.Y + this.SquareHeight;
                        break;
                    case 27:
                    case 28:
                        // Left towards the blue house
                        bounds.X = this._squares[index - 1].Bounds.X - this.SquareWidth;
                        bounds.Y = this._squares[index - 1].Bounds.Y;
                        break;
                    case 29:
                    case 30:
                    case 31:
                    case 32:
                    case 33:
                        // To the right of the blue house
                        bounds.X = this._squares[index - 1].Bounds.X;
                        bounds.Y = this._squares[index - 1].Bounds.Y - this.SquareHeight;
                        break;
                    case 34:
                        // First square over the blue house
                        bounds.X = this._squares[index - 1].Bounds.X - this.SquareWidth;
                        bounds.Y = this._squares[index - 1].Bounds.Y - this.SquareHeight;
                        break;
                    case 35:
                    case 36:
                    case 37:
                    case 38:
                    case 39:
                        // Over the blue house
                        bounds.X = this._squares[index - 1].Bounds.X - this.SquareWidth;
                        bounds.Y = this._squares[index - 1].Bounds.Y;
                        break;
                    case 40:
                    case 41:
                        // Up towards the red house
                        bounds.X = this._squares[index - 1].Bounds.X;
                        bounds.Y = this._squares[index - 1].Bounds.Y - this.SquareHeight;
                        break;
                    case 42:
                    case 43:
                    case 44:
                    case 45:
                    case 46:
                        // Under the red house
                        bounds.X = this._squares[index - 1].Bounds.X + this.SquareWidth;
                        bounds.Y = this._squares[index - 1].Bounds.Y;
                        break;
                    case 47:
                        // First square to the right of the red house
                        bounds.X = this._squares[index - 1].Bounds.X + this.SquareWidth;
                        bounds.Y = this._squares[index - 1].Bounds.Y - this.SquareHeight;
                        break;
                    case 48:
                    case 49:
                    case 50:
                    case 51:
                        // To the right of the red house
                        bounds.X = this._squares[index - 1].Bounds.X;
                        bounds.Y = this._squares[index - 1].Bounds.Y - this.SquareHeight;
                        break;
                    case 52:
                    case 53:
                    case 54:
                    case 55:
                    case 56:
                        // Path to yellow goal
                        bounds.X = this._squares[index - 49].Bounds.X - this.SquareWidth;
                        bounds.Y = this._squares[index - 49].Bounds.Y;
                        break;
                    case 58:
                    case 59:
                    case 60:
                    case 61:
                    case 62:
                        // Path to the green goal
                        bounds.X = this._squares[index - 42].Bounds.X;
                        bounds.Y = this._squares[index - 42].Bounds.Y - this.SquareHeight;
                        break;
                    case 64:
                    case 65:
                    case 66:
                    case 67:
                    case 68:
                        // Path to the blue goal
                        bounds.X = this._squares[index - 35].Bounds.X + this.SquareWidth;
                        bounds.Y = this._squares[index - 35].Bounds.Y;
                        break;
                    case 70:
                    case 71:
                    case 72:
                    case 73:
                    case 74:
                        // Path to the red goal
                        bounds.X = this._squares[index - 28].Bounds.X;
                        bounds.Y = this._squares[index - 28].Bounds.Y + this.SquareHeight;
                        break;
                    case 88:
                        // In the red house
                        bounds.X = this.HouseWidth;
                        bounds.Y = this.HouseHeight + this.SquareHeight + this.BetweenSquaresInHouseHeight;
                        break;
                    case 89:
                        // In the red house
                        bounds.X = this.HouseWidth;
                        bounds.Y = this.HouseHeight;
                        break;
                    case 90:
                        // In the red house
                        bounds.X = this._squares[88].Bounds.X + this.SquareWidth + this.BetweenSquaresInHouseWidth;
                        bounds.Y = this._squares[88].Bounds.Y;
                        break;
                    case 91:
                        // In the red house
                        bounds.X = this._squares[90].Bounds.X;
                        bounds.Y = this._squares[89].Bounds.Y;
                        break;
                    case 76:
                        // In the yellow house
                        bounds.X = this._squares[2].Bounds.X + this.SquareWidth + this.HouseWidth;
                        bounds.Y = this.HouseHeight;
                        break;
                    case 77:
                        // In the yellow house
                        bounds.X = this._squares[76].Bounds.X + this.SquareWidth + this.BetweenSquaresInHouseWidth;
                        bounds.Y = this.HouseHeight;
                        break;
                    case 78:
                        // In the yellow house
                        bounds.X = this._squares[76].Bounds.X;
                        bounds.Y = this._squares[76].Bounds.Y + this.SquareHeight + this.BetweenSquaresInHouseHeight;
                        break;
                    case 79:
                        // In the yellow house
                        bounds.X = this._squares[77].Bounds.X;
                        bounds.Y = this._squares[78].Bounds.Y;
                        break;
                    case 80:
                        // In the green house
                        bounds.X = this._squares[77].Bounds.X;
                        bounds.Y = this._squares[15].Bounds.Y + this.SquareHeight + this.HouseHeight;
                        break;
                    case 81:
                        // In the green house
                        bounds.X = this._squares[80].Bounds.X;
                        bounds.Y = this._squares[80].Bounds.Y + this.SquareHeight + this.BetweenSquaresInHouseHeight;
                        break;
                    case 82:
                        // In the green house
                        bounds.X = this._squares[76].Bounds.X;
                        bounds.Y = this._squares[80].Bounds.Y;
                        break;
                    case 83:
                        // In the green house
                        bounds.X = this._squares[82].Bounds.X;
                        bounds.Y = this._squares[81].Bounds.Y;
                        break;
                    case 84:
                        // In the blue house
                        bounds.X = this.HouseWidth + this.SquareWidth + this.BetweenSquaresInHouseWidth;
                        bounds.Y = this._squares[83].Bounds.Y;
                        break;
                    case 85:
                        // In the blue house
                        bounds.X = this.HouseWidth;
                        bounds.Y = this._squares[84].Bounds.Y;
                        break;
                    case 86:
                        // In the blue house
                        bounds.X = this._squares[84].Bounds.X;
                        bounds.Y = this._squares[82].Bounds.Y;
                        break;
                    case 87:
                        // In the blue house
                        bounds.X = this._squares[85].Bounds.X;
                        bounds.Y = this._squares[86].Bounds.Y;
                        break;
                    case 75:
                        // Hidden square in the red goal
                        bounds.X = this._squares[74].Bounds.Left + this.SquareWidth;
                        bounds.Y = this._squares[74].Bounds.Top;
                        break;
                    case 57:
                        // Hidden square in the yellow goal
                        bounds.X = this._squares[56].Bounds.Left;
                        bounds.Y = this._squares[56].Bounds.Top + this.SquareHeight;
                        break;
                    case 63:
                        // Hidden square in the green goal
                        bounds.X = this._squares[62].Bounds.Left - this.SquareWidth;
                        bounds.Y = this._squares[62].Bounds.Top;
                        break;
                    case 69:
                        // Hidden square in the blue goal
                        bounds.X = this._squares[68].Bounds.Left;
                        bounds.Y = this._squares[68].Bounds.Top - this.SquareHeight;
                        break;
                }

                bounds.Width = this.SquareWidth;
                bounds.Height = this.SquareHeight;
                squareRender.Bounds = bounds;
            }

            // Set the bounds of the houses
            this.RedHouseRender.HouseBounds = new RectangleF(this._squares[41].Bounds.X, this._squares[0].Bounds.Y, this.SquareWidth * 6, this.SquareHeight * 6);
            this.RedHouseRender.WhiteBounds = new RectangleF(this.RedHouseRender.HouseBounds.X + this.ColoredHouseWidth, this.RedHouseRender.HouseBounds.Y + this.ColoredHouseHeight, this.RedHouseRender.HouseBounds.Width - 2 * this.ColoredHouseWidth, this.RedHouseRender.HouseBounds.Height - 2 * this.ColoredHouseHeight);            
            this.YellowHouseRender.HouseBounds = new RectangleF(this._squares[8].Bounds.X, this._squares[0].Bounds.Y, this.SquareWidth * 6, this.SquareHeight * 6);
            this.YellowHouseRender.WhiteBounds = new RectangleF(this.YellowHouseRender.HouseBounds.X + this.ColoredHouseWidth, this.YellowHouseRender.HouseBounds.Y + this.ColoredHouseHeight, this.YellowHouseRender.HouseBounds.Width - 2 * this.ColoredHouseWidth, this.YellowHouseRender.HouseBounds.Height - 2 * this.ColoredHouseHeight);
            this.GreenHouseRender.HouseBounds = new RectangleF(this.YellowHouseRender.HouseBounds.X, this._squares[21].Bounds.Y, this.SquareWidth * 6, this.SquareHeight * 6);
            this.GreenHouseRender.WhiteBounds = new RectangleF(this.GreenHouseRender.HouseBounds.X + this.ColoredHouseWidth, this.GreenHouseRender.HouseBounds.Y + this.ColoredHouseHeight, this.GreenHouseRender.HouseBounds.Width - 2 * this.ColoredHouseWidth, this.GreenHouseRender.HouseBounds.Height - 2 * this.ColoredHouseHeight);
            this.BlueHouseRender.HouseBounds = new RectangleF(this.RedHouseRender.HouseBounds.X, this.GreenHouseRender.HouseBounds.Y, this.SquareWidth * 6, this.SquareHeight * 6);
            this.BlueHouseRender.WhiteBounds = new RectangleF(this.BlueHouseRender.HouseBounds.X + this.ColoredHouseWidth, this.BlueHouseRender.HouseBounds.Y + this.ColoredHouseHeight, this.BlueHouseRender.HouseBounds.Width - 2 * this.ColoredHouseWidth, this.BlueHouseRender.HouseBounds.Height - 2 * this.ColoredHouseHeight);

            // Set the bounds of the goals            
            this.RedGoalRender.GoalBounds = new RectangleF(this._squares[51].Bounds.X, this._squares[46].Bounds.Y, this.SquareWidth * 1.5f, this.SquareHeight * 3);            
            this.YellowGoalRender.GoalBounds = new RectangleF(this._squares[51].Bounds.X, this._squares[46].Bounds.Y, this.SquareWidth * 3, this.SquareHeight * 1.5f);
            this.GreenGoalRender.GoalBounds = new RectangleF(this._squares[56].Bounds.X + this.SquareWidth * 0.5f, this._squares[13].Bounds.Y, this.SquareWidth * 1.5f, this.SquareHeight * 3);
            this.BlueGoalRender.GoalBounds = new RectangleF(this._squares[33].Bounds.X, this._squares[74].Bounds.Y + this.SquareHeight * 0.5f, this.SquareWidth * 3, this.SquareHeight * 1.5f);

            // Set the bounds of the pieces
            foreach (PieceRender pieceRender in this._pieces)
            {
                SquareRender square = this._squares[pieceRender.Piece.Square.Number];
                pieceRender.PieceBounds = square.Bounds;

                RectangleF pieceBounds = pieceRender.PieceBounds;
                pieceBounds.Size = pieceBounds.Size - square.Padding.Size;

                PointF pieceLocation = pieceBounds.Location;
                pieceLocation.X += square.Padding.Left;
                pieceLocation.Y += square.Padding.Top;
                pieceBounds.Location = pieceLocation;
                pieceRender.PieceBounds = pieceBounds;
            }
        }

        /// <summary>
        /// Raises the Paint event.
        /// </summary>
        /// <param name="e">A System.Windows.Forms.PaintEventArgs that contains the event data.</param>
        /// <remarks>
        /// <para>
        /// Raising an event invokes the event handler through a delegate. For more information, Raising an Event.
        /// </para>
        /// <para>
        /// The OnPaint method also allows derived classes to handle the event without attaching a delegate. This is the preferred technique for handling the event in a derived class.
        /// </para>
        /// <para>
        /// Note for Inheritors:
        /// When overriding OnPaint in a derived class, be sure to call the base class's OnPaint method so that registered delegates receive the event.
        /// </para>
        /// </remarks>
        protected override void OnPaint(System.Windows.Forms.PaintEventArgs e)
        {
            if (!this.Disposing)
            {
                base.OnPaint(e);
                Rectangle modifiedRectangle = new Rectangle(0, 0, this.Width, this.Height);
                this.BoardRender.DisplayRectangle = modifiedRectangle;
                this.BoardRender.Render(e);
                this.RedHouseRender.Render(e);
                this.YellowHouseRender.Render(e);
                this.GreenHouseRender.Render(e);
                this.BlueHouseRender.Render(e);
                this.RedGoalRender.Render(e);
                this.YellowGoalRender.Render(e);
                this.GreenGoalRender.Render(e);
                this.BlueGoalRender.Render(e);

                foreach (SquareRender square in this._squares)
                {
                    if (square.Square.Number != 75 && square.Square.Number != 69 && square.Square.Number != 63 && square.Square.Number != 57)
                        square.Render(e);

                    //switch (square.Square.Number)
                    //{
                    //    case 70:
                    //    case 71:
                    //    case 72:
                    //    case 73:
                    //    case 74:
                    //        square.Render(e);
                    //        break;
                    //}
                }

                foreach (PieceRender piece in this._pieces)
                {
                    piece.Render(e);
                }
            }
        }

        private void InitializeComponent()
        {
            this.SuspendLayout();
            // 
            // Board
            //             
            this.MouseClick += new System.Windows.Forms.MouseEventHandler(this.Board_MouseClick);
            this.ResumeLayout(false);

        }

        private void Board_MouseClick(object sender, MouseEventArgs e)
        {
            if (e.Button == MouseButtons.Left)
            {
                Square square = this.GetSquareAt(new PointF(e.X, e.Y));

                if (square != null && square.Pieces.Count > 0)
                {
                    // Check if one of these pieces can be moved
                    foreach (Piece piece in square.Pieces)
                    {
                        foreach (MoveablePiece moveablePiece in this._board.MoveablePieces)
                        {
                            if (moveablePiece.Piece == piece)
                            {
                                // Move piece
                                this._game.Engine.MovePiece(moveablePiece);
                                return;
                            }
                        }
                    }
                }
            }
        }

        /// <summary>
        /// Gets the <see cref="Square"/> at det given point.
        /// </summary>
        /// <param name="point">Point where mouse was clicked</param>
        /// <returns><see cref="Square"/> or null</returns>
        private Square GetSquareAt(PointF point)
        {
            for (int i = 0; i < this._squares.Count; i++)
            {
                if(this._squares[i].Bounds.Contains(point))
                {
                    return this._squares[i].Square;
                }
            }

            return null;
        }

    }
}
