using System;
using System.Collections.Generic;
using System.Text;
using Lokeiv.Games.Ludo.Model;

namespace Lokeiv.Games.Ludo.Model.ComputerPlayer
{
    class CompositeBestForPlayer : IMoveStrategy 
    {
        private List<IMoveStrategy> _moves = new List<IMoveStrategy>();

        #region Constructors
        public CompositeBestForPlayer()
        {

        }
        #endregion

        #region Private Methods
        
        /// <summary>
        /// Creates the right type of strategy and adds it to the list of moves.
        /// </summary>
        /// <param name="moveable"><see cref="MoveablePiece"/> representing the move</param>
        /// <param name="moveType">Type of move</param>
        private void CreateStrategy(MoveablePiece moveable, Moves moveType)
        {
            switch (moveType)
            {
                case Moves.CreateBlock:
                    this._moves.Add(new CreateBlockStrategy(moveable));
                    break;
                case Moves.Generic:
                    this._moves.Add(new GenericMoveStrategy(moveable));
                    break;
                case Moves.OutOfHouse:
                    this._moves.Add(new OutOfHouseStrategy(moveable));
                    break;
                case Moves.RemoveBlock:
                    this._moves.Add(new RemoveBlockStrategy(moveable));
                    break;
                case Moves.ToGoal:
                    this._moves.Add(new ToGoalStrategy(moveable));
                    break;
                case Moves.ToSecureSquare:
                    this._moves.Add(new ToSecureSquareStrategy(moveable));
                    break;
                case Moves.TurnIn:
                    this._moves.Add(new TurnInStrategy(moveable));
                    break;
            }
        }
        
        #endregion

        #region Public Methods

        /// <summary>
        /// Performs logic to create all possible stategies for the given <paramref name="movable"/>.
        /// </summary>
        /// <param name="movable"><see cref="MovablePiece"/> that describes a possible move.</param>
        public void AddCorrectMoveStrategy(MoveablePiece movable)
        {
            /* Create all possible strategies for the move
             *      Possible strategies are:
             *          TurnIn = 0, 
             *          OutOfHouse = 1, 
             *          CreateBlock = 2, 
             *          ToSecureSquare = 3, 
             *          ToGoal = 4, 
             *          Generic = 5, 
             *          RemoveBlock = 6
             */
            bool isNoOtherStrategy = true;

            if (movable.Piece.Square.IsHouse && movable.Square.IsCrossing)
            {
                // Move out of house
                this.CreateStrategy(movable, Moves.OutOfHouse);
                isNoOtherStrategy = false;
            }
            
            if(movable.Square.Pieces.Count > 0)
            {
                // Other pieces on the destination square
                bool isExistingOwnBlock = false;
                bool isExitingOpponentBlock = false;
                int numberOfOwnPieces = 0;
                int numberOfOpponentPieces = 0;

                foreach (Piece piece in movable.Square.Pieces)
                {
                    // Check if destination square pieces are only own color
                    if (piece.Color != movable.Piece.Color)                        
                        numberOfOpponentPieces++;                    

                    // Check if destination square pieces are only opponent color
                    if (piece.Color == movable.Piece.Color)
                        numberOfOwnPieces++;
                }

                // Check if destination square has existing own block                
                isExistingOwnBlock = (numberOfOwnPieces > 1);

                // Check if destination square has existing opponent block
                isExitingOpponentBlock = (numberOfOpponentPieces > 1);

                if (!isExitingOpponentBlock && numberOfOpponentPieces > 0)
                {
                    // Turn in
                    this.CreateStrategy(movable, Moves.TurnIn);
                    isNoOtherStrategy = false;
                }

                if (!isExitingOpponentBlock && numberOfOwnPieces > 0)
                {
                    // Create block (or add to existing block)
                    this.CreateStrategy(movable, Moves.CreateBlock);
                    isNoOtherStrategy = false;
                }

                if (!isExitingOpponentBlock) 
                {
                    bool isPieceSafeAlready = false;

                    // If currently on a square of its own color, that is not in the house and not the crossing, it is already safe.
                    if (movable.Piece.Square.Color == movable.Piece.Color && !movable.Piece.Square.IsHouse && !movable.Piece.Square.IsCrossing)
                        isPieceSafeAlready = true;

                    if (!isPieceSafeAlready)
                    {
                        // If moving to a square of its own color, that is not the crossing, it is moving from unsafe to safe square.
                        if (movable.Square.Color == movable.Piece.Color && !movable.Square.IsCrossing)
                        {
                            // To secure square
                            this.CreateStrategy(movable, Moves.ToSecureSquare);
                            isNoOtherStrategy = false;
                        }
                    }
                }

                if (movable.Square.IsGoal && movable.Square.Color == movable.Piece.Color)
                {
                    // To goal
                    this.CreateStrategy(movable, Moves.ToGoal);
                    isNoOtherStrategy = false;
                }
            }  // if(movable.Square.Pieces.Count > 0)

            // Check if an existing block will be removed
            if (movable.Piece.Square.Pieces.Count > 1)
            {
                int numberOfOwnPieces = 0;

                foreach(Piece piece in movable.Piece.Square.Pieces)
                {
                    if (piece.Color == movable.Piece.Color)
                        numberOfOwnPieces++;
                }

                if (numberOfOwnPieces > 1)
                {
                    // Remove block
                    this.CreateStrategy(movable, Moves.RemoveBlock);
                    isNoOtherStrategy = false;
                }
            }  // if (movable.Piece.Square.Pieces.Count > 1)

            if (isNoOtherStrategy)
            {
                // Generic move
                this.CreateStrategy(movable, Moves.Generic);
            }
        }

        /// <summary>
        /// Performs AI to select the best possible move.
        /// </summary>
        /// <returns><see cref="MovablePiece"/> for the movement to make.</returns>
        public MoveablePiece GetBestMove()
        {
            IMoveStrategy bestMove = null;

            // If there is no possible move, return null
            if (this._moves == null || this._moves.Count == 0)
                return null;

            // If there is only one possible move, use it.
            if (this._moves.Count == 1)
                return this._moves[0].MovablePiece;

            // Start with the first move as the best one, and see if any of the others are better
            bestMove = this._moves[0];

            // For each possible move, choose the best one
            foreach (IMoveStrategy move in this._moves)
            {
                if (move.GetType().Equals(typeof(TurnInStrategy)))  // The best move
                {
                    bestMove = move;
                }
                else if (move.GetType().Equals(typeof(OutOfHouseStrategy)))  // The second best move, and so on...
                {
                    if (!bestMove.GetType().Equals(typeof(TurnInStrategy)))
                        bestMove = move;
                }
                else if (move.GetType().Equals(typeof(CreateBlockStrategy)))
                {
                    if (!(bestMove.GetType().Equals(typeof(TurnInStrategy)) || bestMove.GetType().Equals(typeof(OutOfHouseStrategy))))
                        bestMove = move;
                }
                else if (move.GetType().Equals(typeof(ToSecureSquareStrategy)))
                {
                    if (!(bestMove.GetType().Equals(typeof(TurnInStrategy)) || bestMove.GetType().Equals(typeof(OutOfHouseStrategy))
                        || bestMove.GetType().Equals(typeof(CreateBlockStrategy))))
                        bestMove = move;
                }
                else if (move.GetType().Equals(typeof(ToGoalStrategy)))
                {
                    if (!(bestMove.GetType().Equals(typeof(TurnInStrategy)) || bestMove.GetType().Equals(typeof(OutOfHouseStrategy))
                        || bestMove.GetType().Equals(typeof(CreateBlockStrategy)) || bestMove.GetType().Equals(typeof(ToSecureSquareStrategy))))
                        bestMove = move;
                }
                else if (move.GetType().Equals(typeof(GenericMoveStrategy)))
                {
                    if (!(bestMove.GetType().Equals(typeof(TurnInStrategy)) || bestMove.GetType().Equals(typeof(OutOfHouseStrategy))
                        || bestMove.GetType().Equals(typeof(CreateBlockStrategy)) || bestMove.GetType().Equals(typeof(ToSecureSquareStrategy))
                        || bestMove.GetType().Equals(typeof(ToGoalStrategy))))
                        bestMove = move;
                }
                else if (move.GetType().Equals(typeof(RemoveBlockStrategy)))  // The worst move
                {
                    if (!(bestMove.GetType().Equals(typeof(TurnInStrategy)) || bestMove.GetType().Equals(typeof(OutOfHouseStrategy))
                        || bestMove.GetType().Equals(typeof(CreateBlockStrategy)) || bestMove.GetType().Equals(typeof(ToSecureSquareStrategy))
                        || bestMove.GetType().Equals(typeof(ToGoalStrategy)) || bestMove.GetType().Equals(typeof(RemoveBlockStrategy))))
                        bestMove = move;
                }
            }  // foreach
            
            return bestMove.MovablePiece;
        }

        #endregion

        #region IMoveStrategy Members

        public MoveablePiece MovablePiece
        {
            get 
            {
                return this.GetBestMove();
            }
        }

        #endregion
    }
}
