using System;
using System.Collections.Generic;
using System.Text;
using System.Threading;
using Lokeiv.Games.Ludo.Model;
using Lokeiv.Games.Dice;

namespace Lokeiv.Games.Ludo
{
    public class Ludo 
    {
        private Die _die;
        private Board _board;
        private Response _lastResponse;
        private bool _lastRollWasSix = false;
        private List<Player> _players;
        private int _numberOfPlayers;
        private int _currentPlayer;

        #region Public properties
        public Die Die
        {
            get { return this._die; }
        }

        public Board Board
        {
            get { return this._board; }
        }

        public List<Player> Players
        {
            get { return this._players; }
        }

        public Response LastResponse
        {
            get { return this._lastResponse; }
        }

        public Player CurrentPlayer
        {
            get { return this._players[this._currentPlayer]; }
        }

        #endregion

        public Ludo(List<Player> players, Die die)
        {
            if (die.GetType() == typeof(NumericOneToSixDie))
            {                
                this._players = players;
                this._numberOfPlayers = this._players.Count;
                this._currentPlayer = 0;
                this._die = die;
                this._board = new Board(players);
                this._lastResponse = new Response(die.Value, this._players[this._currentPlayer], Action.RollDie);
            }
            else
            {
                throw new ArgumentException(die.GetType().ToString());
            }
        }

        public void StartGame()
        {

        }

        public void EndGame()
        {

        }

        /// <summary>
        /// Returns true if all players are finished.
        /// </summary>
        public bool areAllFinished()
        {
            foreach (Player player in this._players)
            {
                if (!player.HasWon)
                {
                    return false;
                }
            }

            return true;
        }

        /// <summary>
        /// Moves a pice and returns a response with what's to happen next.
        /// </summary>
        /// <param name="movablePiece">Piece to move</param>
        /// <returns><see cref="Response">What's to happen next</see></returns>
        public Response MovePiece(MoveablePiece movablePiece)
        {
            if (this._lastResponse == null || this._lastResponse.NextAction != Action.MovePiece)
            {
                return null;  // A piece should not be moved now
            }

            Response response = new Response();
            List<MoveablePiece> moveablePieces = this._board.MovePiece(movablePiece);

            // Save new position of moved pieces
            response.MovablePieces.Add(movablePiece);

            if(this._board.HasFinished(movablePiece.Piece.Color))
            {
                this._players[this._currentPlayer - 1].SetWon(true);
                response.PlayerWon = this._players[this._currentPlayer];
            }

            // Pass on to the next player
            this.NextPlayersTurn();

            response.PlayersTurn = this._players[this._currentPlayer];
            response.NextAction = Action.RollDie;
            this._lastResponse = response;

            return response;
        }

        /// <summary>
        /// Rolls the die and returns a response with what's to happen next.
        /// </summary>
        /// <returns><see cref="Response">What's to happen next</see></returns>
        public Response RollDie()
        {
            if (this._lastResponse == null || this._lastResponse.NextAction != Action.RollDie)
            {
                return null;  // The die should not be rolled now
            }

            Response response = new Response();
            response.DieValue = this._die.Roll();

            if (this._die.GetType() != typeof(NumericOneToSixDie))
            {
                throw new Exception(this._die.GetType().ToString());
            }

            if ((int)this._die.Value.Value == 6)
            {
                this._lastRollWasSix = true;
            }
            else
            {
                this._lastRollWasSix = false;
            }
      
            response.MovablePieces = this._board.GetPossibleMoves(this._die, this._players[this._currentPlayer].Color);

            if (response.MovablePieces.Count == 0)
            {
                // No possible moves, pass on to the next player
                this.NextPlayersTurn();                
                response.NextAction = Action.RollDie;
            }
            else
            {
                // Possible moves
                response.NextAction = Action.MovePiece;
            }
            
            response.PlayersTurn = this._players[this._currentPlayer];
            this._lastResponse = response;

            return response;
        }

        /// <summary>
        /// Finds which player is next.
        /// </summary>
        private void NextPlayersTurn()
        {
            if (!this._lastRollWasSix)
            {
                bool passOn = true;
                int index = 0;

                while (passOn)
                {
                    this._currentPlayer++;

                    if (this._currentPlayer > this._numberOfPlayers - 1)
                    {
                        this._currentPlayer = 0;
                    }

                    if (!this._players[this._currentPlayer].HasWon)
                    {
                        // It's this players turn
                        passOn = false;
                    }

                    index++;

                    if (index >= 4)
                    {
                        // All players have finished, no players can keep on
                        passOn = false;
                    }
                }
            }
        }
    }
}
