using System;
using System.Collections.Generic;
using System.Text;
using System.Xml.Serialization;

namespace Lokeiv.Games.Ludo.Model
{
    public class Piece : IXmlSerializable
    {
        public EventHandler SquareChanged;

        #region Properties
        private Guid _pieceId = Guid.Empty;
        public Guid PieceId
        {
            get { return this._pieceId; }
        }

        private int _number;
        public int Number
        {
            get { return this._number; }
        }

        private Color _color = Color.White;
        public Color Color
        {
            get { return this._color; }
        }

        private Boolean _readyForFinish = false;
        public Boolean IsReadyForFinish
        {
            get { return this._readyForFinish; }
        }

        private Square _square = null;
        public Square Square
        {
            get { return this._square; }
        }

        private int _numberOfMovesLeft;
        public int NumberOfMovesLeft
        {
            get { return this._numberOfMovesLeft; }
        }
        #endregion

        #region Constructors
        /// <summary>
        /// Constructor.        
        /// </summary>
        /// <remarks>Non-public default constructor for deserialization.</remarks>
        private Piece() 
        {
            this._pieceId = Guid.NewGuid();
        }

        /// <summary>
        /// Constructor used to deserialize the object.
        /// </summary>
        public Piece(Guid pieceId, Color color, bool readyForFinish, Square square)
        {
            this._pieceId = pieceId;
            this._color = color;
            this._readyForFinish = readyForFinish;
            this._square = square;
        }

        /// <summary>
        /// Constructor.
        /// </summary>
        /// <param name="color"><see cref="Color"/> of this piece.</param>
        internal Piece(Color color, int number) : this()
        {
            this._color = color;
            this._number = number;
        }
        #endregion

        protected void OnSquareChanged(EventArgs e)
        {
            if (this.SquareChanged != null)
            {
                this.SquareChanged(this, e);
            }
        }

        #region Internal Methods
        internal void SetReadyForFinish(Boolean isReadyForFinish)
        {
            this._readyForFinish = isReadyForFinish;
        }

        /// <summary>
        /// Sets the <see cref="Square"/> that this <see cref="Piece"/> is at, 
        /// and updates the <see cref="Square"/> about the <see cref="Piece"/> that is on it.
        /// </summary>
        /// <param name="square">The <see cref="Square"/> that this <see cref="Piece"/> is at</param>
        internal void SetSquare(Square square)
        {
            // Todo: Run this code within a transaction
            if (this._square != null)
                this._square.Pieces.Remove(this);

            this._square = square;
            this._square.Pieces.Add(this);
            this._numberOfMovesLeft = this.CalculateNumberOfMovesLeft();
            this.OnSquareChanged(EventArgs.Empty);
        }

        /// <summary>
        /// Calculates number of moves left based on the <see cref="Color"/> and <see cref="Square"/> properties.
        /// </summary>
        /// <returns>Number of moves left.</returns>
        /// <remarks>This method must be refactored into the rules when rules are implemented.</remarks>
        private int CalculateNumberOfMovesLeft()
        {
            int maxNumberOfMoves = 59;

            if (Square == null) return maxNumberOfMoves;

            switch (Color)
            {
                case Model.Color.Red:
                    if (Square.Number <= 91 && Square.Number >= 88)
                    {
                        // 88 - 91 (in house)
                        return maxNumberOfMoves;
                    }
                    else if (Square.Number == 42)
                    {
                        // Red crossing
                        if (this.IsReadyForFinish)
                            return 6;
                        else
                            return maxNumberOfMoves - 1;
                    }
                    else if (Square.Number <= 75 && Square.Number >= 70)
                    {
                        // 70 - 75 (red path to goal)
                        return 6 - (Square.Number - 69);
                    }
                    else if (Square.Number <= 51 && Square.Number >= 43)
                    {
                        // 43 - 51
                        return maxNumberOfMoves - (Square.Number - 42);
                    }
                    else
                    {
                        return maxNumberOfMoves - 11 - Square.Number;
                    }
                case Model.Color.Yellow:
                    if (Square.Number <= 79 && Square.Number >= 76)
                    {
                        // 76 - 79 (in house)
                        return maxNumberOfMoves;
                    }
                    else if (Square.Number == 3)
                    {
                        // Yellow crossing
                        if (this.IsReadyForFinish)
                            return 6;
                        else
                            return maxNumberOfMoves - 1;
                    }
                    else if (Square.Number <= 57 && Square.Number >= 52)
                    {
                        // 52 - 57 (yellow path to goal)
                        return 6 - (Square.Number - 51);
                    }
                    else if (Square.Number <= 51 && Square.Number >= 4)
                    {
                        // 4 - 51
                        return maxNumberOfMoves + 2 - Square.Number;
                    }
                    else
                    {
                        // 0 - 2
                        return 9 - Square.Number;
                    }
                case Model.Color.Green:
                    if (Square.Number <= 83 && Square.Number >= 80)
                    {
                        // 80 - 83 (in house)
                        return maxNumberOfMoves;
                    }
                    else if (Square.Number == 16)
                    {
                        // Green crossing
                        if (this.IsReadyForFinish)
                            return 6;
                        else
                            return maxNumberOfMoves - 1;
                    }
                    else if (Square.Number <= 63 && Square.Number >= 58)
                    {
                        // 58 - 63 (green path to goal)
                        return 6 - (Square.Number - 57);
                    }
                    else if (Square.Number <= 51 && Square.Number >= 17)
                    {
                        // 17 - 51
                        return maxNumberOfMoves + 15 - Square.Number;
                    }
                    else
                    {
                        // 0 - 15
                        return 22 - Square.Number;
                    }
                case Model.Color.Blue:
                    if (Square.Number <= 87 && Square.Number >= 84)
                    {
                        // 84 - 87 (in house)
                        return maxNumberOfMoves;
                    }
                    else if (Square.Number == 29)
                    {
                        // Blue crossing
                        if (this.IsReadyForFinish)
                            return 6;
                        else
                            return maxNumberOfMoves - 1;
                    }
                    else if (Square.Number <= 69 && Square.Number >= 64)
                    {
                        // 64 - 69 (blue path to goal)
                        return 6 - (Square.Number - 63);
                    }
                    else if (Square.Number <= 51 && Square.Number >= 30)
                    {
                        // 30 - 51
                        return maxNumberOfMoves + 28 - Square.Number;
                    }
                    else
                    {
                        // 0 - 28
                        return 35 - Square.Number;
                    }
            }

            return maxNumberOfMoves;
        }
        #endregion

        #region Public Methods
        /// <summary>
        /// Gets the number of pieces of the same color as this one that's on the same square as this one.
        /// </summary>        
        /// <returns><see cref="int"/> number of <see cref="Piece"/>s with the same <see cref="Color"/>.</returns>
        public int NumberOfSamePiecesOnSameSquare()
        {
            var piecesOnSquare = _square.Pieces;
            if (piecesOnSquare.Count == 1) return 1;
            int counter = 0;

            foreach (Piece pieceOnSquare in piecesOnSquare)
            {
                if (pieceOnSquare.Color == _color)
                    counter++;
            }

            return counter;
        }
        #endregion

        #region IXmlSerializable
        public System.Xml.Schema.XmlSchema GetSchema()
        {
            return null;
        }

        public void ReadXml(System.Xml.XmlReader reader)
        {
            throw new NotImplementedException();
        }

        public static Piece ReadXml(System.Xml.XmlReader reader, Board board)
        {
            reader.MoveToContent();

            // Read PieceId reference 
            Guid pieceId = new Guid(reader.GetAttribute("PieceId"));
            Piece piece = board.Pieces[pieceId];

            if (piece != null)
                return piece;

            // This was not a reference to an existing piece, so read rest of the content
            piece = new Piece(pieceId, Color.White, false, null);
            piece._pieceId = pieceId;
            
            string number = reader.GetAttribute("Number");
            piece._number = int.Parse(number);

            string color = reader.GetAttribute("Color");
            piece._color = (Color)Enum.Parse(typeof(Color), color, true);

            string isReadyForFinish = reader.GetAttribute("IsReadyForFinish");
            piece._readyForFinish = bool.Parse(isReadyForFinish);

            string numberOfMovesLeft = reader.GetAttribute("NumberOfMovesLeft");
            piece._numberOfMovesLeft = int.Parse(numberOfMovesLeft);

            while (reader.Read())
            {
                if (reader.NodeType == System.Xml.XmlNodeType.Element)
                {
                    switch(reader.Name.ToLower())
                    {
                        case "square":
                            piece._square = Square.ReadXmlReference(reader.ReadSubtree(), board);
                            reader.Skip();
                            break;
                    }
                }
            }

            return piece;
        }

        public void WriteXml(System.Xml.XmlWriter writer)
        {
            this.WriteXmlReference(writer);
            writer.WriteAttributeString("Number", this.Number.ToString());
            writer.WriteAttributeString("Color", this.Color.ToString());
            writer.WriteAttributeString("IsReadyForFinish", this.IsReadyForFinish.ToString());
            writer.WriteAttributeString("NumberOfMovesLeft", this.NumberOfMovesLeft.ToString());
    
            // Square reference
            writer.WriteStartElement("Square");
            this.Square.WriteXmlReference(writer);
            writer.WriteEndElement();
        }

        public void WriteXmlReference(System.Xml.XmlWriter writer)
        {
            writer.WriteAttributeString("PieceId", this.PieceId.ToString());
        }
        #endregion
    }
}
