using System;
using System.Collections.Generic;
using System.Text;
using System.Xml.Serialization;

namespace Lokeiv.Games.Ludo.Model
{
    public class Player : IXmlSerializable
    {

        #region Events
        public EventHandler<EventArgs> ProgressChanged;

        public event EventHandler PlayerTypeChanged;

        protected void OnPlayerTypeChanged(EventArgs e)
        {
            if (this.PlayerTypeChanged != null)
                this.PlayerTypeChanged(this, e);
        }
        #endregion

        #region Properties
        private Guid _playerId = Guid.Empty;
        public Guid PlayerId
        {
            get { return this._playerId; }            
        }

        private PlayerType _type = PlayerType.Human;
        public PlayerType Type
        {
            get { return this._type; }
            set 
            {
                if (this._type != value)
                {
                    this._type = value;
                    this.OnPlayerTypeChanged(EventArgs.Empty);
                }                
            }
        }

        private String _name = String.Empty;
        public String Name
        {
            get { return this._name; }
            set { this._name = value; }
        }

        private Color _color = Color.White;
        public Color Color
        {
            get { return this._color; }
        }

        private Boolean _hasWon = false;
        public Boolean HasWon
        {
            get { return this._hasWon; }
        }

        private int _numberOfMovesLeft = 0;
        public int NumberOfMovesLeft
        {
            get { return this._numberOfMovesLeft; }
        }
        
        private Action _currentAction = Action.Nothing;        
        public Action CurrentAction
        {
            get { return this._currentAction; }
            set { this._currentAction = value; }
        }
        
        public double Progress
        {
            get { return (double)((236d - (double)this._numberOfMovesLeft) * 100d / 236d); }
        }
        #endregion

        #region Contructors
        /// <summary>
        /// Non-public default constructor for deserialization.
        /// </summary>
        private Player()
        {
            this._playerId = Guid.NewGuid();
        }

        /// <summary>
        /// Constructor used to deserialize the object.
        /// </summary>
        public Player(Guid playerId, string name, PlayerType type, Color color, bool hasWon, Action currentAction, int numberOfMovesLeft)
        {
            this._playerId = playerId;
            this._name = name;
            this._type = type;
            this._color = color;
            this._hasWon = hasWon;
            this._numberOfMovesLeft = numberOfMovesLeft;
        }

        public Player(String name, PlayerType type, Color color) : this()
        {
            this._name = name;
            this._type = type;
            this._color = color;
            this._hasWon = false;
            this._currentAction = Action.Nothing;
            this._numberOfMovesLeft = 236;
        }
        #endregion

        #region Internal Methods
        internal void SetColor(Color color)
        {
            this._color = color;
        }
        #endregion

        /// <summary>
        /// Sets <see cref="NumberOfMovesLeft"/>, which updates <see cref="Progress"/> and raises the <see cref="ProgressChanged"/> event.
        /// </summary>
        /// <param name="numberOfMovesLeft">Number of moves left to finish</param>
        public void SetNumberOfMovesLeft(int numberOfMovesLeft)
        {
            this._numberOfMovesLeft = numberOfMovesLeft;
            this.OnProgressChanged(EventArgs.Empty);
        }

        /// <summary>
        /// Sets if the player has finished.
        /// </summary>
        /// <param name="hasWon"><see cref="Boolean"/></param>
        public void SetWon(Boolean hasWon)
        {
            this._hasWon = hasWon;
        }

        /// <summary>
        /// Sorts a list of players using the <see cref="Comparison">CompareByColor</see>.
        /// </summary>
        /// <param name="players">The list of players to sort</param>
        public static void SortPlayersList(List<Player> players)
        {
            Comparison<Player> compareByColor = new Comparison<Player>(Player.CompareByColor);
            players.Sort(compareByColor);
        }

        /// <summary>
        /// Used to create a new <see cref="Comparison"/> of <see cref="Player"/>, for
        /// sorting a list of players by color.
        /// </summary>
        /// <param name="player1"><see cref="Player"/></param>
        /// <param name="player2"><see cref="Player"/></param>
        /// <returns><see cref="int"/></returns>
        public static int CompareByColor(Player player1, Player player2)
        {
            if (player1.Color == player2.Color)
                return 0;  // Equal

            if (player1.Color < player2.Color)
                return -1;  // Player 1 should play before player 2

            return 1;
        }

        /// <summary>
        /// Raises the <see cref="ProgressChanged"/> event.
        /// </summary>
        /// <param name="e"><see cref="EventArgs"/></param>
        protected void OnProgressChanged(EventArgs e)
        {
            if (ProgressChanged != null)
                ProgressChanged(this, e);
        }

        #region IXmlSerializable
        public System.Xml.Schema.XmlSchema GetSchema()
        {
            return null;
        }

        public void ReadXml(System.Xml.XmlReader reader)
        {
            reader.MoveToContent();

            string playerId = reader.GetAttribute("PlayerId");
            this._playerId = new Guid(playerId);

            string type = reader.GetAttribute("Type");
            this._type = (PlayerType)Enum.Parse(typeof(PlayerType), type, true);

            this._name = reader.GetAttribute("Name");

            string color = reader.GetAttribute("Color");
            this._color = (Model.Color)Enum.Parse(typeof(Model.Color), color, true);

            string hasWon = reader.GetAttribute("HasWon");
            this._hasWon = bool.Parse(hasWon);

            string numberOfMovesLeft = reader.GetAttribute("NumberOfMovesLeft");
            this._numberOfMovesLeft = int.Parse(numberOfMovesLeft);

            string currentAction = reader.GetAttribute("CurrentAction");
            this._currentAction = (Action)Enum.Parse(typeof(Action), currentAction, true);
        }

        public static Player ReadXml(System.Xml.XmlReader reader, PlayerCollection players)
        {
            reader.MoveToContent();

            if (reader.HasAttributes)
            {
                Guid playerId = new Guid(reader.GetAttribute("PlayerId"));
                return players[playerId];
            }

            return null;
        }

        public static PlayerCollection ReadCollectionFromXml(System.Xml.XmlReader reader)
        {
            PlayerCollection players = new PlayerCollection();

            while(reader.Read())
            {
                if (reader.NodeType == System.Xml.XmlNodeType.Element)
                {
                    switch(reader.Name.ToLower())
                    {
                        case "player":
                            Player player = new Player();
                            player.ReadXml(reader.ReadSubtree());
                            reader.Skip();
                            players.Add(player);
                            break;
                    }
                }
            }

            return players;
        }

        public void WriteXml(System.Xml.XmlWriter writer)
        {
            this.WriteXmlReference(writer);
            writer.WriteAttributeString("Type", this.Type.ToString());
            writer.WriteAttributeString("Name", this.Name.ToString());
            writer.WriteAttributeString("Color", this.Color.ToString());
            writer.WriteAttributeString("HasWon", this.HasWon.ToString());
            writer.WriteAttributeString("NumberOfMovesLeft", this.NumberOfMovesLeft.ToString());
            writer.WriteAttributeString("CurrentAction", this.CurrentAction.ToString());
        }

        public void WriteXmlReference(System.Xml.XmlWriter writer)
        {
            writer.WriteAttributeString("PlayerId", this.PlayerId.ToString());
        }
        #endregion
    }
}
