﻿using System;
using System.Collections.Generic;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Ink;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using System.Xml.Serialization;

namespace Ludo.WP7.Client.Utils.BoardCoordinates
{
    public abstract class BoardCoordinates
    {

        #region Properties

        #region Public properties
        
        /// <summary>
        /// Gets or sets Board.X.
        /// </summary>
        public float X { get; set; }

        /// <summary>
        /// Gets or sets Board.Y.
        /// </summary>
        public float Y { get; set; }
        
        /// <summary>
        /// Gets or sets Board.Width.
        /// </summary>
        public float Width { get; set; }
        
        /// <summary>
        /// Gets or sets Board.Height.
        /// </summary>
        public float Height { get; set; }
        
        /// <summary>
        /// Gets bounds for all squares.
        /// </summary>
        public Rect[] SquareBounds { get; protected set; }
        
        /// <summary>
        /// Gets bounds for the outer part of the red house.
        /// </summary>        
        public Rect RedHouseBounds { get; protected set; }

        /// <summary>
        /// Gets bounds for the inner part of the red house.
        /// </summary>        
        public Rect RedHouseWhiteBounds { get; protected set; }

        /// <summary>
        /// Gets bounds for the outer part of the yellow house.
        /// </summary>        
        public Rect YellowHouseBounds { get; protected set; }

        /// <summary>
        /// Gets bounds for the inner part of the yellow house.
        /// </summary>        
        public Rect YellowHouseWhiteBounds { get; protected set; }

        /// <summary>
        /// Gets bounds for the outer part of the green house.
        /// </summary>        
        public Rect GreenHouseBounds { get; protected set; }

        /// <summary>
        /// Gets bounds for the inner part of the green house.
        /// </summary>        
        public Rect GreenHouseWhiteBounds { get; protected set; }

        /// <summary>
        /// Gets bounds for the outer part of the blue house.
        /// </summary>        
        public Rect BlueHouseBounds { get; protected set; }

        /// <summary>
        /// Gets bounds for the inner part of the blue house.
        /// </summary>        
        public Rect BlueHouseWhiteBounds { get; protected set; }

        /// <summary>
        /// Gets bounds for the red goal.
        /// </summary>
        public Rect RedGoalBounds { get; protected set; }
        
        /// <summary>
        /// Gets bounds for the yellow goal.
        /// </summary>
        public Rect YellowGoalBounds { get; protected set; }
        
        /// <summary>
        /// Gets bounds for the green goal.
        /// </summary>
        public Rect GreenGoalBounds { get; protected set; }

        /// <summary>
        /// Gets bounds for the blue goal.
        /// </summary>
        public Rect BlueGoalBounds { get; protected set; }
        #endregion  // Public properties

        #region None public properties

        /// <summary>
        /// Gets or sets the height of a square.
        /// </summary>
        protected float SquareHeight { get; set; }

        /// <summary>
        /// Gets or sets the width of a square.
        /// </summary>
        protected float SquareWidth { get; set; }

        /// <summary>
        /// Gets or sets the X coordinate of square 1 from which all other
        /// square coordinates are calculated.
        /// </summary>
        protected float Square1Left { get; set; }

        /// <summary>
        /// Gets or sets the Y coordinate of square 1 from which all other
        /// square coordinates are calculated.
        /// </summary>
        protected float Square1Top { get; set; }

        /// <summary>
        /// Gets or sets the height of the outer bounds of a house.
        /// </summary>
        protected float ColoredHouseHeight { get; set; }

        /// <summary>
        /// Gets or sets the width of the outer bounds of a house.
        /// </summary>
        protected float ColoredHouseWidth { get; set; }

        /// <summary>
        /// Gets or sets the height of the inner bounds of a house.
        /// </summary>
        protected float WhiteHouseHeight { get; set; }

        /// <summary>
        /// Gets or sets the width of the inner bounds of a house.
        /// </summary>
        protected float WhiteHouseWidth { get; set; }

        /// <summary>
        /// Gets or sets the vertical space between squares inside a house.
        /// </summary>        
        protected float BetweenSquaresInHouseHeight { get; set; }

        /// <summary>
        /// Gets or sets the horizontal space between squares inside a house.
        /// </summary>
        protected float BetweenSquaresInHouseWidth { get; set; }

        /// <summary>
        /// Gets or sets the total height of a house.
        /// </summary>
        protected float HouseHeight { get; set; }

        /// <summary>
        /// Gets or sets the total width of a house.
        /// </summary>
        protected float HouseWidth { get; set; }

        #endregion  // None public properties

        #endregion  // Properties

        #region Constructors
        /// <summary>
        /// Constructor.
        /// </summary>
        public BoardCoordinates()
        {
            // The default layout has 69 squares
            this.SquareBounds = new Rect[92];
        }
        #endregion  // Constructors

        #region Methods

        #region Factory methods
        /// <summary>
        /// Creates an instance of <see cref="BoardCoordinates"/> for the given <paramref name="view"/>.
        /// </summary>
        /// <param name="view"><see cref="Views"/> for which to create <see cref="BoardCoordinates"/>.</param>
        /// <returns><see cref="BoardCoordinates"/></returns>
        public static BoardCoordinates CreateBoardCoordinates(Views view)
        {
            switch (view)
            {
                case Views.WindowsPhone7Portrait:
                    return new WinPhone7PortraitBoardCoordinates();                    
                case Views.WindowsPhone7Landscape:
                    break;
            }

            return null;
        }
        #endregion  // Factory methods

        #region Calculation methods

        /// <summary>
        /// Finds the best values for the size of squares etc.
        /// </summary>
        public virtual void CalculateBoardSize()
        {
            // Set a static board size until support for dynamic board size is implemented.
            this.SetBoardSize();

            // All methods called below implements a default calculation that 
            // may work with most views, but may be overridden for special layouts.

            // Calculates the basic bounds to be used to set bounds for all content.
            this.CalcualteBasicBounds();

            // Update bounds of all squares
            this.UpdateSquareBounds();

            // Updates bounds of all houses
            this.UpdateHouseBounds();

            // Updates bounds of all goals
            this.UpdateGoalBounds();
        }

        /// <summary>
        /// Used to set a static board size. Initializes these properties:
        /// <see cref="X"/>, <see cref="Y"/>, <see cref="Width"/> and <see cref="Height"/>.
        /// </summary>
        protected abstract void SetBoardSize();

        /// <summary>
        /// Calculates the basic bounds to be used to set bounds for all content.
        /// </summary>
        protected virtual void CalcualteBasicBounds()
        {
            this.SquareHeight = this.Height / 15f;
            this.SquareWidth = this.Width / 15f;
            this.Square1Left = this.SquareWidth * 6f;
            this.Square1Top = 0;
            this.ColoredHouseHeight = this.SquareHeight;
            this.ColoredHouseWidth = this.SquareWidth;
            this.WhiteHouseHeight = this.SquareHeight * 2f / 3f;
            this.WhiteHouseWidth = this.SquareWidth * 2f / 3f;
            this.BetweenSquaresInHouseHeight = this.WhiteHouseHeight;
            this.BetweenSquaresInHouseWidth = this.WhiteHouseWidth;
            this.HouseHeight = this.ColoredHouseHeight + this.WhiteHouseHeight;
            this.HouseWidth = this.ColoredHouseWidth + this.WhiteHouseWidth;
        }

        /// <summary>
        /// Updates the bounds for every square when the board size is calculated.
        /// </summary>
        protected virtual void UpdateSquareBounds()
        {
            for (int index = 0; index < this.SquareBounds.Length; index++)
            {
                Rect bounds = new Rect();

                switch (index)
                {
                    case 0:
                    case 1:
                    case 2:
                        // The three white squares at the top of the board
                        bounds.X = this.Square1Left + index * this.SquareWidth;
                        bounds.Y = this.Square1Top;
                        break;
                    case 3:
                    case 4:
                    case 5:
                    case 6:
                    case 7:
                        // To the left of the yellow house
                        bounds.X = this.SquareBounds[index - 1].X;
                        bounds.Y = this.SquareBounds[index - 1].Y + this.SquareHeight;
                        break;
                    case 8:
                        // First square beneath the yellow house
                        bounds.X = this.SquareBounds[index - 1].X + this.SquareWidth;
                        bounds.Y = this.SquareBounds[index - 1].Y + this.SquareHeight;
                        break;
                    case 9:
                    case 10:
                    case 11:
                    case 12:
                    case 13:
                        // Beneath the yellow house
                        bounds.X = this.SquareBounds[index - 1].X + this.SquareWidth;
                        bounds.Y = this.SquareBounds[index - 1].Y;
                        break;
                    case 14:
                    case 15:
                        // Down towards the green house
                        bounds.X = this.SquareBounds[index - 1].X;
                        bounds.Y = this.SquareBounds[index - 1].Y + this.SquareHeight;
                        break;
                    case 16:
                    case 17:
                    case 18:
                    case 19:
                    case 20:
                        // Over the green house
                        bounds.X = this.SquareBounds[index - 1].X - this.SquareWidth;
                        bounds.Y = this.SquareBounds[index - 1].Y;
                        break;
                    case 21:
                        // First square to the left of the green house
                        bounds.X = this.SquareBounds[index - 1].X - this.SquareWidth;
                        bounds.Y = this.SquareBounds[index - 1].Y + this.SquareHeight;
                        break;
                    case 22:
                    case 23:
                    case 24:
                    case 25:
                    case 26:
                        // To the left of the green house
                        bounds.X = this.SquareBounds[index - 1].X;
                        bounds.Y = this.SquareBounds[index - 1].Y + this.SquareHeight;
                        break;
                    case 27:
                    case 28:
                        // Left towards the blue house
                        bounds.X = this.SquareBounds[index - 1].X - this.SquareWidth;
                        bounds.Y = this.SquareBounds[index - 1].Y;
                        break;
                    case 29:
                    case 30:
                    case 31:
                    case 32:
                    case 33:
                        // To the right of the blue house
                        bounds.X = this.SquareBounds[index - 1].X;
                        bounds.Y = this.SquareBounds[index - 1].Y - this.SquareHeight;
                        break;
                    case 34:
                        // First square over the blue house
                        bounds.X = this.SquareBounds[index - 1].X - this.SquareWidth;
                        bounds.Y = this.SquareBounds[index - 1].Y - this.SquareHeight;
                        break;
                    case 35:
                    case 36:
                    case 37:
                    case 38:
                    case 39:
                        // Over the blue house
                        bounds.X = this.SquareBounds[index - 1].X - this.SquareWidth;
                        bounds.Y = this.SquareBounds[index - 1].Y;
                        break;
                    case 40:
                    case 41:
                        // Up towards the red house
                        bounds.X = this.SquareBounds[index - 1].X;
                        bounds.Y = this.SquareBounds[index - 1].Y - this.SquareHeight;
                        break;
                    case 42:
                    case 43:
                    case 44:
                    case 45:
                    case 46:
                        // Under the red house
                        bounds.X = this.SquareBounds[index - 1].X + this.SquareWidth;
                        bounds.Y = this.SquareBounds[index - 1].Y;
                        break;
                    case 47:
                        // First square to the right of the red house
                        bounds.X = this.SquareBounds[index - 1].X + this.SquareWidth;
                        bounds.Y = this.SquareBounds[index - 1].Y - this.SquareHeight;
                        break;
                    case 48:
                    case 49:
                    case 50:
                    case 51:
                        // To the right of the red house
                        bounds.X = this.SquareBounds[index - 1].X;
                        bounds.Y = this.SquareBounds[index - 1].Y - this.SquareHeight;
                        break;
                    case 52:
                    case 53:
                    case 54:
                    case 55:
                    case 56:
                        // Path to yellow goal
                        bounds.X = this.SquareBounds[index - 49].X - this.SquareWidth;
                        bounds.Y = this.SquareBounds[index - 49].Y;
                        break;
                    case 58:
                    case 59:
                    case 60:
                    case 61:
                    case 62:
                        // Path to the green goal
                        bounds.X = this.SquareBounds[index - 42].X;
                        bounds.Y = this.SquareBounds[index - 42].Y - this.SquareHeight;
                        break;
                    case 64:
                    case 65:
                    case 66:
                    case 67:
                    case 68:
                        // Path to the blue goal
                        bounds.X = this.SquareBounds[index - 35].X + this.SquareWidth;
                        bounds.Y = this.SquareBounds[index - 35].Y;
                        break;
                    case 70:
                    case 71:
                    case 72:
                    case 73:
                    case 74:
                        // Path to the red goal
                        bounds.X = this.SquareBounds[index - 28].X;
                        bounds.Y = this.SquareBounds[index - 28].Y + this.SquareHeight;
                        break;
                    case 88:
                        // In the red house
                        bounds.X = this.HouseWidth;
                        bounds.Y = this.HouseHeight + this.SquareHeight + this.BetweenSquaresInHouseHeight;
                        break;
                    case 89:
                        // In the red house
                        bounds.X = this.HouseWidth;
                        bounds.Y = this.HouseHeight;
                        break;
                    case 90:
                        // In the red house
                        bounds.X = this.SquareBounds[88].X + this.SquareWidth + this.BetweenSquaresInHouseWidth;
                        bounds.Y = this.SquareBounds[88].Y;
                        break;
                    case 91:
                        // In the red house
                        bounds.X = this.SquareBounds[90].X;
                        bounds.Y = this.SquareBounds[89].Y;
                        break;
                    case 76:
                        // In the yellow house
                        bounds.X = this.SquareBounds[2].X + this.SquareWidth + this.HouseWidth;
                        bounds.Y = this.HouseHeight;
                        break;
                    case 77:
                        // In the yellow house
                        bounds.X = this.SquareBounds[76].X + this.SquareWidth + this.BetweenSquaresInHouseWidth;
                        bounds.Y = this.HouseHeight;
                        break;
                    case 78:
                        // In the yellow house
                        bounds.X = this.SquareBounds[76].X;
                        bounds.Y = this.SquareBounds[76].Y + this.SquareHeight + this.BetweenSquaresInHouseHeight;
                        break;
                    case 79:
                        // In the yellow house
                        bounds.X = this.SquareBounds[77].X;
                        bounds.Y = this.SquareBounds[78].Y;
                        break;
                    case 80:
                        // In the green house
                        bounds.X = this.SquareBounds[77].X;
                        bounds.Y = this.SquareBounds[15].Y + this.SquareHeight + this.HouseHeight;
                        break;
                    case 81:
                        // In the green house
                        bounds.X = this.SquareBounds[80].X;
                        bounds.Y = this.SquareBounds[80].Y + this.SquareHeight + this.BetweenSquaresInHouseHeight;
                        break;
                    case 82:
                        // In the green house
                        bounds.X = this.SquareBounds[76].X;
                        bounds.Y = this.SquareBounds[80].Y;
                        break;
                    case 83:
                        // In the green house
                        bounds.X = this.SquareBounds[82].X;
                        bounds.Y = this.SquareBounds[81].Y;
                        break;
                    case 84:
                        // In the blue house
                        bounds.X = this.HouseWidth + this.SquareWidth + this.BetweenSquaresInHouseWidth;
                        bounds.Y = this.SquareBounds[83].Y;
                        break;
                    case 85:
                        // In the blue house
                        bounds.X = this.HouseWidth;
                        bounds.Y = this.SquareBounds[84].Y;
                        break;
                    case 86:
                        // In the blue house
                        bounds.X = this.SquareBounds[84].X;
                        bounds.Y = this.SquareBounds[82].Y;
                        break;
                    case 87:
                        // In the blue house
                        bounds.X = this.SquareBounds[85].X;
                        bounds.Y = this.SquareBounds[86].Y;
                        break;
                    case 75:
                        // Hidden square in the red goal
                        bounds.X = this.SquareBounds[74].Left + this.SquareWidth;
                        bounds.Y = this.SquareBounds[74].Top;
                        break;
                    case 57:
                        // Hidden square in the yellow goal
                        bounds.X = this.SquareBounds[56].Left;
                        bounds.Y = this.SquareBounds[56].Top + this.SquareHeight;
                        break;
                    case 63:
                        // Hidden square in the green goal
                        bounds.X = this.SquareBounds[62].Left - this.SquareWidth;
                        bounds.Y = this.SquareBounds[62].Top;
                        break;
                    case 69:
                        // Hidden square in the blue goal
                        bounds.X = this.SquareBounds[68].Left;
                        bounds.Y = this.SquareBounds[68].Top - this.SquareHeight;
                        break;
                }

                bounds.Width = this.SquareWidth;
                bounds.Height = this.SquareHeight;
                this.SquareBounds[index] = bounds;
            }
        }

        /// <summary>
        /// Updates the bounds for every house when the board size is calculated.
        /// </summary>
        protected virtual void UpdateHouseBounds()
        {
            this.RedHouseBounds = new Rect(this.SquareBounds[41].X, this.SquareBounds[0].Y, this.SquareWidth * 6, this.SquareHeight * 6);
            this.RedHouseWhiteBounds = new Rect(this.RedHouseBounds.X + this.ColoredHouseWidth, this.RedHouseBounds.Y + this.ColoredHouseHeight, this.RedHouseBounds.Width - 2 * this.ColoredHouseWidth, this.RedHouseBounds.Height - 2 * this.ColoredHouseHeight);
            this.YellowHouseBounds = new Rect(this.SquareBounds[8].X, this.SquareBounds[0].Y, this.SquareWidth * 6, this.SquareHeight * 6);
            this.YellowHouseWhiteBounds = new Rect(this.YellowHouseBounds.X + this.ColoredHouseWidth, this.YellowHouseBounds.Y + this.ColoredHouseHeight, this.YellowHouseBounds.Width - 2 * this.ColoredHouseWidth, this.YellowHouseBounds.Height - 2 * this.ColoredHouseHeight);
            this.GreenHouseBounds = new Rect(this.YellowHouseBounds.X, this.SquareBounds[21].Y, this.SquareWidth * 6, this.SquareHeight * 6);
            this.GreenHouseWhiteBounds = new Rect(this.GreenHouseBounds.X + this.ColoredHouseWidth, this.GreenHouseBounds.Y + this.ColoredHouseHeight, this.GreenHouseBounds.Width - 2 * this.ColoredHouseWidth, this.GreenHouseBounds.Height - 2 * this.ColoredHouseHeight);
            this.BlueHouseBounds = new Rect(this.RedHouseBounds.X, this.GreenHouseBounds.Y, this.SquareWidth * 6, this.SquareHeight * 6);
            this.BlueHouseWhiteBounds = new Rect(this.BlueHouseBounds.X + this.ColoredHouseWidth, this.BlueHouseBounds.Y + this.ColoredHouseHeight, this.BlueHouseBounds.Width - 2 * this.ColoredHouseWidth, this.BlueHouseBounds.Height - 2 * this.ColoredHouseHeight);
        }

        /// <summary>
        /// Updates the bounds for every goal when the board size is calculated.
        /// </summary>
        protected virtual void UpdateGoalBounds()
        {            
            this.RedGoalBounds = new Rect(this.SquareBounds[51].X, this.SquareBounds[46].Y, this.SquareWidth * 1.5f, this.SquareHeight * 3);
            this.YellowGoalBounds = new Rect(this.SquareBounds[51].X, this.SquareBounds[46].Y, this.SquareWidth * 3, this.SquareHeight * 1.5f);
            this.GreenGoalBounds = new Rect(this.SquareBounds[56].X + this.SquareWidth * 0.5f, this.SquareBounds[13].Y, this.SquareWidth * 1.5f, this.SquareHeight * 3);
            this.BlueGoalBounds = new Rect(this.SquareBounds[33].X, this.SquareBounds[74].Y + this.SquareHeight * 0.5f, this.SquareWidth * 3, this.SquareHeight * 1.5f);
        }

        #endregion  // Calculation methods

        #endregion  // Methods

    }
}
