using System;
using System.Collections.Generic;
using System.Text;
using System.Threading;
using System.ComponentModel;
using Lokeiv.Games.Ludo.Model;
using Lokeiv.Games.Dice;

namespace Lokeiv.Games.Ludo
{

    public delegate void DieRolledEventHandler(object engine, ActionEventArgs e);
    public delegate void PieceMovedEventHandler(object engine, ActionEventArgs e);
    public delegate void PlayerFinishedEventHandler(object engine, ActionEventArgs e);
    public delegate void GameOverEventHandler(object engine, ActionEventArgs e);

    public class Engine 
    {
        private NumericOneToSixDie _numericOneToSixDie;
        private Board _board;
        private MoveablePiece _lastMovablePiece;
        private Response _lastResponse;
        private bool _lastRollWasSix = false;
        private List<Player> _players;
        private int _numberOfPlayers;
        private int _currentPlayer;
        private GameOptions _options;
        private BackgroundWorker _worker = null;


        #region Public events

        /// <summary>
        /// Occures when the die has been rolled.
        /// </summary>
        public event DieRolledEventHandler DieRolled;

        /// <summary>
        /// Occures when a piece has been moved.
        /// </summary>
        public event PieceMovedEventHandler PieceMoved;

        /// <summary>
        /// Occures when a player has finished.
        /// </summary>
        public event PlayerFinishedEventHandler PlayerFinished;

        /// <summary>
        /// Occures when all players have finished.
        /// </summary>
        public event GameOverEventHandler GameOver;

        /// <summary>
        /// Fires the <see cref="DieRolled" /> event.
        /// </summary>
        /// <param name="engine"><see cref="Engine"/></param>
        /// <param name="e"><see cref="ActionEventArgs"/></param>
        protected void OnDieRolled(object engine, ActionEventArgs e)
        {
            if (DieRolled != null)
            {
                DieRolled(engine, e);
            }
        }

        /// <summary>
        /// Fires the <see cref="PieceMoved" /> event.
        /// </summary>
        /// <param name="engine"><see cref="Engine"/></param>
        /// <param name="e"><see cref="ActionEventArgs"/></param>
        protected void OnPieceMoved(object engine, ActionEventArgs e)
        {
            if (PieceMoved != null)
            {
                PieceMoved(engine, e);
            }
        }

        /// <summary>
        /// Fires the <see cref="PlayerFinished" /> event.
        /// </summary>
        /// <param name="engine"><see cref="Engine"/></param>
        /// <param name="e"><see cref="ActionEventArgs"/></param>
        protected void OnPlayerFinished(object engine, ActionEventArgs e)
        {
            if (PlayerFinished != null)
            {
                PlayerFinished(engine, e);
            }
        }

        /// <summary>
        /// Fires the <see cref="GameOver" /> event.
        /// </summary>
        /// <param name="engine"><see cref="Engine"/></param>
        /// <param name="e"><see cref="ActionEventArgs"/></param>
        protected void OnGameOver(object engine, ActionEventArgs e)
        {
            if (GameOver != null)
            {
                GameOver(engine, e);
            }
        }

        #endregion

        #region Public properties

        /// <summary>
        /// The current game options used for the game this engine runs.
        /// </summary>
        public GameOptions Options
        {
            get { return _options; }
            set { _options = value; }
        }

        public Die Die
        {
            get { return this._numericOneToSixDie; }
        }

        public Board Board
        {
            get { return this._board; }
        }

        public List<Player> Players
        {
            get { return this._players; }
        }

        #endregion

        #region Constructors

        /// <summary>
        /// Constructor creating a new game engine for the given players and options.
        /// </summary>
        /// <param name="players">List of players that participates in the game</param>
        /// <param name="options">Game options to use for the current game</param>
        public Engine(List<Player> players, GameOptions options)
        {            
            this._players = players;
            this._numberOfPlayers = this._players.Count;
            this._currentPlayer = 0;
            this._numericOneToSixDie = new NumericOneToSixDie();
            this._board = new Board(players);
            this._lastResponse = new Response(this._numericOneToSixDie.Value, this._players[this._currentPlayer], Action.RollDie);

            // Initialize BackgroundWorker for computer players
            this._worker = new BackgroundWorker();
            this._worker.DoWork += new DoWorkEventHandler(Worker_DoWork);
            this._worker.ProgressChanged += new ProgressChangedEventHandler(Worker_ProgressChanged);
            this._worker.WorkerSupportsCancellation = true;
            this._worker.WorkerReportsProgress = true;
        }

        #endregion

        public void StartGame()
        {
            bool computerPlayers = false;

            foreach(Player player in this._players) 
            {
                if (player.Type == PlayerType.Computer)
                {
                    computerPlayers = true;
                }
            }

            if (computerPlayers)
            {
                // There are computer players in this game, so start the background worker running the computer AI
                this._worker.RunWorkerAsync();
            }
        }

        public void EndGame()
        {
            if (this._worker.IsBusy)
            {
                this._worker.CancelAsync();
            }
        }

        void Worker_ProgressChanged(object sender, ProgressChangedEventArgs e)
        {
            
        }

        #region Worker thread methods

        /// <summary>
        /// Runs computer AI
        /// </summary>
        private void Worker_DoWork(object sender, DoWorkEventArgs e)
        {
            BackgroundWorker worker = sender as BackgroundWorker;

            while (!e.Cancel)
            {
                if (this.IsComputerPlayer())
                {
                    // Do action for the current computer player
                    if (this._lastResponse.NextAction == Action.RollDie)
                    {
                        // Roll die
                        this.RollDie();
                    }
                    else if (this._lastResponse.NextAction == Action.MovePiece)
                    {
                        // Move piece
                        //this.MovePiece(this._lastResponse.MovablePieces);
                    }
                }

                if (this.AreAllFinished())
                {
                    e.Cancel = true;
                }

                if (worker.CancellationPending)
                {
                    e.Cancel = true;
                }

                Thread.Sleep(500);
            }
        }

        /// <summary>
        /// Returns true if all players are finished.
        /// </summary>
        public bool AreAllFinished()
        {
            foreach (Player player in this._players)
            {
                if (!player.HasWon)
                {
                    return false;
                }
            }

            return true;
        }

        /// <summary>
        /// Returns true if the current player is a computer player.
        /// </summary>        
        private bool IsComputerPlayer()
        {
            return (this._players[this._currentPlayer].Type == PlayerType.Computer);
        }

        /// <summary>
        /// Moves a pice and returns a response with what's to happen next.
        /// </summary>
        /// <param name="movablePiece">Piece to move</param>
        /// <returns><see cref="Response">What's to happen next</see></returns>
        public Response MovePiece(MoveablePiece movablePiece)
        {
            if (this._lastResponse == null || this._lastResponse.NextAction != Action.MovePiece)
            {
                return null;  // A piece should not be moved now
            }

            Response response = new Response();
            List<MoveablePiece> moveablePieces = this._board.MovePiece(movablePiece);

            // Save new position of moved pieces
            response.MovablePieces.Add(movablePiece);

            if(this._board.HasFinished(movablePiece.Piece.Color))
            {
                this._players[this._currentPlayer - 1].SetWon(true);
                response.PlayerWon = this._players[this._currentPlayer - 1];
            }

            // Pass on to the next player
            this.NextPlayersTurn();

            response.PlayersTurn = this._players[this._currentPlayer - 1];
            response.NextAction = Action.RollDie;
            this._lastResponse = response;

            // Raise event to update UI
            this.OnPieceMoved(this, new ActionEventArgs(response));

            return response;
        }

        /// <summary>
        /// Rolls the die and returns a response with what's to happen next.
        /// </summary>
        /// <returns><see cref="Response">What's to happen next</see></returns>
        public Response RollDie()
        {
            if (this._lastResponse == null || this._lastResponse.NextAction != Action.RollDie)
            {
                return null;  // The die should not be rolled now
            }

            Response response = new Response();
            response.DieValue = this._numericOneToSixDie.Roll();            

            if ((int)this._numericOneToSixDie.Value.Value == 6)
            {
                this._lastRollWasSix = true;
            }
            else
            {
                this._lastRollWasSix = false;
            }
      
            response.MovablePieces = this._board.GetPossibleMoves(this._numericOneToSixDie, this._players[this._currentPlayer].Color);

            if (response.MovablePieces.Count == 0)
            {
                // No possible moves, pass on to the next player
                this.NextPlayersTurn();                
                response.NextAction = Action.RollDie;
            }
            else
            {
                // Possible moves
                response.NextAction = Action.MovePiece;
            }
            
            response.PlayersTurn = this._players[this._currentPlayer - 1];
            this._lastResponse = response;

            // Raise event to update UI
            this.OnDieRolled(this, new ActionEventArgs(response));  

            return response;
        }

        /// <summary>
        /// Finds which player is next.
        /// </summary>
        private void NextPlayersTurn()
        {
            if (!this._lastRollWasSix)
            {
                bool passOn = true;
                int index = 0;

                while (passOn)
                {
                    this._currentPlayer++;

                    if (this._currentPlayer > this._numberOfPlayers)
                    {
                        this._currentPlayer = 0;
                    }

                    if (!this._players[this._currentPlayer - 1].HasWon)
                    {
                        // It's this players turn
                        passOn = false;
                    }

                    index++;

                    if (index >= 4)
                    {
                        // All players have finished, no players can keep on
                        passOn = false;
                    }
                }
            }
        }

        #endregion  // Worker thread methods

    }
}
