using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Text;
using System.Windows.Forms;
using Lokeiv.Rendering;
using Lokeiv.Games.Ludo;
using Lokeiv.Games.Ludo.Model;

namespace Lokeiv.Games.Ludo.Forms.Client
{
    public partial class GameGlass : Form, IGame
    {
        private NewGameForm _newGameForm = null;
        private Ludo _ludo;
        private Engine _engine;

        #region Aero Glass Handling

        private DesktopWindowManagerApi.MARGINS m_glassMargins;
        private System.Drawing.Region m_blurRegion;

        protected override void WndProc(ref Message msg)
        {
            base.WndProc(ref msg); // let the normal winproc process it

            const int WM_DWMCOMPOSITIONCHANGED = 0x031E;
            const int WM_NCHITTEST = 0x84;
            const int HTCLIENT = 0x01;

            switch (msg.Msg)
            {
                case WM_NCHITTEST:
                    if (HTCLIENT == msg.Result.ToInt32())
                    {
                        // it's inside the client area

                        // Parse the WM_NCHITTEST message parameters
                        // get the mouse pointer coordinates (in screen coordinates)
                        System.Drawing.Point p = new System.Drawing.Point();
                        p.X = (msg.LParam.ToInt32() & 0xFFFF);// low order word
                        p.Y = (msg.LParam.ToInt32() >> 16); // hi order word

                        // convert screen coordinates to client area coordinates
                        p = PointToClient(p);

                        // if it's on glass, then convert it from an HTCLIENT
                        // message to an HTCAPTION message and let Windows handle it from then on
                        if (PointIsOnGlass(p))
                            msg.Result = new IntPtr(2);
                    }
                    break;

                case WM_DWMCOMPOSITIONCHANGED:
                    if (!DesktopWindowManagerApi.DwmIsCompositionEnabled())
                    {                        
                        m_glassMargins = null;
                        if (m_blurRegion != null)
                        {
                            m_blurRegion.Dispose();
                            m_blurRegion = null;
                        }
                    }
                    break;
            }
        }

        private void Game_Paint(object sender, PaintEventArgs e)
        {
            if (DesktopWindowManagerApi.DwmIsCompositionEnabled())
            {
                e.Graphics.FillRectangle(System.Drawing.Brushes.Black, this.ClientRectangle);
            }
        }

        private void ResetDwmBlurBehind()
        {
            if (DesktopWindowManagerApi.DwmIsCompositionEnabled())
            {
                DesktopWindowManagerApi.DWM_BLURBEHIND bbhOff = new DesktopWindowManagerApi.DWM_BLURBEHIND();
                bbhOff.dwFlags = DesktopWindowManagerApi.DWM_BLURBEHIND.DWM_BB_ENABLE | DesktopWindowManagerApi.DWM_BLURBEHIND.DWM_BB_BLURREGION;
                bbhOff.fEnable = false;
                bbhOff.hRegionBlur = IntPtr.Zero;
                DesktopWindowManagerApi.DwmEnableBlurBehindWindow(this.Handle, bbhOff);
            }
        }

        private bool PointIsOnGlass(System.Drawing.Point p)
        {
            // test for region or entire client area
            // or if upper window glass and inside it.
            // not perfect, but you get the idea
            return m_glassMargins != null &&
                (m_glassMargins.cyTopHeight <= 0 ||
                 m_glassMargins.cyTopHeight > p.Y);
        }

        private void InitializeGlassEffect()
        {
            this.ResetDwmBlurBehind();

            m_glassMargins = new DesktopWindowManagerApi.MARGINS(-1, 0, 0, 0);            

            if (DesktopWindowManagerApi.DwmIsCompositionEnabled()) DesktopWindowManagerApi.DwmExtendFrameIntoClientArea(this.Handle, m_glassMargins);

            this.Invalidate();
        }

        #endregion


        /// <summary>
        /// Property to manipulate the ludo game used in this form.
        /// Used by test cases.
        /// </summary>
        public Ludo Ludo
        {
            get { return this._ludo; }
            set { this._ludo = value; }
        }

        public GameGlass()
        {
            InitializeComponent();
            this._newGameForm = new NewGameForm();

            // Enable glass effect on Vista
            this.Paint += new PaintEventHandler(Game_Paint);  // Paints Aero Glass effect
            this.InitializeGlassEffect();
            this.board1.AttachToForm(this);
            this.SizeBoard();
        }

        private void StartNewGame()
        {            
            switch(this._newGameForm.ShowDialog(this))
            {
                case DialogResult.OK:
                    List<Player> players = this._newGameForm.Players;
                    this._engine = new Engine(players, new GameOptions());
                    //this._engine.ActionHappened += new Engine.ActionHappenedEventHandler(Engine_ActionHappened);
                    this.PrepareGame(players);

                    //this.gameEngine.RunWorkerAsync(players);
                    //this.board1.StartNewGame(numberOfPlayers);
                    break;
                default:
                    break;
            }
        }

        private void Engine_ActionHappened(object engine, EventArgs e)
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public void PrepareGame(List<Player> players)
        {
            // Order list of players by color
            Player.SortPlayersList(players);
            this._ludo = new Ludo(players, this.die1.DieModel);
            this.board1.SetGameModel(this._ludo.Board, this);
            this.InitializePlayers();
            this._ludo.StartGame();
            this.gameInstructionsLabel.Text = String.Format(Properties.Resources.InstructionRollDie, this._ludo.LastResponse.PlayersTurn.Name);
        }
        

        /// <summary>
        /// Updates players in GUI.
        /// </summary>
        private void InitializePlayers()
        {            
            int defaultImageIndex = 9;
            this.redPlayerCaption.ImageIndex = defaultImageIndex;
            this.redPlayerCaption.Text = "Red player";
            this.yellowPlayerCaption.ImageIndex = defaultImageIndex;
            this.yellowPlayerCaption.Text = "Yellow player";
            this.greenPlayerCaption.ImageIndex = defaultImageIndex;
            this.greenPlayerCaption.Text = "Green player";
            this.bluePlayerCaption.ImageIndex = defaultImageIndex;
            this.bluePlayerCaption.Text = "Blue player";

            // Set the right image and caption
            foreach (Player player in this._ludo.Players)
            {
                switch (player.Color)
                {
                    case Color.Red:
                        this.redPlayerCaption.Text = player.Name;

                        if (player.Type == PlayerType.Computer)
                            this.redPlayerCaption.ImageIndex = 5;
                        else
                            this.redPlayerCaption.ImageIndex = 0;
                        break;
                    case Color.Yellow:
                        this.yellowPlayerCaption.Text = player.Name;

                        if (player.Type == PlayerType.Computer)
                            this.yellowPlayerCaption.ImageIndex = 6;
                        else
                            this.yellowPlayerCaption.ImageIndex = 1;
                        break;
                    case Color.Green:
                        this.greenPlayerCaption.Text = player.Name;

                        if (player.Type == PlayerType.Computer)
                            this.greenPlayerCaption.ImageIndex = 7;
                        else
                            this.greenPlayerCaption.ImageIndex = 2;
                        break;
                    case Color.Blue:
                        this.bluePlayerCaption.Text = player.Name;

                        if (player.Type == PlayerType.Computer)
                            this.bluePlayerCaption.ImageIndex = 8;
                        else
                            this.bluePlayerCaption.ImageIndex = 3;
                        break;
                }
            }            
        }

        private void UpdatePlayersTurn(Player player)
        {
            this.redPlayerCaption.Enabled = false;
            this.yellowPlayerCaption.Enabled = false;
            this.greenPlayerCaption.Enabled = false;
            this.bluePlayerCaption.Enabled = false;
            
            if (player.Name == this.redPlayerCaption.Text)
            {
                this.redPlayerCaption.Enabled = true;
            }
            else if (player.Name == this.yellowPlayerCaption.Text)
            {
                this.yellowPlayerCaption.Enabled = true;
            }
            else if (player.Name == this.greenPlayerCaption.Text)
            {
                this.greenPlayerCaption.Enabled = true;
            }
            else if (player.Name == this.bluePlayerCaption.Text)
            {
                this.bluePlayerCaption.Enabled = true;
            }
        }

        public void MovePiece(MoveablePiece moveablePiece)
        {
            Response response = this._ludo.MovePiece(moveablePiece);

            if (response != null)
            {
                this.UpdatePlayersTurn(response.PlayersTurn);
                string text = "";

                if (response.PlayerWon != null)
                {
                    text += String.Format(Properties.Resources.InstructionHasFinished + (char)13, response.PlayerWon.Name);
                }

                if (response.NextAction == Action.RollDie)
                {
                    text += String.Format(Properties.Resources.InstructionRollDie, response.PlayersTurn.Name);
                }

                this.gameInstructionsLabel.Text = text;                
            }

            this.Invalidate();
        }

        private void SizeBoard()
        {            
            this.board1.Bounds = new System.Drawing.Rectangle(0, 24, this.Width - 180, this.Height - 24 - 37);
        }

        #region Event handlers
        private void newToolStripMenuItem_Click(object sender, EventArgs e)
        {
            this.StartNewGame();
        }

        private void Game_Shown(object sender, EventArgs e)
        {
            this.StartNewGame();            
        }

        private void Game_FormClosed(object sender, FormClosedEventArgs e)
        {
            Application.Exit();
        }

        private void Game_Resize(object sender, EventArgs e)
        {
            this.SizeBoard();
        }

        private void Game_SizeChanged(object sender, EventArgs e)
        {
            this.SizeBoard();
        }

        #endregion

        private void gameEngine_DoWork(object sender, DoWorkEventArgs e)
        {
            List<Player> players = (List<Player>)e.Argument;
            Ludo ludo = new Ludo(players, this.die1.DieModel);
            this.board1.SetGameModel(ludo.Board, this);
            ludo.StartGame();

            while(!ludo.areAllFinished())
            {
                //gameEngine.ReportProgress(0, ludo.MovePiece(new MoveablePiece()));
            }
        }

        private void gameEngine_ProgressChanged(object sender, ProgressChangedEventArgs e)
        {
            Response response = e.UserState as Response;
            this.die1.DieValue = response.DieValue;
            this.Invalidate();
        }

        private void die1_Click(object sender, EventArgs e)
        {
            Response response = this._ludo.RollDie();

            if (response != null)
            {
                this.UpdatePlayersTurn(response.PlayersTurn);
                string text = "";

                if (response.NextAction == Action.MovePiece)
                {
                    text += String.Format(Properties.Resources.InstructionMovePiece, response.PlayersTurn.Name);
                }
                else if (response.NextAction == Action.RollDie)
                {
                    text += String.Format(Properties.Resources.InstructionRollDie, response.PlayersTurn.Name);
                }

                this.gameInstructionsLabel.Text = text;
            }
        }

    }
}