using System;
using System.Collections.Generic;
using System.Text;
using Lokeiv.Games.Dice;

namespace Lokeiv.Games.Ludo.Model
{
    public class Board
    {
        private List<Square> _squares = new List<Square>(92);
        private List<Piece> _pieces = new List<Piece>();
        private List<MoveablePiece> _moveablePieces = new List<MoveablePiece>();

        #region Properties
        public List<Square> Squares
        {
            get { return this._squares; }
        }

        public List<Piece> Pieces
        {
            get { return this._pieces; }
        }

        public List<MoveablePiece> MoveablePieces
        {
            get { return this._moveablePieces; }
        }
        #endregion

        #region Constructors
        
        /// <summary>
        /// Initializes a <see cref="Board"/> for the given players.
        /// </summary>
        /// <param name="numberOfPlayers">Number of players</param>
        public Board(List<Player> players)
        {
            // Define square objects for this board
            this.InitializeBoard();

            // Create and place pieces for each player
            this.CreatePieces(players);
            this.SetUpPieces(players);
        }

        #endregion

        #region Private Methods
        /// <summary>
        /// Creates Square objects and assigns Color and NextSquare etc.
        /// </summary>
        private void InitializeBoard()
        {
            for (int i = 0; i < 92; i++)
                this._squares.Add(new Square(Color.White, i));

            for (int i = this._squares.Count - 2; i >= 0; i--)
                this._squares[i].SetNextSquare(this._squares[i + 1]);                    

            // Yellow house
            for (int i = 76; i <= 79; i++)
            {
                this._squares[i].SetColor(Color.Yellow);
                this._squares[i].SetHouse(true);
                this._squares[i].SetNextSquare(this._squares[3]);
            }

            // Green house
            for (int i = 80; i <= 83; i++)
            {
                this._squares[i].SetColor(Color.Green);
                this._squares[i].SetHouse(true);
                this._squares[i].SetNextSquare(this._squares[16]);
            }

            // Blue house
            for (int i = 84; i <= 87; i++)
            {
                this._squares[i].SetColor(Color.Blue);
                this._squares[i].SetHouse(true);
                this._squares[i].SetNextSquare(this._squares[29]);
            }

            // Red house
            for (int i = 88; i <= 91; i++)
            {
                this._squares[i].SetColor(Color.Red);
                this._squares[i].SetHouse(true);
                this._squares[i].SetNextSquare(this._squares[42]);
            }

            // Yellow goal
            this._squares[57].SetColor(Color.Yellow);
            this._squares[57].SetGoal(true);
            this._squares[57].SetNextSquare(null);

            // Green goal
            this._squares[63].SetColor(Color.Green);
            this._squares[63].SetGoal(true);
            this._squares[63].SetNextSquare(null);

            // Blue goal
            this._squares[69].SetColor(Color.Blue);
            this._squares[69].SetGoal(true);
            this._squares[69].SetNextSquare(null);

            // Red goal
            this._squares[75].SetColor(Color.Red);
            this._squares[75].SetGoal(true);
            this._squares[75].SetNextSquare(null);

            // Yellow crossing
            this._squares[3].SetColor(Color.Yellow);
            this._squares[3].SetCrossing(true);
            this._squares[3].SetAlternateSquare(this._squares[52]);
            this._squares[3].SetNextSquare(this._squares[4]);

            // Green crossing
            this._squares[16].SetColor(Color.Green);
            this._squares[16].SetCrossing(true);
            this._squares[16].SetAlternateSquare(this._squares[58]);
            this._squares[16].SetNextSquare(this._squares[17]);

            // Blue crossing
            this._squares[29].SetColor(Color.Blue);
            this._squares[29].SetCrossing(true);
            this._squares[29].SetAlternateSquare(this._squares[64]);
            this._squares[29].SetNextSquare(this._squares[30]);

            // Red crossing
            this._squares[42].SetColor(Color.Red);
            this._squares[42].SetCrossing(true);
            this._squares[42].SetAlternateSquare(this._squares[70]);
            this._squares[42].SetNextSquare(this._squares[43]);

            // Yellow path to goal
            for (int i = 52; i <= 56; i++)
                this._squares[i].SetColor(Color.Yellow);
            for (int i = 56; i >= 52; i--)
                this._squares[i].SetNextSquare(this._squares[i + 1]);

            // Green path to goal
            for (int i = 58; i <= 62; i++)
                this._squares[i].SetColor(Color.Green);
            for (int i = 62; i >= 58; i--)
                this._squares[i].SetNextSquare(this._squares[i + 1]);

            // Blue path to goal
            for (int i = 64; i <= 68; i++)
                this._squares[i].SetColor(Color.Blue);
            for (int i = 68; i >= 64; i--)
                this._squares[i].SetNextSquare(this._squares[i + 1]);

            // Red path to goal
            for (int i = 70; i <= 74; i++)
                this._squares[i].SetColor(Color.Red);
            for (int i = 74; i >= 70; i--)
                this._squares[i].SetNextSquare(this._squares[i + 1]);

            // The last white square should point back to the first square
            this._squares[51].SetNextSquare(this._squares[0]);
        }
        
        /// <summary>
        /// Creates right number of pieces and assign Color.
        /// </summary>
        /// <param name="players">Players to create pieces for</param>
        private void CreatePieces(List<Player> players)
        {
            this._pieces.Clear();

            foreach (Player player in players)
            {
                for (int i = 0; i < 4; i++)
                {
                    this._pieces.Add(new Piece(player.Color));
                }
            }
        }

        /// <summary>
        /// Places the pieces at their start positions, based on number of players.
        /// </summary>
        /// <param name="players">Players to set up pieces for</param>
        private void SetUpPieces(List<Player> players)
        {
            int redIndex = 0;
            int yellowIndex = 0;
            int greenIndex = 0;
            int blueIndex = 0;

            foreach (Player player in players)
            {
                foreach (Piece piece in this._pieces)
                {
                    if (player.Color == piece.Color)
                    {
                        switch (player.Color)
                        {
                            case Color.Red:
                                piece.SetSquare(this._squares[88 + redIndex++]);
                                break;
                            case Color.Yellow:
                                piece.SetSquare(this._squares[76 + yellowIndex++]);
                                break;
                            case Color.Green:
                                piece.SetSquare(this._squares[80 + greenIndex++]);
                                break;
                            case Color.Blue:
                                piece.SetSquare(this._squares[84 + blueIndex++]);
                                break;
                        }
                    }
                }
            }
        }

        /// <summary>
        /// Returns the first free <see cref="Square"/> in a given house.
        /// </summary>
        /// <param name="playersColor"><see cref="Color"/> representing the player</param>
        /// <returns><see cref="Square"/> inside house</returns>
        private Square GetEmptySquareInHouse(Color playersColor)
        {
            switch (playersColor)
            {
                case Color.Red:
                    for (int i = 88; i <= 91; i++)
                        if (this._squares[i].Pieces.Count == 0)
                            return this._squares[i];                    
                    break;
                case Color.Yellow:
                    for (int i = 76; i <= 79; i++)
                        if (this._squares[i].Pieces.Count == 0)
                            return this._squares[i];
                    break;
                case Color.Green:
                    for (int i = 80; i <= 83; i++)
                        if (this._squares[i].Pieces.Count == 0)
                            return this._squares[i];
                    break;
                case Color.Blue:
                    for (int i = 84; i <= 87; i++)
                        if (this._squares[i].Pieces.Count == 0)
                            return this._squares[i];
                    break;
            }

            // We'll never reach this line
            return null;
        }

        /// <summary>
        /// Used from GetPossibleMoves to check if there is a Block on the path.
        /// </summary>
        /// <param name="piece"><see cref="Piece"/> to be moved</param>
        /// <param name="possibleMove">The current value of the possibleMove variable</param>
        /// <param name="squareOnPath">The current <see cref="Square"/> on the path being checked</param>
        /// <param name="lastSquareOnPath">If true, no block leads to new moveable piece. If false, 
        /// we should check next square on path.</param>
        /// <returns></returns>
        private Boolean CheckForBlock(Piece piece, Boolean possibleMove, ref Square squareOnPath, bool lastSquareOnPath)
        {
            if (squareOnPath.NextSquare.Block != null)  // Possible Block
            {
                if (squareOnPath.NextSquare.Block.Color != piece.Color)
                {
                    possibleMove = false;
                }
                else
                {
                    if (lastSquareOnPath)
                    {
                        this._moveablePieces.Add(new MoveablePiece(piece, squareOnPath.NextSquare));
                    }
                    else
                    {
                        squareOnPath = squareOnPath.NextSquare;
                    }
                }
            }
            else  // No Block
            {
                if (lastSquareOnPath)
                {
                    this._moveablePieces.Add(new MoveablePiece(piece, squareOnPath.NextSquare));
                }
                else
                {
                    squareOnPath = squareOnPath.NextSquare;
                }
            }

            return possibleMove;
        }

        /// <summary>
        /// Turns in a Piece when an opponent has stopped on the same <see cref="Square"/>.
        /// </summary>
        /// <param name="piece"><see cref="Piece"/> to be turned in</param>
        private void TurnInPiece(Piece piece)
        {
            this.MovePiece(piece, this.GetEmptySquareInHouse(piece.Color));
            piece.SetReadyForFinish(false);
        }

        /// <summary>
        /// When a <see cref="Piece"/> has finished being moved, this method is called to check if there
        /// are other <see cref="Piece"/>s on the <see cref="Square"/>. If it is, those <see cref="Piece"/>s
        /// are turned in.
        /// </summary>
        /// <param name="piece"><see cref="Piece"/> that has been moved</param>
        private void CheckAndTurnInPiece(Piece piece)
        {
            if (piece.Square.Pieces.Count >= 2)
            {
                // There is at least one other Piece on this Square
                for (int index = 0; index < piece.Square.Pieces.Count; index++)
                {
                    Piece pieceToTurnIn = piece.Square.Pieces[index];

                    // Turn in all pieces with other Color than the Piece being moved
                    if (pieceToTurnIn.Color != piece.Color)
                    {
                        this.TurnInPiece(pieceToTurnIn);
                        index--;
                    }

                }
            }
        }

        /// <summary>
        /// When a <see cref="Piece"/> has finished being moved, this method is called to check if there
        /// are other <see cref="Piece"/>s with the same <see cref="Color"/> on the <see cref="Square"/>.
        /// If it is, a <see cref="Block"/> is formed.
        /// </summary>
        /// <param name="piece"><see cref="Piece"/> that has been moved</param>
        private void CheckAndFormBlock(Piece piece)
        {
            if (piece.Square.Pieces.Count >= 2)
            {
                // There is at least one other Piece on this Square
                foreach (Piece pieceOnSquare in piece.Square.Pieces)
                {
                    if (pieceOnSquare != piece)  // Make sure we don't form a Block because we compare the color of the same Piece
                        if (pieceOnSquare.Color == piece.Color)
                            piece.Square.SetBlock(new Block(piece.Color));
                }
            }
        }

        /// <summary>
        /// When a <see cref="Piece"/> has moved past the crossing outside it's house, 
        /// the <see cref="Piece.IsReadyForFinish"/> property should be set to true, to make it
        /// move towards goal next time it comes to the crossing.
        /// </summary>
        /// <param name="piece"><see cref="Piece"/> that has been moved</param>
        private void CheckAndSetReadyForFinish(Piece piece)
        {
            if (piece.Color == Color.Red && piece.Square.Number == 42)
                piece.SetReadyForFinish(true);

            if (piece.Color == Color.Yellow && piece.Square.Number == 3)
                piece.SetReadyForFinish(true);

            if (piece.Color == Color.Green && piece.Square.Number == 16)
                piece.SetReadyForFinish(true);

            if (piece.Color == Color.Blue && piece.Square.Number == 29)
                piece.SetReadyForFinish(true);
        }

        /// <summary>
        /// Before a <see cref="Piece"/> is moved any existing block on the current <see cref="Square"/> 
        /// is removed if there is just one <see cref="Piece"/> left on the <see cref="Square"/> after 
        /// the move.
        /// </summary>
        /// <param name="piece"><see cref="Piece"/> being moved</param>
        private void ChechAndRemoveBlock(Piece piece)
        {
            if (piece.Square.Block != null)
                if (piece.Square.Pieces.Count == 2)
                    piece.Square.SetBlock(null);
        }

        #endregion

        #region Internal Methods
        /// <summary>
        /// Moves a <see cref="Piece"/> to a <see cref="Square"/>.
        /// </summary>        
        /// <param name="piece"><see cref="Piece"/> to move</param>
        /// <param name="square"><see cref="Square"/> to move to</param>
        /// <returns>List of <see cref="MovablePiece">position for all pieces</see> after move</returns>
        internal List<MoveablePiece> MovePiece(Piece piece, Square square)
        {
            if (piece == null)
                throw new ArgumentNullException("piece");
            if (square == null)
                throw new ArgumentNullException("square");

            this.CheckAndSetReadyForFinish(piece);
            this.ChechAndRemoveBlock(piece);

            // Perform movement by updating piece and square objects involved            
            piece.Square.Pieces.Remove(piece);
            piece.SetSquare(square);

            this.CheckAndFormBlock(piece);
            this.CheckAndTurnInPiece(piece);

            return this.GetAllPiecesPosition();
        }
        #endregion

        #region Public Methods
        /// <summary>
        /// Moves a <see cref="Piece"/> to a <see cref="Square"/> as passed in
        /// the given <see cref="MovablePiece"/>.
        /// </summary>
        /// <param name="movable"><see cref="MovablePiece"/> representing the move to make</param>
        /// <returns>List of <see cref="MovablePiece">position for all pieces</see> after move</returns>
        public List<MoveablePiece> MovePiece(MoveablePiece movable)
        {
            if (movable == null)
                throw new ArgumentNullException("movable");

            return this.MovePiece(movable.Piece, movable.Square);
        }


        /// <summary>
        /// Returns the <see cref="Square"/> with the given index.
        /// Throws Exception if index is out of range (0 - 92).
        /// </summary>
        /// <param name="index">Index of <see cref="Square"/> to get</param>
        /// <returns><see cref="Square"/></returns>
        public Square GetSquareAt(int index)
        {
            if (index >= 92 || index < 0)
                throw new Exception(String.Format(Resources.Errors.IndexBetween, "0", "91"));

            return this._squares[index];
        }

        /// <summary>
        /// Returns true if the given <see cref="Player"/> has finished.
        /// </summary>
        /// <param name="playersColor"><see cref="Color"/> of the player</param>
        /// <returns>True if <see cref="Player"/> has finished, false if not.</returns>
        public Boolean HasFinished(Color playersColor)
        {
            switch (playersColor)
            {
                case Color.Red:
                    return (this._squares[75].Pieces.Count == 4);
                case Color.Yellow:
                    return (this._squares[57].Pieces.Count == 4);
                case Color.Green:
                    return (this._squares[63].Pieces.Count == 4);
                case Color.Blue:
                    return (this._squares[69].Pieces.Count == 4);
            }

            return false;
        }

        /// <summary>
        /// Returns a list of <see cref="MovablePiece">position for all pieces</see>.
        /// </summary>
        /// <returns>List of <see cref="MovablePiece"/></returns>
        public List<MoveablePiece> GetAllPiecesPosition()
        {
            List<MoveablePiece> boardState = new List<MoveablePiece>();

            foreach (Piece piece in this._pieces)
            {
                boardState.Add(new MoveablePiece(piece, piece.Square));
            }

            return boardState;
        }

        /// <summary>
        /// Gets a list of <see cref="MoveablePiece"/>s for all possible moves for the given <see cref="Player"/>.
        /// </summary>
        /// <param name="die">A rolled <see cref="Die"/> showing how many steps to move</param>
        /// <param name="playersColor"><see cref="Color"/> of the player who can move</param>
        /// <returns></returns>
        public List<MoveablePiece> GetPossibleMoves(Die die, Color playersColor)
        {
            if (die.GetType() != typeof(NumericOneToSixDie))
            {
                // Trying to use a die type that is not supported
                throw new ArgumentException(die.GetType().ToString());
            }

            int number = (int)die.Value.Value;

            // Use the member _moveablePieces so the client can get the list later too
            this._moveablePieces.Clear();

            foreach (Piece piece in this._pieces)
            {
                Boolean possibleMove = true;  // Will be set to false if a blocked path is found
                Square nextSquare = null;  // Position in a path

                if (piece.Color == playersColor)
                {
                    if (piece.Square.IsGoal)
                    {
                        // A Piece standing in Goal can't be moved
                    }
                    else if (piece.Square.IsHouse)
                    {
                        // Dies number must be 6 to move this Piece out of the house
                        if (number == 6)
                        {
                            // If there is a Block outside the house, it must be of the current Players Color
                            if (piece.Square.NextSquare.Block != null && piece.Square.NextSquare.Block.Color != piece.Color)
                                possibleMove = false;
                            else
                                this._moveablePieces.Add(new MoveablePiece(piece, piece.Square.NextSquare));
                        }
                    }
                    else  // The Piece is on a regular Square
                    {
                        nextSquare = piece.Square;

                        for (int i = 1; i <= number; i++)  // For each eye on the Die
                        {
                            if (piece.IsReadyForFinish)  // Have been around the Board
                            {
                                if (nextSquare.IsCrossing && nextSquare.Color == piece.Color)  // Is Crosssing
                                {
                                    if (i == number)  // Last square on the path
                                        this._moveablePieces.Add(new MoveablePiece(piece, nextSquare.AlternateSquare));
                                    else
                                        nextSquare = nextSquare.AlternateSquare;
                                }
                                else  // Isn't Crossing
                                {
                                    if (i == number)  // Last square on the path
                                    {
                                        if (nextSquare.NextSquare != null)  // Path didn't lead past the Goal
                                            possibleMove = CheckForBlock(piece, possibleMove, ref nextSquare, true);
                                        else  // Path lead past the Goal
                                            possibleMove = false;
                                    }
                                    else  // Not last Square on the path
                                    {
                                        if (nextSquare.NextSquare == null)  // Path lead past the Goal
                                            possibleMove = false;
                                        else  // There's a NextSquare on this path
                                            possibleMove = CheckForBlock(piece, possibleMove, ref nextSquare, false);
                                    }
                                }
                            }  // if (piece.IsReadyForFinish)
                            else  // Piece has just come out of its house
                            {
                                if (i == number)  // Last square on the path
                                {
                                    possibleMove = CheckForBlock(piece, possibleMove, ref nextSquare, true);
                                }
                                else  // Not last square on the path
                                {
                                    possibleMove = CheckForBlock(piece, possibleMove, ref nextSquare, false);
                                }
                            }
                        }  // For each eye on the Die
                    }  // The Piece is on a regular Square
                }  // if (piece.Color == playersColor)
            }  // foreach Piece

            return this._moveablePieces;
        }

        #endregion

    }
}
