using System;
using System.Collections.Generic;
using System.Text;
using System.Drawing;

namespace Lokeiv.Rendering
{
    public class StringRendering
    {
        /// <summary>
        /// Gets the best text size to fill the given bounds.
        /// </summary>
        /// <param name="font">Font the text will be drawn with</param>
        /// <param name="text">The text string to be drawn</param>
        /// <param name="bounds">The bounds to fit the text inside</param>
        /// <returns>Text size in pixels</returns>
        public static float CalculateBestTextSize(Font font, string text, RectangleF bounds)
        {
            // Todo: The constant 0.6f could be switched by font family
            // Todo: The calculation assumes each character is equal in size, which is not true for most modern fonts
            float textSizeForOneCharacter = (bounds.Width > bounds.Height ? bounds.Height * 0.6f : bounds.Width * 0.6f);
            return textSizeForOneCharacter / (text.Length > 0 ? text.Length : 1);
        }

        /// <summary>
        /// Gets the best text size to fill the given bounds.
        /// </summary>
        /// <param name="font">Font the text will be drawn with</param>
        /// <param name="text">The text string to be drawn</param>
        /// <param name="bounds">The bounds to fit the text inside</param>
        /// <returns>Text size in pixels</returns>
        public static float CalculateBestTextSize(Font font, string text, Rectangle bounds)
        {
            return CalculateBestTextSize(font, text, new RectangleF((float)bounds.X, (float)bounds.Y, (float)bounds.Width, (float)bounds.Height));
        }

        /// <summary>
        /// Returns a font with the size set to fit the text inside the bounds.
        /// </summary>
        /// <param name="prototype">The existing <see cref="System.Drawing.Font"/> from which to create the new <see cref="System.Drawing.Font"/></param>
        /// <param name="text">The text string to be drawn</param>
        /// <param name="bounds">The bounds to fit the text inside</param>
        /// <returns>A new font to use when drawing the text</returns>
        public static Font GetFontToFitStringInsideBounds(Font prototype, string text, RectangleF bounds)
        {
            return new Font(prototype.Name, CalculateBestTextSize(prototype, text, bounds), prototype.Style, GraphicsUnit.Pixel);
        }

        /// <summary>
        /// Returns a font with the size set to fit the text inside the bounds.
        /// </summary>
        /// <param name="prototype">The existing <see cref="System.Drawing.Font"/> from which to create the new <see cref="System.Drawing.Font"/></param>
        /// <param name="text">The text string to be drawn</param>
        /// <param name="bounds">The bounds to fit the text inside</param>
        /// <returns>A new font to use when drawing the text</returns>
        public static Font GetFontToFitStringInsideBounds(Font prototype, string text, Rectangle bounds)
        {
            return GetFontToFitStringInsideBounds(prototype, text, new RectangleF((float)bounds.X, (float)bounds.Y, (float)bounds.Width, (float)bounds.Height));
        }

        /// <summary>
        /// Draws text centered inside the given bounds.
        /// </summary>
        /// <param name="graphics"><see cref="System.Drawing.Graphics"/> used to draw the string</param>
        /// <param name="brush"><see cref="System.Drawing.Brush"/> used to draw the string</param>
        /// <param name="prototype">The existing <see cref="System.Drawing.Font"/> from which the <see cref="System.DrawingFont"/> used to draw the string should be based on</param>
        /// <param name="text">The string to be drawn</param>
        /// <param name="bounds">The bounds to center the text inside</param>
        public static void DrawTextCentered(Graphics graphics, Brush brush, Font prototype, string text, RectangleF bounds)
        {
            Font font = GetFontToFitStringInsideBounds(prototype, text, bounds);
            RectangleF textBounds = new RectangleF();
            textBounds.Size = graphics.MeasureString(text, font, bounds.Size);
            textBounds.X = bounds.X + (bounds.Width / 2.0f - textBounds.Width / 2.0f);
            textBounds.Y = bounds.Y + (bounds.Height / 2.0f - textBounds.Height / 2.0f);
            graphics.DrawString(text, font, brush, textBounds);
        }

        /// <summary>
        /// Draws text centered inside the given bounds.
        /// </summary>
        /// <param name="graphics"><see cref="System.Drawing.Graphics"/> used to draw the string</param>
        /// <param name="brush"><see cref="System.Drawing.Brush"/> used to draw the string</param>
        /// <param name="prototype">The existing <see cref="System.Drawing.Font"/> from which the <see cref="System.DrawingFont"/> used to draw the string should be based on</param>
        /// <param name="text">The string to be drawn</param>
        /// <param name="bounds">The bounds to center the text inside</param>
        public static void DrawTextCentered(Graphics graphics, Brush brush, Font prototype, string text, Rectangle bounds)
        {
            DrawTextCentered(graphics, brush, prototype, text, new RectangleF((float)bounds.X, (float)bounds.Y, (float)bounds.Width, (float)bounds.Height));
        }
    }
}
